/* $Id: BalTree.c 354 2004-07-17 06:28:14Z cher $ */

/* Copyright (C) 1996-2004 Alexander Chernov <cher@ispras.ru> */
/* Created: Thu Nov  7 20:32:12 1996 by cher (Alexander Chernov) */

/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 */

#define __REUSE__ 1

/* internal include directives */
#include "lconfig.h"
#include "reuse/config.h"
#include "embed_version.h"

#include <stdio.h>

/* We don't need exported prototype for bt_balance */
#define BT_NO_BALANCE_PROTO
#include <reuse/BalTree.h>

typedef struct tree_t
{
  struct tree_t *left, *right;
  int           height;         /* Height of this tree */
} tree_t;

tree_t *
bt_balance(tree_t *p)
{
  tree_t *p1, *p2;
  int    diff;

  if (p->left == NULL && p->right == NULL)
    {
      p->height = 1;
      return p;
    }
  if (p->right == NULL)
    {
      if (p->left->height > 1)
        {
          p1 = p->left;
          if (p1->left != NULL)
            {
              /* Sigle LL rotate */
              p->left = p1->right;
              p1->right = p;
              if (p->left != NULL)
                p->height = 2;
              else
                p->height = 1;
              p1->height = p->height + 1;
              return p1;
            }
          /* Double LR rotate */
          p2 = p1->right;     /*   X  <- p         Z   */
          p2->right = p;      /*  /               / \  */
          p2->left = p1;      /* Y    <- p1 =>   Y   X */
          p1->right = NULL;   /*  \                    */
          p->left = NULL;     /*   Z  <- p2            */
          p->height = 1;
          p1->height = 1;
          p2->height = 2;
          return p2;
        }
      else
        {
          p->height = 2;
          return p;
        }
    }
  if (p->left == NULL)
    {
      if (p->right->height > 1)
        {
          p1 = p->right;
          if (p1->right != NULL)
            {
              /* Single RR rotate */
              p->right = p1->left;
              p1->left = p;
              if (p->right != NULL)
                p->height = 2;
              else
                p->height = 1;
              p1->height = p->height + 1;
              return p1;
            }
          /* Double RL rotate */
          p2 = p1->left;
          p2->left = p;
          p2->right = p1;
          p1->left = NULL;
          p->right = NULL;
          p->height = 1;
          p1->height = 1;
          p2->height = 2;
          return p2;
        }
      else
        {
          p->height = 2;
          return p;
        }
    }
  /* Difference of heights between left and right subtrees */
  diff = p->left->height - p->right->height;
  if (diff == 1 || diff == 0)
    {
      p->height = p->left->height + 1;
      return p;
    }
  if (diff == -1)
    {
      p->height = p->right->height + 1;
      return p;
    }
  /* Test for left disbalance */
  if (diff == 2)
    {
      /* Balancing required */
      p1 = p->left;
      p->height = p1->height + 1;
      if (p1->left == NULL || p1->left->height == p->right->height)
        {
          /* Double LR rotate */
          p2 = p1->right;
          p1->right = p2->left;
          p2->left = p1;
          p->left = p2->right;
          p2->right = p;
          p1->height--;
          p2->height++;
          p->height -= 2;
          return p2;
        }
      /* Single LL rotate */
      p->left = p1->right;
      p1->right = p;
      p->height -= 2;
      return p1;
    }
  /* Test for right disbalance */
  if (diff == -2)
    {
      /* Balancing required */
      p1 = p->right;
      p->height = p1->height + 1;
      if (p1->right == NULL || p1->right->height == p->left->height)
        {
          /* Double RL rotate */
          p2 = p1->left;
          p1->left = p2->right;
          p2->right = p1;
          p->right = p2->left;
          p2->left = p;
          p1->height--;
          p2->height++;
          p->height -= 2;
          return p2;
        }
      /* Single RR rotate */
      p->right = p1->left;
      p1->left = p;
      p->height -= 2;
      return p1;
    }
  return p;
}

#if REUSE_COMPILE_CVSID - 0 != 0
static const char cvsid[] __attribute__((unused)) =
"$Id: BalTree.c 354 2004-07-17 06:28:14Z cher $";
#endif /* REUSE_COMPILE_CVSID */

/*
 * Local variables:
 *  compile-command: "make"
 *  c-font-lock-extra-types: ("\\sw+_t" "FILE" "R_DLL")
 * End:
 */
