/* $Id: DynArray.c 354 2004-07-17 06:28:14Z cher $ */

/* Copyright (C) 1995-2004 Alexander Chernov <cher@ispras.ru> */
/* Ich, Doktor Josef Grosch, Informatiker, Sept. 1987 */
/* Alexander Chernov, December 1995 */

/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 */

#define __REUSE__ 1

/* internal include directives */
#include "lconfig.h"
#include "reuse/config.h"
#include "embed_version.h"

/* reuse include directives */
#include <reuse/integral.h>
#include <reuse/Defs.h>
#include <reuse/ReuseDefs.h>
#include <reuse/DynArray.h>
#include <reuse/General.h>
#include <reuse/Memory.h>

#include <stdio.h>
#include <assert.h>
#include <string.h>
#include <stdlib.h>

/* INVARIANT ElmtCount * AlignedSize (ElmtSize) % sizeof (long) == 0 */

  static rulong_t
AlignedSize (rulong_t ElmtSize)
{
  register unsigned long Align;
  
  if (ElmtSize >= yyMaxAlign)
    {
      Align = yyMaxAlign;
    }
  else
    {
      Align = Exp2 (Log2 (ElmtSize + ElmtSize - 2));
    }
  return ElmtSize + Align - 1 - (ElmtSize - 1) % Align;
}

  void
MakeArray(char **ArrayPtr, rulong_t *ElmtCount,rulong_t ElmtSize)
{
  ElmtSize = AlignedSize (ElmtSize);
  switch (ElmtSize % 4)
    {
    case 0:
      break;
    case 2:
      if (* ElmtCount & 1) (* ElmtCount) ++;
      break;
    case 1:
    case 3:
      * ElmtCount += sizeof (long) - 1 - (* ElmtCount - 1) % sizeof (long);
      break;
    }

  * ArrayPtr = Alloc(* ElmtCount * ElmtSize);

  if (* ArrayPtr == NULL)
    {
      (void) fprintf (stderr, "MakeArray: out of memory\n");
      abort();
    }
}

  void
ExtendArray(char **ArrayPtr,rulong_t *ElmtCount,rulong_t ElmtSize)
{
  char          *NewPtr;
  rulong_t        mem_size;

  ElmtSize = AlignedSize (ElmtSize);
  mem_size = *ElmtCount * ElmtSize;

  NewPtr = Alloc(mem_size * 2);
  memcpy(NewPtr, *ArrayPtr, mem_size);
  Free(mem_size, *ArrayPtr);

  assert(NewPtr != NULL);
  *ElmtCount *= 2;
  *ArrayPtr = NewPtr;
}

  void
ReleaseArray(char **ArrayPtr,rulong_t *ElmtCount,rulong_t ElmtSize)
{
  ElmtSize = AlignedSize (ElmtSize);
  /*  free(* ArrayPtr); */
  Free(*ElmtCount * ElmtSize, *ArrayPtr);
}

  void
dynInitModule (void)
{
}

  void
dynCloseModule(void)
{
}

  void
dynStatistics(FILE *out)
{
}

#if REUSE_COMPILE_CVSID - 0 != 0
static char const rcsid [] __attribute__((unused)) =
"$Id: DynArray.c 354 2004-07-17 06:28:14Z cher $";
#endif /* REUSE_COMPILE_CVSID */

/**
 * Local variables:
 *  compile-command: "make"
 *  c-font-lock-extra-types: ("FILE" "\\sw+_t" "Bool")
 *  tags-file-name: "../TAGS"
 * End:
 */
