/* $Id: GenOut.c 354 2004-07-17 06:28:14Z cher $ */

/* Copyright (C) 1995-2004 Alexander Chernov <cher@ispras.ru> */
/* Created: <95/11/09 16:22:56 cher> */

/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 */

#define __REUSE__ 1

/* internal include directives */
#include "lconfig.h"
#include "reuse/config.h"
#include "embed_version.h"

/* reuse include directives */
#include <reuse/GenOut.h>

#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include <stdarg.h>

static char strTempBuf[2048];

  static void
go_file_char(tFileOut *pFileOut,int c)
{
  fputc(c,pFileOut->file);
}

  static void
go_file_string(tFileOut *pFileOut,char *str)
{
  fputs(str,pFileOut->file);
}

  static void
go_file_format(tFileOut *pFileOut,char *format,...)
{
  va_list vaArgs;

  va_start(vaArgs,format);
  vfprintf(pFileOut->file,format,vaArgs);
  va_end(vaArgs);
}

  static void
go_file_vformat(tFileOut *pFileOut,char *format,va_list args)
{
  vfprintf(pFileOut->file,format,args);
}

static tGenOutVT go_file_virtual =
{
  (tCharOutFunc)    go_file_char,
  (tStringOutFunc)  go_file_string,
  (tFormatOutFunc)  go_file_format,
  (tVFormatOutFunc) go_file_vformat
};

  tFileOut
goFileOut(FILE *f)
{
  tFileOut r;

/* Fill up virtual function table */
  r.GenOutPart.Virtual = &go_file_virtual;
  /* Fill up data */
  r.file = f;
  return r;
}

  static void
go_fix_char(tFixedOut *pOut,int c)
{
  if(pOut->Used >= pOut->Size-1)
    return;
  pOut->String[pOut->Used++] = c;
  pOut->String[pOut->Used] = 0;
}

  static void
go_fix_string(tFixedOut *pOut,char *str)
{
  int iLen;
  
  assert(str != NULL);
  if(pOut->Used >= pOut->Size-1)
    return;
  iLen = strlen(str);
  if(iLen + pOut->Used > pOut->Size-1)
    {
      iLen = pOut->Size-1 - pOut->Used;
    }
  memcpy(pOut->String+pOut->Used,str,iLen);
  pOut->Used += iLen;
  pOut->String[pOut->Used] = '\0';
}

  static void
go_fix_format(tFixedOut *pOut,char *format,...)
{
  va_list vaArgs;

  va_start(vaArgs,format);
  vsprintf(strTempBuf,format,vaArgs);
  va_end(vaArgs);
  go_fix_string(pOut,strTempBuf);
}

  static void
go_fix_vformat(tFixedOut *pOut,char *format,va_list args)
{
  vsprintf(strTempBuf,format,args);
  go_fix_string(pOut,strTempBuf);
}

static tGenOutVT go_fix_virtual =
{
  (tCharOutFunc)    go_fix_char,
  (tStringOutFunc)  go_fix_string,
  (tFormatOutFunc)  go_fix_format,
  (tVFormatOutFunc) go_fix_vformat
};

  tFixedOut
goFixedOut(char *buf,ruint_t size)
{
  tFixedOut r;
  
  /* Fill up virtual function table */
  r.GenOutPart.Virtual = &go_fix_virtual;
  /* Fill up data */
  r.String = buf;
  r.Size = size;
  r.Used = 0;
  
  return r;
}

  void
go_printf(tGenOut *pOut,char *format,...)
{
  va_list vaArgs;
  
  va_start(vaArgs,format);
  (*pOut->Virtual->VFormatOut)(pOut,format,vaArgs);
  va_end(vaArgs);
}

  void
goInitModule(void)
{
}

  void
goCloseModule(void)
{
}

  void
goStatistics(FILE *_unused)
{
}

#if REUSE_COMPILE_CVSID - 0 != 0
static char const rcsid[]=
"$Id: GenOut.c 354 2004-07-17 06:28:14Z cher $";
#endif /* REUSE_COMPILE_CVSID */

/**
 * Local variables:
 *  compile-command: "make"
 *  tags-file-name: "../TAGS"
 *  c-font-lock-extra-types: ("\\sw+_t" "FILE" "va_list" "tGenOut" "tFixedOut" "tCharOutFunc" "tStringOutFunc" "tFormatOutFunc" "tVFormatOutFunc" "tGenOutVT" "tFileOut")
 * End:
 */
