/* $Id: Idents.c 354 2004-07-17 06:28:14Z cher $ */

/* Copyright (C) 1995-2004 Alexander Chernov <cher@ispras.ru> */
/* Ich, Doktor Josef Grosch, Informatiker, Sept. 1987 */
/* Alexander Chernov, December 1995 */

/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 */

#define __REUSE__ 1

/* internal include directives */
#include "lconfig.h"
#include "reuse/config.h"
#include "embed_version.h"

/* reuse include directives */
#include <reuse/Defs.h>
#include <reuse/Idents.h>
#include <reuse/FlexString.h>

#include <string.h>
#include <assert.h>
#include <stdlib.h>

#define IDENT_CLUSTER           64
#define HashTableSize           256
#define cNoIdent                0

static Bool initialized;

typedef struct {
   tStringRef   String;
   Cardinal     Length;
   tIdent       Collision;
} IdentTableEntry;

static IdentTableEntry *TablePtr        = NULL;
static unsigned long    IdentTableSize  = 0;
static tIdent           IdentCount      = 1;
static tIdent           HashTable [HashTableSize];
static tIdent           _NoIdent        = 1;

  static void
do_init(void)
{
  int i;

  initialized = True;
  
  for (i = 0; i < HashTableSize; i ++) {
    HashTable[i] = cNoIdent;
  }
      
  IdentCount     = 1;
  IdentTableSize = 0;
  TablePtr = NULL;

  _NoIdent = MakeIdent ("", 0);
  assert(_NoIdent == 1);
}


  tIdent
MakeIdent(char * string, Cardinal length)
{
  register      Cardinal        HashTableIndex;
  register      tIdent          CurIdent;
  register      IdentTableEntry * TablePtrReg;

  if (!initialized) do_init();

  HashTableIndex = length == 0 ? 0 :
    ((Cardinal) * string + (Cardinal) string [length - 1] * 11
     + length * 26) & (HashTableSize - 1);

  CurIdent = HashTable [HashTableIndex];

  while (CurIdent != cNoIdent)
    {
      TablePtrReg = & TablePtr [CurIdent];
      if (TablePtrReg->Length == length &&
          IsEqualSt (TablePtrReg->String, string))
        return CurIdent;                        /* found */
      CurIdent = TablePtrReg->Collision;
    }

  if (IdentCount >= IdentTableSize)
    {
      TablePtr = (IdentTableEntry*)
        realloc(TablePtr, (IdentTableSize += IDENT_CLUSTER)
                * sizeof(IdentTableEntry));
    }

  TablePtrReg = & TablePtr [IdentCount];
  TablePtrReg->String   = PutString (string, length);
  TablePtrReg->Length   = length;
  TablePtrReg->Collision        = HashTable [HashTableIndex];
  HashTable [HashTableIndex] = IdentCount;
  return IdentCount++;
}

  tIdent
idMake(char *str)
{
  tIdent res = MakeIdent(str, (Cardinal) strlen(str));
  return res;
}

  void
GetString(tIdent ident, char * string)
{
  if (!initialized) do_init();

  StGetString (TablePtr [ident].String, string);
}

  tStringRef
GetStringRef(tIdent ident)
{
  if (!initialized) do_init();

  return TablePtr [ident].String;
}

  tIdent
MaxIdent(void)
{
  if (!initialized) do_init();

  return IdentCount;
}

  void
WriteIdent(FILE * file, tIdent ident)
{
  char  string [256];

  if (!initialized) do_init();
  
  GetString (ident, string);
  (void) fputs (string, file);
}

  void
WriteIdents(FILE *file)
{
  Cardinal      i;

  if (!initialized) do_init();
  
  for (i = 1; i <= IdentCount; i ++)
    {
      fprintf (file, "%5d ", i);
      WriteIdent (file, i);
      fputc ('\n', file);
    }
}

  void
WriteHashTable(FILE *file)
{
  tIdent        CurIdent;
  Cardinal      i;
  Cardinal      Count;

  if (!initialized) do_init();
   
  for (i = 0; i < HashTableSize; i ++)
    {
      fprintf (file, "%5d", i);
      
      Count = 0;
      CurIdent = HashTable [i];
      while (CurIdent != cNoIdent)
        {
          Count ++;
          CurIdent = TablePtr [CurIdent].Collision;
        }
      fprintf (file, "%5d", Count);

      CurIdent = HashTable [i];
      while (CurIdent != cNoIdent)
        {
          fputc (' ', file);
          WriteIdent (file, CurIdent);
          CurIdent = TablePtr [CurIdent].Collision;
        }
      fputc ('\n', file);
    }
  fprintf (file, "\nIdents = %5d\n", IdentCount);
}

  void
idClear(void)
{
  int i;

  if (!initialized) do_init();

  for (i = 0; i < HashTableSize; i++)
    HashTable[i] = cNoIdent;

  IdentCount = 0;
  _NoIdent = MakeIdent ("", 0);
  assert(_NoIdent == 1);
}
    
  void
InitIdents(void)
{
  if (!initialized) do_init();
}

  void
idCloseModule(void)
{
  if (initialized)
    {
      initialized = 0;
      free(TablePtr);
    }
}

  void
idStatistics(FILE *file)
{
  int allocated_entries = 0;
  int index;

  if (!initialized) do_init();

  for (index = 0; index < HashTableSize; index++)
    {
      if (HashTable[index] != cNoIdent)
        allocated_entries++;
    }

  fprintf(file,
          "Idents module statistics:\n"
          "Hash Table Size:       %d\n"
          "Allocated Entries:     %d\n"
          "Identifiers Number:    %d\n"
          "Allocated Identifiers: %ld\n",
          HashTableSize,
          allocated_entries,
          IdentCount,
          IdentTableSize);
}

  char*
idGetString(tIdent ident)
{
  if (!initialized) do_init();
  return stmString(TablePtr[ident].String);
}

  char*
idDupString(tIdent ident)
{
  if (!initialized) do_init();
  return strdup(stmString(TablePtr[ident].String));
}

  tIdent
idReadIdent(FILE *f)
{
  fsRead(reuse_fsTempBuf, f);
  return (tIdent) idMakeFS(reuse_fsTempBuf);
}

  tIdent
idGetIdent(FILE *f)
{
  fsGet(reuse_fsTempBuf, f);
  return idMakeFS(reuse_fsTempBuf);
}

#if REUSE_COMPILE_CVSID - 0 != 0
static char const rcsid [] __attribute__((unused)) =
"$Id: Idents.c 354 2004-07-17 06:28:14Z cher $";
#endif /* REUSE_COMPILE_CVSID */

/* === Do not add stuff after this line === */

/**
 * Local variables:
 *  compile-command: "make"
 *  c-font-lock-extra-types: ("tIdent" "FILE" "\\sw+_t" "Bool" "Cardinal" "tStringRef" "IdentTableEntry")
 *  tags-file-name: "../TAGS"
 * End:
 */
