/* $Id: Printf.c 354 2004-07-17 06:28:14Z cher $ */

/* Copyright (C) 1995-2004 Alexander Chernov <cher@ispras.ru> */
/* Created: <95/12/04 21:12:36 cher> */

/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 */

#define __REUSE__ 1

/* internal include directives */
#include "lconfig.h"
#include "reuse/config.h"
#include "embed_version.h"

/* reuse include directives */
#include <reuse/ReuseDefs.h>
#include <reuse/Printf.h>
#include <reuse/FlexString.h>
#include <reuse/Idents.h>

#include <assert.h>

/* free letters that can be redefined by user */
static int f_free (tGenOut *g, char *c, int i, va_list va_args)
{
  return 1;
}

/* disabled letters that cannot be used as format specifiers */
static int f_dis (tGenOut *g, char *c, int i, va_list va_args)
{
  return 1;
}

/* stands for letters that are modifiers of format conversion.
   These letters are: 'L', 'h', 'l' */
static int f_part (tGenOut *g, char *c, int i, va_list va_args)
{
  return 1;
}

static int I_conv (tGenOut *g, char *c, int i, va_list va_args);

/* standard printf conversion letters (handled in-line) */
static int f_std (tGenOut *g, char *c, int i, va_list va_args)
{
  return 1;
}

#define TOTAL_LETTERS 64

#define NO_MODIFIER   0
#define L_MODIFIER    1
#define h_MODIFIER    2
#define l_MODIFIER    3
#define w_MODIFIER    4

static tUserPrintfFunc letter_table[TOTAL_LETTERS] = 
{
/*'@'      'A'      'B'      'C'      'D'      'E'      'F'      'G'*/
  f_dis,   f_free,  f_free,  f_free,  f_free,  f_std,   f_free,  f_std,  
/*'H'      'I'      'J'      'K'      'L'      'M'      'N'      'O'*/
  f_free,  I_conv,  f_free,  f_free,  f_part,  f_free,  f_free,  f_free, 
/*'P'      'Q'      'R'      'S'      'T'      'U'      'V'      'W'*/
  f_free,  f_free,  f_free,  f_free,  f_free,  f_free,  f_free,  f_free, 
/*'X'      'Y'      'Z'      '['      '\'      ']'      '^'      '_'*/
  f_std,   f_free,  f_free,  f_dis,   f_dis,   f_dis,   f_dis,   f_dis,  
/*'`'      'a'      'b'      'c'      'd'      'e'      'f'      'g'*/
  f_dis,   f_free,  f_free,  f_std,   f_std,   f_std,   f_std,   f_std,  
/*'h'      'i'      'j'      'k'      'l'      'm'      'n'      'o'*/
  f_part,  f_std,   f_free,  f_free,  f_part,  f_free,  f_std,   f_std,  
/*'p'      'q'      'r'      's'      't'      'u'      'v'      'w'*/
  f_std,   f_free,  f_free,  f_std,   f_free,  f_std,   f_free,  f_part, 
/*'x'      'y'      'z'      '{'      '|'      '}'      '~'      '?'*/
  f_std,   f_free,  f_free,  f_dis,   f_dis,   f_dis,   f_dis,   f_dis 
};

static int letter_indexes[TOTAL_LETTERS] =
{
/*'@'      'A'      'B'      'C'      'D'      'E'      'F'      'G'*/
  0,       0,       0,       0,       0,       0,       0,       0,
/*'H'      'I'      'J'      'K'      'L'      'M'      'N'      'O'*/
  0,       0,       0,       0,       L_MODIFIER,0,     0,      0,
/*'P'      'Q'      'R'      'S'      'T'      'U'      'V'      'W'*/
  0,       0,       0,       0,       0,       0,       0,       0,
/*'X'      'Y'      'Z'      '['      '\'      ']'      '^'      '_'*/
  0,       0,       0,       0,       0,       0,       0,       0,
/*'`'      'a'      'b'      'c'      'd'      'e'      'f'      'g'*/
  0,       0,       0,       0,       0,       0,       0,       0,
/*'h'      'i'      'j'      'k'      'l'      'm'      'n'      'o'*/
  h_MODIFIER, 0,    0,       0,       l_MODIFIER, 0,    0,       0,    
/*'p'      'q'      'r'      's'      't'      'u'      'v'      'w'*/
  0,       0,       0,       0,       0,       0,       0,       w_MODIFIER,
/*'x'      'y'      'z'      '{'      '|'      '}'      '~'      '?'*/
  0,       0,       0,       0,       0,       0,       0,       0
};

/* modifier indexes -- 0 - no, 1 - L, 2 - h, 3 - l, 4 - w */

static size_t _s_size[5] =
{ sizeof(char*), 0, 0, 0, sizeof(char*)};

static size_t _i_size[5] =
{ sizeof(int), 0, sizeof(int), sizeof(long), 0};

static size_t _n_size[5] =
{ sizeof(int*), 0, sizeof(short*), sizeof(long*), 0};

static size_t _f_size[5] =
{ sizeof(double), sizeof(long double), 0, 0, 0};

static size_t _c_size[5] =
{ sizeof(int), 0, 0, 0, sizeof(int)};

static size_t _p_size[5] =
{ sizeof(void*), 0, 0, 0, 0};

static size_t *letter_sizes[TOTAL_LETTERS] =
{
/*'@'      'A'      'B'      'C'      'D'      'E'      'F'      'G'*/
  NULL,    NULL,    NULL,    NULL,    NULL,    _f_size, NULL,    _f_size,
/*'H'      'I'      'J'      'K'      'L'      'M'      'N'      'O'*/
  NULL,    NULL,    NULL,    NULL,    NULL,    NULL,    NULL,    NULL,
/*'P'      'Q'      'R'      'S'      'T'      'U'      'V'      'W'*/
  NULL,    NULL,    NULL,    NULL,    NULL,    NULL,    NULL,    NULL,
/*'X'      'Y'      'Z'      '['      '\'      ']'      '^'      '_'*/
  _i_size, NULL,    NULL,    NULL,    NULL,    NULL,    NULL,    NULL,
/*'`'      'a'      'b'      'c'      'd'      'e'      'f'      'g'*/
  NULL,    NULL,    NULL,    _c_size, _i_size, _f_size, _f_size, _f_size,
/*'h'      'i'      'j'      'k'      'l'      'm'      'n'      'o'*/
  NULL,    _i_size, NULL,    NULL,    NULL,    NULL,    _n_size, _i_size,
/*'p'      'q'      'r'      's'      't'      'u'      'v'      'w'*/
  _p_size, NULL,    NULL,    _s_size, NULL,    _i_size, NULL,    NULL,
/*'x'      'y'      'z'      '{'      '|'      '}'      '~'      '?'*/
  _i_size, NULL,    NULL,    NULL,    NULL,    NULL,    NULL,    NULL
};

  Bool
Printf_Register (int letter, tUserPrintfFunc user_func)
{
  if (letter < '@' || letter > '~')
    return False;
  
  if (letter_table[letter] != f_free)
    return False;
  
  letter_table[letter] = user_func;
  return True;
}

static void modifier_error() REUSE_UNUSED;

  static void
modifier_error (void)
{
  fputs ("VGPrintf: Invalid modifier combination in format specification!\n",
         stderr);
  abort();
}

  void
VGPrintf (tGenOut *gen_out, char *format_str, va_list va_args)
{
  char *str;
  tFString format_buf;
  int modifier_index;
  int letter_index;
  int asterisk_num;
  
#ifdef REUSE_DEBUG
  assert(gen_out != NULL);
  assert(format_str != NULL);
#endif

  fsInit (format_buf);
  
  for (str = format_str; *str != '\0'; str++)
    {
      if (*str != '%')
        {
          go_putc (gen_out, *str);
          continue;
        }
      
      str++;
      if (*str == '%')
        {
          go_putc (gen_out, '%');
          continue;
        }
      
      fsAdd (format_buf, '%');
      asterisk_num = 0;
      modifier_index = 0;
      
      for(; *str != '\0' &&
              (*str < 'A' || *str > 'z' ||
               letter_table[*str - '@'] == f_part);
         str++)
        {
          if (*str == '*')
            asterisk_num++;
          
          if(*str >= 'A' && *str <= 'z')
            {
#ifdef REUSE_DEBUG
              if (modifier_index != 0)
                modifier_error ();
#endif
              
              modifier_index = letter_indexes[*str - '@'];
            }
          
          fsAdd (format_buf, *str);
        }
      
      fsAdd (format_buf, *str);
      
      if (*str == '\0')
        {
          go_puts (gen_out, fsString (format_buf));
          break;
        }
      
      letter_index = *str - '@';
      
      if (letter_table[letter_index] == f_free
          || letter_table[letter_index] == f_dis)
        {
          go_puts (gen_out, fsString (format_buf));
        }
      else if (letter_table[letter_index] != f_std)
        {
          int fetched;
          int count;
          
          fetched = (*letter_table[letter_index]) (gen_out,
                                                   fsString (format_buf),
                                                   fsLength (format_buf),
                                                   va_args);
          assert (fetched >= 0);
          
          /* fetch ints */
          for (count = fetched / sizeof (int); count; count--)
            (void) va_arg (va_args, int);
          /* fetch chars */
          for (count = fetched % sizeof (int); count; count--)
            (void) va_arg (va_args, int);
            
        }
      else /* if (letter_table[letter_index == f_std]) */
        {
          int fetched;
          int count;
          size_t *sizes;
          
          fetched = asterisk_num * sizeof (int);
          sizes = letter_sizes[letter_index];
          assert (sizes != NULL);
          
          if (sizes[modifier_index] == 0)
            {
              fprintf (stderr,"VGPrintf: Invalid modifier in format: %s\n",
                       fsString (format_buf));
              abort ();
            }
          
          fetched += sizes[modifier_index];
          
          go_vprintf (gen_out, fsString (format_buf), va_args);
          assert (fetched >= 0);
          
          /* fetch ints */
          for (count = fetched / sizeof (int); count; count--)
            (void) va_arg (va_args, int);
          /* fetch chars */
          for (count = fetched % sizeof (int); count; count--)
            (void) va_arg (va_args, int);
        }
      
      fsClear (format_buf);
    }
  
  fsDestroy (format_buf);
}

  void
GPrintf (tGenOut *gen_out, char *format_str, ...)
{
  va_list va_args;
  
#ifdef REUSE_DEBUG
  assert(gen_out != NULL);
  assert(format_str != NULL);
#endif

  va_start (va_args, format_str);
  VGPrintf(gen_out, format_str, va_args);
  va_end (va_args);
}

/* prints an identifier */
  static int
I_conv (tGenOut *g, char *c, int i, va_list va_args)
{
  tIdent     string_id;
  
  c[i-1] = 's';  /* transform format to %s */
  string_id = va_arg (va_args, int);
  
  go_printf (g, c, idGetString (string_id));
  return sizeof (int);
}

  void
prfInitModule(void)
{
}

  void
prfCloseModule(void)
{
}

  void
prfStatistics(FILE * _unused)
{
}

#if REUSE_COMPILE_CVSID - 0 != 0
static char const rcsid[]=
"$Id: Printf.c 354 2004-07-17 06:28:14Z cher $";
#endif /* REUSE_COMPILE_CVSID */

/**
 * Local variables:
 *  compile-command: "make"
 *  c-font-lock-extra-types: ("tIdent" "FILE" "\\sw+_t" "tGenOut" "va_list" "Bool" "tUserPrintfFunc")
 *  tags-file-name: "TAGS"
 * End:
 */
