/* $Id: Sets.c 354 2004-07-17 06:28:14Z cher $ */

/* Copyright (C) 1995-2004 Alexander Chernov <cher@ispras.ru> */
/* Ich, Doktor Josef Grosch, Informatiker, Sept. 1987 */
/* Alexander Chernov, Nov. 1995 */

/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 */

#define __REUSE__ 1

/* internal include directives */
#include "lconfig.h"
#include "reuse/config.h"
#include "embed_version.h"

/* reuse include directives */
#include <reuse/integral.h>

#include <reuse/Defs.h>
#include <reuse/Sets.h>
#include <reuse/General.h>

#include <assert.h>

#define BitsPerByte             8
#define BytesPerBitset          BitsPerBitset / BitsPerByte
#define ONE                     0x80000000
#define NoCard                  -1

static void setDefaultItemPrint(rushort_t number,tGenOut *pOut);

  void
MakeSet(tSet *Set,rushort_t MaxSize)
{
  setInit(Set,MaxSize,setDefaultItemPrint);
}

  void
setInit(tSet *Set,rushort_t MaxSize,tsetElemPrint pfPrint)
{
  rulong_t ElmtCount;
  
  ElmtCount =
    (MaxSize + BitsPerBitset - (MaxSize & MaskBitsPerBitset))
      >> LdBitsPerBitset;
  Set->BitsetPtr = (BITSET*) malloc(ElmtCount*sizeof(BITSET));
  assert(Set->BitsetPtr != NULL);
  Set->MaxElmt = MaxSize;
  Set->LastBitset = (rushort_t) (ElmtCount - 1);
#ifdef REUSE_DEBUG
  if ((rulong_t) Set->LastBitset != ElmtCount - 1) {
    fprintf(stderr, "_setInit: Integer overflow\n");
    abort();
  }
#endif /* REUSE_DEBUG */
  Set->PrintFunc = pfPrint;
  AssignEmpty (Set);
}

  void
ReleaseSet (tSet *Set)
{
  rulong_t ElmtCount;

  ElmtCount = Set->LastBitset + 1;
  free(Set->BitsetPtr);
}

  void
Union(tSet * Set1,tSet * Set2)
{
  rushort_t i = Set1->LastBitset + 1;
  BITSET *s1= Set1->BitsetPtr;
  BITSET *s2= Set2->BitsetPtr;

  do {* s1 ++ |= * s2 ++;} while (-- i);
  Set1->Card      = NoCard;
  Set1->FirstElmt = Min (Set1->FirstElmt, Set2->FirstElmt);
  Set1->LastElmt  = Max (Set1->LastElmt , Set2->LastElmt );
}

  void
Difference(tSet * Set1,tSet * Set2)
{
  rushort_t i = Set1->LastBitset + 1;
  BITSET *s1 = Set1->BitsetPtr;
  BITSET *s2 = Set2->BitsetPtr;

  do {* s1 ++ &= ~ * s2 ++;} while (-- i);
  Set1->Card = NoCard;
}

  void
Intersection(tSet *Set1,tSet *Set2)
{
  rushort_t i = Set1->LastBitset + 1;
  BITSET *s1 = Set1->BitsetPtr;
  BITSET *s2 = Set2->BitsetPtr;

  do {* s1 ++ &= * s2 ++;} while (-- i);
  Set1->Card      = NoCard;
  Set1->FirstElmt = Max (Set1->FirstElmt,Set2->FirstElmt);
  Set1->LastElmt  = Min (Set1->LastElmt ,Set2->LastElmt);
}

  void
SymDiff(tSet *Set1,tSet *Set2)
{
  rushort_t i = Set1->LastBitset + 1;
  BITSET *s1 = Set1->BitsetPtr;
  BITSET *s2 = Set2->BitsetPtr;

  do {* s1 ++ ^= * s2 ++;} while (-- i);
  Set1->Card = NoCard;
  Set1->FirstElmt = Min(Set1->FirstElmt,Set2->FirstElmt);
  Set1->LastElmt  = Max(Set1->LastElmt ,Set2->LastElmt);
}

  void
Complement(tSet *Set)
{
  rushort_t i = Set->LastBitset;
  BITSET *s1 = Set->BitsetPtr;

  while (i --) {* s1 = ~ * s1; s1 ++;}
  * s1 = ((long) ONE >> (Set->MaxElmt & MaskBitsPerBitset)) & ~ * s1;
  if (Set->Card != NoCard)
    Set->Card = (rushort_t) Set->MaxElmt + 1 - Set->Card;
  Set->FirstElmt = 0;
  Set->LastElmt  = Set->MaxElmt;
}

  void
Include(tSet * Set, rushort_t Elmt)
{
  Set->BitsetPtr[Elmt >> LdBitsPerBitset] |=
    (rulong_t) ONE >> (Elmt & MaskBitsPerBitset);
  Set->Card = NoCard;
  Set->FirstElmt = Min(Set->FirstElmt,Elmt);
  Set->LastElmt = Max(Set->LastElmt ,Elmt);
}

  void
Exclude(tSet * Set, rushort_t Elmt)
{
  Set->BitsetPtr [Elmt >> LdBitsPerBitset]
    &= ~ ((unsigned long) ONE >> (Elmt & MaskBitsPerBitset));
  Set->Card = NoCard;
  if(Elmt == Set->FirstElmt && Elmt < Set->MaxElmt)
    Set->FirstElmt ++;
  if(Elmt == Set->LastElmt && Elmt > 0)
    Set->LastElmt --;
}

  rushort_t
Card(tSet *Set)
{
  if (Set->Card == NoCard)
    {
      rushort_t i, n;
      rushort_t last = Set->LastElmt;

      n = 0;
      for (i = Set->FirstElmt; i <= last; i ++)
        {
          if (IsElement (i, Set)) n ++;
        }
      Set->Card = n;
    }
  return Set->Card;
}
    
  rushort_t
Minimum (tSet *Set)
{
  rushort_t i;
  rushort_t last = Set->LastElmt;

  for (i = Set->FirstElmt; i <= last; i ++)
    {
      if (IsElement (i, Set))
        {
          Set->FirstElmt = i;
          return i;
        }
    }
  return 0;
}
    
  rushort_t
Maximum (tSet *Set)
{
  rushort_t i;
  rushort_t first = Set->FirstElmt;

  for (i = Set->LastElmt; i >= first; i --)
    {
      if(IsElement (i,Set))
        {
          Set->LastElmt = i;
          return i;
        }
    }
  return 0;
}
    
  rushort_t
Extract(tSet *Set)
{
  rushort_t i = Minimum (Set);
  
  Exclude (Set, i);
  return i;
}

  Bool
IsSubset (tSet *Set1,tSet *Set2)
{
  rushort_t i = Set1->LastBitset + 1;
  BITSET *s1 = Set1->BitsetPtr;
  BITSET *s2 = Set2->BitsetPtr;

  do
    {
      if (* s1 ++ & ~ * s2 ++) return False;
    }
  while (-- i);
  return True;
}

  Bool
IsEqual (tSet *Set1,tSet *Set2)
{
  rushort_t i = Set1->LastBitset + 1;
  BITSET *s1 = Set1->BitsetPtr;
  BITSET *s2 = Set2->BitsetPtr;

  do
    {
      if (* s1 ++ != * s2 ++) return False;
    }
  while (-- i);
  return True;
}
    
  Bool
IsEmpty (tSet *Set)
{
  if(Set->FirstElmt <= Set->LastElmt)
    {
      rushort_t i  = Set->LastBitset + 1;
      BITSET * s1 = Set->BitsetPtr;

      do
        {
          if (* s1 ++ != 0) return False;
        }
      while (-- i);
    }
  return True;
}
    
  Bool
Forall (tSet *Set,Bool (*Proc)(rushort_t))
{
  rushort_t i;
  rushort_t last = Set->LastElmt;

  for (i = Set->FirstElmt; i <= last; i ++)
    {
      if(IsElement (i, Set) && ! (*Proc)(i)) return False;
    }
  return True;
}
    
  Bool
Exists(tSet *Set,Bool (*Proc)(rushort_t))
{
  rushort_t i;
  rushort_t last = Set->LastElmt;

  for (i = Set->FirstElmt; i <= last; i ++)
    {
      if (IsElement (i, Set) && (*Proc)(i)) return True;
    }
  return False;
}
    
  Bool
Exists1(tSet *Set,Bool (*Proc)(rushort_t))
{
  rushort_t i, n;
  rushort_t last = Set->LastElmt;

  n = 0;
  for (i = Set->FirstElmt; i <= last; i ++)
    {
      if (IsElement (i, Set) && (*Proc)(i)) n ++;
    }
  return n == 1;
}

  void
Assign (tSet *Set1,tSet *Set2)
{
  rushort_t i = Set1->LastBitset + 1;
  BITSET *s1 = Set1->BitsetPtr;
  BITSET *s2 = Set2->BitsetPtr;

  do
    {
      * s1 ++ = * s2 ++;
    }
  while (-- i);
  Set1->Card      = Set2->Card;
  Set1->FirstElmt = Set2->FirstElmt;
  Set1->LastElmt  = Set2->LastElmt;
}

  void
AssignElmt(tSet * Set, rushort_t Elmt)
{
  AssignEmpty (Set);
  Include (Set, Elmt);
  Set->Card      = 1;
  Set->FirstElmt = Elmt;
  Set->LastElmt  = Elmt;
}

  void
AssignEmpty(tSet *Set)
{
  rushort_t i = Set->LastBitset + 1;
  BITSET *s1 = Set->BitsetPtr;

  do
    {
      * s1 ++ = 0;
    } 
  while (-- i);
  Set->Card      = 0;
  Set->FirstElmt = Set->MaxElmt;
  Set->LastElmt  = 0;
}

  void
ForallDo (tSet *Set,void (*Proc)(rushort_t))
{
  rushort_t i;
  rushort_t last   = Set->LastElmt;

  for(i = Set->FirstElmt; i <= last; i ++)
    {
      if (IsElement (i, Set)) (*Proc) (i);
    }
}

  void
ReadSet (FILE *File,tSet *Set)
{
  long i, card = 0;

  while (fgetc (File) != '{');
  AssignEmpty (Set);
  for (;;) {
    if (fgetc (File) == '}') break;
    (void) fscanf (File, "%ld", & i);
    Include (Set, (unsigned short) i);
    card ++;
  }
  Set->Card = (unsigned short) card;
}

  static void
setDefaultItemPrint(rushort_t num,tGenOut *pOut)
{
  go_printf(pOut,"%u",num);
}

  void
WriteSet(FILE *f,tSet *Set)
{
  tFileOut Out;
  Out = goFileOut(f);
  setWrite((tGenOut*) &Out,Set,"{"," ","}");
}

  void
setWrite(tGenOut *pOut,tSet *Set,char *Left,char *Center,char *Right)
{
  rushort_t i;
  rushort_t last   = Set->LastElmt;
  Bool IsFirst = 1;

  go_puts(pOut,Left);
  for(i = Set->FirstElmt; i <= last; i ++)
    {
      if (IsElement (i, Set))
        {
          if(!IsFirst)
            go_puts(pOut,Center);
          else
            IsFirst = 0;
          (*Set->PrintFunc) (i,pOut);
        }
    }
  go_puts(pOut,Right);
}

  void
setInitModule(void)
{
}

  void
setCloseModule(void)
{
}

  void
setStatistics(FILE *out)
{
}

#if REUSE_COMPILE_CVSID - 0 != 0
static char const rcsid [] =
"$Id: Sets.c 354 2004-07-17 06:28:14Z cher $";
#endif /* REUSE_COMPILE_CVSID */

/**
 * Local variables:
 *  compile-command: "make"
 *  tags-file-name: "TAGS"
 *  c-font-lock-extra-types: ("\\sw+_t" "FILE" "tFileOut" "tGenOut" "tSet" "tsetElemPrint" "BITSET" "Bool")
 * End:
 */
