/* $Id: StringMem.c 354 2004-07-17 06:28:14Z cher $ */

/* Copyright (C) 1995-2004 Alexander Chernov <cher@ispras.ru> */
/* Ich, Doktor Josef Grosch, Informatiker, Sept. 1987 */

/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 */

#define __REUSE__ 1

/* internal include directives */
#include "lconfig.h"
#include "reuse/config.h"
#include "embed_version.h"

/* reuse include directives */
#include <reuse/Defs.h>
#include <reuse/StringMem.h>
#include <reuse/General.h>
#include <reuse/GenOut.h>
#include <reuse/xalloc.h>

#include <assert.h>
#include <string.h>

#define MemorySize      1024 * 20

typedef struct lBlock
{
  struct lBlock *Next;
  rulong_t         SizeLeft;
  tStringRef    Last;
} lBlock;

typedef struct tModuleState
{
  lBlock *list;
  rulong_t   number;
  rulong_t   size;
  rulong_t   used;
} tModuleState;

static  lBlock *BlockList       = NULL;

static rulong_t Chunk_Number;
static rulong_t Chunk_Size;
static rulong_t Chunk_Used;

/* ROUND MASK and REQ_SIZE depends on sizeof(tStringRefLen) *
 * here assumed that sizeof(tStringRefLen) == 4
 */
#define ROUND_MASK  0xfffffffcu
#define REQ_SIZE(s) ((s + 8) & ROUND_MASK)

static Bool initialized;
static void do_init(void)
{
  initialized = True;

  BlockList = NULL;
  Chunk_Number = 0;
  Chunk_Size = 0;
  Chunk_Used = 0;
}

  tStringRef
PutString(const char * string, tStringRefLen length)
{
  rulong_t      SizeNeeded = REQ_SIZE(length);
  lBlock     *Block_ptr;
  tStringRef result;

  if (!initialized) do_init();

  for (Block_ptr = BlockList;
       Block_ptr != NULL;
       Block_ptr = Block_ptr->Next)
    {
      if (Block_ptr->SizeLeft >= SizeNeeded)
        break;
    }

  if (Block_ptr == NULL)
    {
      rulong_t Block_Size = (MemorySize > SizeNeeded)? MemorySize : SizeNeeded;

      Block_ptr = (lBlock*) xmalloc(sizeof(lBlock) + Block_Size);
      assert(Block_ptr != NULL);

      Block_ptr->Last = (tStringRef) ((char*) Block_ptr + sizeof(lBlock));
      Block_ptr->SizeLeft = Block_Size;
      Block_ptr->Next = BlockList;
      BlockList = Block_ptr;
      Chunk_Number++;
      Chunk_Size += Block_Size;
    }

  result = Block_ptr->Last;
  *(Block_ptr->Last) = length;
  memcpy(Block_ptr->Last + 1, string, length);
  *((char*) (Block_ptr->Last + 1) + length) = 0;
  Block_ptr->Last += SizeNeeded / sizeof(tStringRefLen);
  Block_ptr->SizeLeft -= SizeNeeded;
  Chunk_Used += SizeNeeded;

  return result;
}

  tStringRefLen
_LengthSt(tStringRef str_ref)
{
  return *str_ref;
}

  char *
_stmString(tStringRef str_ref)
{
  return (char*) (str_ref + 1);
}

  void
StGetString( tStringRef stringref, char *string)
{
  Cardinal     length    = LengthSt (stringref);
  char *       stringReg = (char *) (stringref + 1);

  if (!initialized) do_init(); 
  
  while (length -- > 0) * string ++ = * stringReg ++;
  * string = '\0';
}

  Bool
IsEqualSt(tStringRef stringref, char *string)
{
   Cardinal     length    = LengthSt (stringref);
   char *       stringReg = (char *) (stringref + 1);

   if (!initialized) do_init();
  
  while (length -- > 0)
    if (* stringReg ++ != * string ++)
      return False;
  return True;
}

  void
WriteString(FILE *file, tStringRef stringref)
{
  Cardinal length = LengthSt (stringref);
  char     *ptr = (char*) (stringref + 1);
  
  if (!initialized) do_init();
  
  for(; length; length--)
    putc(*ptr++, file);
}

  void
stmWrite(tGenOut *gen_out, tStringRef stringref)
{
  Cardinal length = LengthSt (stringref);
  char     *ptr = (char*) (stringref + 1);
  
  if (!initialized) do_init();
  
  for(; length; length--)
    go_putc(gen_out, *ptr++);  
}

  void
WriteStringMemory (FILE *file)
{
  lBlock *BlockPtr;

  if (!initialized) do_init();
  
  for (BlockPtr = BlockList; BlockPtr != NULL; BlockPtr = BlockPtr->Next)
    {
      tStringRef Cur_Ref;

      for (Cur_Ref = (tStringRef) ((char*) BlockPtr + sizeof(lBlock));
           Cur_Ref != BlockPtr->Last;
           Cur_Ref += REQ_SIZE(*Cur_Ref) / sizeof(tStringRefLen))
        {
          fprintf(file, "%s\n", (char*) (Cur_Ref + 1));
        }
    }
}

  void
stmClear(void)
{
  lBlock *ptr;

  if (!initialized) do_init();

  for (ptr = BlockList; ptr != NULL; ptr = ptr->Next)
    {
      ptr->SizeLeft += (char*) ptr->Last - (char*)(ptr + 1);
      ptr->Last = (tStringRef) (ptr + 1);
    }
}

  void
InitStringMemory(void)
{
  if (!initialized) do_init();
}

  void
stmCloseModule(void)
{
  if (initialized)
    {
      lBlock *cur_ptr;
      lBlock *saved_ptr;

      initialized = False;

      for (cur_ptr = BlockList; cur_ptr != NULL;)
        {
          saved_ptr = cur_ptr->Next;
          xfree(cur_ptr);
          cur_ptr = saved_ptr;
        }
      BlockList = NULL;
      Chunk_Number = 0;
      Chunk_Size = 0;
      Chunk_Used = 0;
    }
}

  void *
stmSaveState(void)
{
  tModuleState *state = 0;

  /* allocate state structure */
  state = xcalloc(1, sizeof(*state));
  if (!state) return NULL;

  /* save the module state */
  state->list   = BlockList;
  state->number = Chunk_Number;
  state->size   = Chunk_Size;
  state->used   = Chunk_Used;

  /* initialize the state variables */
  BlockList     = NULL;
  Chunk_Number  = 0;
  Chunk_Size    = 0;
  Chunk_Used    = 0;

  return state;
}

  void *
stmRestoreState(void *pvstate)
{
  tModuleState *state = (tModuleState*) pvstate;
  lBlock *cur_ptr = 0, *saved_ptr = 0;

  if (!state) return NULL;

  /* free currently allocated memory */
  for (cur_ptr = BlockList; cur_ptr != NULL;)
    {
      saved_ptr = cur_ptr->Next;
      xfree(cur_ptr);
      cur_ptr = saved_ptr;
    }

  /* restore module state */
  BlockList    = state->list;
  Chunk_Number = state->number;
  Chunk_Size   = state->size;
  Chunk_Used   = state->used;

  xfree(state);
  return NULL;
}

  void
stmStatistics(FILE *file)
{
  fprintf(file,
          "StringMem module statistics:\n"
          "Allocated chunks:    %lu\n"
          "Chunks total size:   %lu\n"
          "Chunks used size:    %lu\n",
          Chunk_Number,
          Chunk_Size,
          Chunk_Used);
}

#if REUSE_COMPILE_CVSID - 0 != 0
static char const rcsid [] =
"$Id: StringMem.c 354 2004-07-17 06:28:14Z cher $";
#endif /* REUSE_COMPILE_CVSID */

/**
 * Local variables:
 *  compile-command: "make"
 *  c-font-lock-extra-types: ("\\sw+_t" "FILE" "Bool" "tModuleState" "lBlock" "tStringRef" "tStringRefLen" "Cardinal")
 *  tags-file-name: "TAGS"
 * End:
 */
