/* -*- mode:c -*- */
/* $Id: _vprintf.c 354 2004-07-17 06:28:14Z cher $ */

/* Copyright (C) 2002-2004 Alexander Chernov <cher@ispras.ru> */

/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 */

#define __REUSE__ 1

/* internal include directives */
#include "lconfig.h"
#include "reuse/config.h"
#include "embed_version.h"

/* reuse include directives */
#include <reuse/integral.h>
#include <reuse/number_io.h>
#include <reuse/format_io_int.h>
#include <reuse/str_utils.h>

#include <stdio.h>
#include <stdarg.h>
#include <ctype.h>
#include <string.h>
#include <stdlib.h>

#include "alloca_impl.h"

// valid size qualifiers
enum
  {
    SQ_int = 1,
    SQ_h,
    SQ_hh,
    SQ_l,
    SQ_ll,
    SQ_L
  };

#define ALLOCATE_BUF() \
  do {\
    int exp_len = width;\
    if (prec > width) exp_len = prec;\
    exp_len += 32;\
    if (local_len < exp_len) {\
      unsigned char *temp;\
      while (local_len < exp_len) local_len *= 2;\
      temp = (unsigned char*) do_alloca(local_len);\
      if (!temp) goto bad_format;\
      local_buf = temp;\
    }\
    output_buf = local_buf;\
  } while (0)

#define ASSEMBLE_FORMAT() \
  i = 0;\
  subformat_buf[i++] = '%';\
  if (alternate_form_flag) {\
    subformat_buf[i++] = '#';\
  }\
  if (blank_flag) {\
    subformat_buf[i++] = ' ';\
  }\
  if (always_sign_flag) {\
    subformat_buf[i++] = '+';\
  }\
  if (prec >= 0) {\
    subformat_buf[i++] = '.';\
    subformat_buf[i++] = '*';\
  }

int
_reuse_vprintf(struct reuse_printf_funcs *funcs, void *data,
               char const *format, va_list args)
{
  register unsigned char const *s = (unsigned char const*) format;
  int written = 0;
  unsigned char const *saved_s;
  char alternate_form_flag;
  char zero_padded_flag;
  char left_adjusted_flag;
  char blank_flag;
  char always_sign_flag;
  int width;
  int prec;
  int size_qualifier;

  unsigned char local_mem[512];
  unsigned char *local_buf = local_mem;
  int local_len = sizeof(local_mem);
  unsigned char *output_buf;
  int output_len;
  char is_numeric;
  char subformat_buf[64];
  int i;

  int integral_base;
  int integral_flags;

  if (!s) {
    static char const msg[] = "<null format string>";
    (*funcs->put_string)(data, msg);
    return sizeof(msg) - 1;
  }

  while (*s) {
    if (*s != '%') {
      saved_s = s;
      while (*s && *s != '%') s++;
      (*funcs->put_mem)(data, saved_s, s - saved_s);
      written += s - saved_s;
      continue;
    }

    saved_s = s;
    s++;

    if (*s == '%') {
      (*funcs->put_char)(data, *s);
      s++;
      written++;
      continue;      
    }

    // read flags
    alternate_form_flag = 0;
    zero_padded_flag = 0;
    left_adjusted_flag = 0;
    blank_flag = 0;
    always_sign_flag = 0;
    while (1) {
      switch (*s) {
      case '#':
        alternate_form_flag = 1;
        break;
      case '0':
        zero_padded_flag = 1;
        break;
      case '-':
        left_adjusted_flag = 1;
        break;
      case ' ':
        blank_flag = 1;
        break;
      case '+':
        always_sign_flag = 1;
        break;
      default:
        goto flags_done;
      }
      s++;
    }
  flags_done:
    // mutually incompatible options
    if (left_adjusted_flag && zero_padded_flag) {
      zero_padded_flag = 0;
    }
    if (always_sign_flag && blank_flag) {
      blank_flag = 0;
    }

    // read width, if given
    width = 0;
    if (*s == '*') {
      width = va_arg(args, int);
      if (width < 0) {
        left_adjusted_flag = 1;
        width = -width;
      }
      s++;
    } else if (*s >= '1' && *s <= '9') {
      while (*s >= '0' && *s <= '9' && width >= 0) {
        width = width * 10 + (*s - '0');
        s++;
      }
      if (width < 0) {
        // overflow
        //width = 0;
        //while (*s >= '0' && *s <= '9') {
        //  s++;
        //}
        // not sure about standard conformance
        goto bad_format;
      }
    }

    // read precision, if given
    prec = -1;
    if (*s == '.') {
      s++;
      prec = 0;
      if (*s == '*') {
        prec = va_arg(args, int);
        if (prec < 0) {
          prec = 0;
        }
        s++;
      } else if (*s >= '0' && *s <= '9') {
        while (*s >= '0' && *s <= '9' && prec >= 0) {
          prec = prec * 10 + (*s - '0');
          s++;
        }
        if (prec < 0) {
          // overflow
          //prec = 0;
          //while (*s >= '0' && *s <= '9') {
          //  s++;
          //}
          // not sure about standard conformance
          goto bad_format;
        }
      }
    }

    // read size qualifiers
    size_qualifier = 0;
    while (1) {
      switch (*s) {
      case 'h':
        if (!size_qualifier) {
          size_qualifier = SQ_h;
        } else if (size_qualifier == SQ_h) {
          size_qualifier = SQ_hh;
        } else {
          // not sure about standard conformance
          goto bad_format;
        }
        break;
      case 'l':
        if (!size_qualifier) {
          size_qualifier = SQ_l;
        } else if (size_qualifier == SQ_l) {
          size_qualifier = SQ_ll;
        } else {
          // not sure about standard conformance
          goto bad_format;
        }
        break;
      case 'L':
        // not sure about standard conformance
        if (size_qualifier) goto bad_format;
        size_qualifier = SQ_L;
        break;
      case 'q':
        // not sure about standard conformance
        if (size_qualifier) goto bad_format;
        size_qualifier = SQ_ll;
        break;
      case 'j':
        // not sure about standard conformance
        if (size_qualifier) goto bad_format;
        // intmax_t or uintmax_t conversion: int?
        size_qualifier = SQ_int;
        break;
      case 'z':
        // not sure about standard conformance
        if (size_qualifier) goto bad_format;
        size_qualifier = SQ_int;
        break;
      case 't':
        // not sure about standard conformance
        if (size_qualifier) goto bad_format;
        size_qualifier = SQ_int;
        break;
      case 'I':
        // not sure about standard conformance
        if (size_qualifier) goto bad_format;
        // this is brain-damaged M$ I64 or I32 conversion specifier :-(
        if (s[1] == '3' && s[2] == '2') {
          s += 3;
          size_qualifier = SQ_int;
          continue;
        }
        if (s[1] == '6' && s[2] == '4') {
          s += 3;
          size_qualifier = SQ_ll;
          continue;
        }
        goto bad_format;
      default:
        goto quals_done;
      }
      s++;
    }
  quals_done:
    is_numeric = 0;

    // parse conversion specifier
    switch (*s) {
    case 'd': case 'i':
      is_numeric = 1;
      if (prec >= 0) zero_padded_flag = 0;
      if (size_qualifier == SQ_ll || size_qualifier == SQ_L) {
#if CONF_PRINTF_INT64 - 0 == 1
        rllong_t value;
        unsigned char *b = R_F64;

        ASSEMBLE_FORMAT();
        while (*b) {
          subformat_buf[i++] = *b++;
        }
        subformat_buf[i++] = *s;
        subformat_buf[i++] = 0;

        value = va_arg(args, rllong_t);
        ALLOCATE_BUF();
        if (prec >= 0) {
          output_len = sprintf(output_buf, subformat_buf, prec, value);
        } else {
          output_len = sprintf(output_buf, subformat_buf, value);
        }
#else
        int flags = 0;
        rllong_t value;

        if (alternate_form_flag) flags |= REUSE_FORMAT_ALT;
        if (blank_flag) flags |= REUSE_FORMAT_SPC;
        if (always_sign_flag) flags |= REUSE_FORMAT_PLUS;

        value = va_arg(args, rllong_t);
        ALLOCATE_BUF();
        output_len = reuse_writell(output_buf,local_len,&value,10,prec,flags);
#endif // there is a way to print 64-bit numbers via printf
      } else {
        int value;

        ASSEMBLE_FORMAT();
        subformat_buf[i++] = *s;
        subformat_buf[i++] = 0;

        value = va_arg(args, int);
        if (size_qualifier == SQ_hh) {
          value = (signed char) value;
        } else if (size_qualifier == SQ_h) {
          value = (short) value;
        }

        ALLOCATE_BUF();
        if (prec >= 0) {
          output_len = sprintf(output_buf, subformat_buf, prec, value);
        } else {
          output_len = sprintf(output_buf, subformat_buf, value);
        }
      }
      goto write_buf;

    case 'o':
      integral_base = 8;
      integral_flags = 0;
      goto write_unsigned;
    case 'u':
      integral_base = 10;
      integral_flags = 0;
      goto write_unsigned;
    case 'x':
      integral_base = 16;
      integral_flags = 0;
      goto write_unsigned;
    case 'X':
      integral_base = 16;
      integral_flags = REUSE_FORMAT_UP;
    write_unsigned:
      is_numeric = 1;
      if (prec >= 0) zero_padded_flag = 0;
      if (size_qualifier == SQ_ll || size_qualifier == SQ_L) {
#if CONF_PRINTF_INT64 - 0 == 1
        rullong_t value;
        unsigned char *b = R_F64;

        ASSEMBLE_FORMAT();
        while (*b) {
          subformat_buf[i++] = *b++;
        }
        subformat_buf[i++] = *s;
        subformat_buf[i++] = 0;

        value = va_arg(args, rullong_t);
        ALLOCATE_BUF();
        if (prec >= 0) {
          output_len = sprintf(output_buf, subformat_buf, prec, value);
        } else {
          output_len = sprintf(output_buf, subformat_buf, value);
        }
#else
        rullong_t value;

        if (alternate_form_flag) integral_flags |= REUSE_FORMAT_ALT;
        if (blank_flag) integral_flags |= REUSE_FORMAT_SPC;
        if (always_sign_flag) integral_flags |= REUSE_FORMAT_PLUS;

        value = va_arg(args, rullong_t);
        ALLOCATE_BUF();
        output_len = reuse_writeull(output_buf, local_len, &value,
                                    integral_base, prec, integral_flags);
#endif // there is a way to print 64-bit numbers via printf
      } else {
        unsigned int value;

        ASSEMBLE_FORMAT();
        subformat_buf[i++] = *s;
        subformat_buf[i++] = 0;

        value = va_arg(args, unsigned int);
        if (size_qualifier == SQ_hh) {
          value = (unsigned char) value;
        } else if (size_qualifier == SQ_h) {
          value = (unsigned short) value;
        }

        ALLOCATE_BUF();
        if (prec >= 0) {
          output_len = sprintf(output_buf, subformat_buf, prec, value);
        } else {
          output_len = sprintf(output_buf, subformat_buf, value);
        }
      }
      goto write_buf;

    case 'e': case 'E':
    case 'f': case 'F':
    case 'g': case 'G':
      is_numeric = 1;
      if (size_qualifier && size_qualifier != SQ_L) goto bad_format;
#if R_LONG_DOUBLE_IS_DOUBLE - 0 != 1
      if (size_qualifier == SQ_L) {
# if CONF_PRINTF_LG_WORKS - 0 == 1
        long double value;

        ASSEMBLE_FORMAT();
        subformat_buf[i++] = 'L';
        subformat_buf[i++] = *s;
        subformat_buf[i++] = 0;

        value = va_arg(args, long double);
        ALLOCATE_BUF();
        if (prec >= 0) {
          output_len = sprintf(output_buf, subformat_buf, prec, value);
        } else {
          output_len = sprintf(output_buf, subformat_buf, value);
        }
# else
        long double value;
        int flags = 0;

        if (alternate_form_flag) flags |= REUSE_FORMAT_ALT;
        if (blank_flag) flags |= REUSE_FORMAT_SPC;
        if (always_sign_flag) flags |= REUSE_FORMAT_PLUS;
        if (*s == 'E' || *s == 'F' || *s == 'G') flags |= REUSE_FORMAT_UP;

        value = va_arg(args, long double);
        ALLOCATE_BUF();
        output_len = reuse_writedld(output_buf, local_len, &value,
                                    prec, flags, *s);
# endif // does printf support long doubles?
        goto write_buf;
      }
#endif // long double is distict from double
      // reassemble format string and run sprintf
      {
        double value;

        ASSEMBLE_FORMAT();
        subformat_buf[i++] = *s;
        subformat_buf[i++] = 0;

        value = va_arg(args, double);
        ALLOCATE_BUF();
        if (prec >= 0) {
          output_len = sprintf(output_buf, subformat_buf, prec, value);
        } else {
          output_len = sprintf(output_buf, subformat_buf, value);
        }
      }
      goto write_buf;

    case 'a': case 'A':
      is_numeric = 0;
      if (size_qualifier && size_qualifier != SQ_L) goto bad_format;
#if R_LONG_DOUBLE_IS_DOUBLE - 0 != 1
      if (size_qualifier == SQ_L) {
# if CONF_PRINTF_LG_WORKS - 0 == 1 && CONF_PRINTF_A_WORKS - 0 == 1
        long double value;

        ASSEMBLE_FORMAT();
        subformat_buf[i++] = 'L';
        subformat_buf[i++] = *s;
        subformat_buf[i++] = 0;

        value = va_arg(args, long double);
        ALLOCATE_BUF();
        if (prec >= 0) {
          output_len = sprintf(output_buf, subformat_buf, prec, value);
        } else {
          output_len = sprintf(output_buf, subformat_buf, value);
        }
# else
        long double value;
        int flags = 0;

        if (alternate_form_flag) flags |= REUSE_FORMAT_ALT;
        if (blank_flag) flags |= REUSE_FORMAT_SPC;
        if (always_sign_flag) flags |= REUSE_FORMAT_PLUS;
        if (*s == 'A') flags |= REUSE_FORMAT_UP;

        value = va_arg(args, long double);
        ALLOCATE_BUF();
        output_len = reuse_writehld(output_buf, local_len, &value,
                                    prec, flags);
# endif
        goto write_buf;
      }
#endif // long double is distict from double
#if CONF_PRINTF_A_WORKS - 0 == 1
      {
        double value;

        ASSEMBLE_FORMAT();
        subformat_buf[i++] = *s;
        subformat_buf[i++] = 0;

        value = va_arg(args, double);
        ALLOCATE_BUF();
        if (prec >= 0) {
          output_len = sprintf(output_buf, subformat_buf, prec, value);
        } else {
          output_len = sprintf(output_buf, subformat_buf, value);
        }
      }
#else
      {
        double value;
        int flags = 0;

        if (alternate_form_flag) flags |= REUSE_FORMAT_ALT;
        if (blank_flag) flags |= REUSE_FORMAT_SPC;
        if (always_sign_flag) flags |= REUSE_FORMAT_PLUS;
        if (*s == 'A') flags |= REUSE_FORMAT_UP;

        value = va_arg(args, double);
        ALLOCATE_BUF();
        output_len = reuse_writehd(output_buf, local_len, &value,
                                   prec, flags);
      }
#endif
      goto write_buf;

    case 'c':
      if (size_qualifier) goto bad_format;
      output_buf = local_buf;
      output_len = 1;
      is_numeric = 0;
      output_buf[0] = (unsigned char) va_arg(args, int);
      goto write_buf;
    case 's':
      if (size_qualifier) goto bad_format;
      output_buf = va_arg(args, unsigned char *);
      is_numeric = 0;
      if (prec < 0) {
        output_len = strlen(output_buf);
      } else {
        output_len = reuse_strnlen(output_buf, prec);
      }

      goto write_buf;
    case 'p':
      if (size_qualifier) goto bad_format;
      alternate_form_flag = 1;
      integral_base = 16;
      integral_flags = 0;
      goto write_unsigned;

    case 'n':
      switch (size_qualifier) {
      case 0: case SQ_int: case SQ_l:
        *va_arg(args, int *) = (int) written;
        break;
      case SQ_hh:
        *va_arg(args, signed char *)  = (signed char) written;
        break;
      case SQ_h:
        *va_arg(args, short *) = (short) written;
        break;
      case SQ_ll: case SQ_L:
        *va_arg(args, rint64_t*) = (rint64_t) written;
        break;
      default:
        goto bad_format;
      }
    default:
      goto bad_format;
    }
    s++;
    continue;

    /*
      output_buf - pointer to the output string location
      output_len - the length of the output string
      is_numeric - is this a number (to avoid inserting '0' before '-')
    */
  write_buf:
    if (output_len >= width) {
      (*funcs->put_mem)(data, output_buf, output_len);
      written += output_len;
      s++;
      continue;
    }
    if (left_adjusted_flag && (output_buf != local_buf || width > local_len)) {
      (*funcs->put_mem)(data, output_buf, output_len);
      written += width;
      width -= output_len;
      if (width > local_len) {
        memset(local_buf, ' ', local_len);
        while (width > local_len) {
          (*funcs->put_mem)(data, local_buf, local_len);
          width -= local_len;
        }
        (*funcs->put_mem)(data, local_buf, width);
      } else {
        memset(local_buf, ' ', width);
        (*funcs->put_mem)(data, local_buf, width);
      }
      s++;
      continue;
    }
    if (left_adjusted_flag) {
      // string to output is located in local_buf, and the width
      // is not too big
      for (; output_len < width; output_len++) {
        output_buf[output_len] = ' ';
      }
      (*funcs->put_mem)(data, output_buf, output_len);
      written += width;
      s++;
      continue;
    }
    if (zero_padded_flag && is_numeric  && output_len > 1
        && (output_buf[0]=='-' || output_buf[0]==' ' || output_buf[0]=='+')
        && isxdigit(output_buf[1])) {
      // keep the first symbol at the first place
      (*funcs->put_char)(data, output_buf[0]);
      written++;
      output_buf++;
      output_len--;
      width--;
    }
    if (zero_padded_flag && is_numeric && isxdigit(output_buf[0])) {
      // insert '0' at the beginning
      written += width;
      width -= output_len;
      for (; width; width--) {
        (*funcs->put_char)(data, '0');
      }
      (*funcs->put_mem)(data, output_buf, output_len);
      s++;
      continue;
    }
    // insert spaces
    written += width;
    width -= output_len;
    for (; width; width--) {
      (*funcs->put_char)(data, ' ');
    }
    (*funcs->put_mem)(data, output_buf, output_len);
    s++;
    continue;

  bad_format:
    s = saved_s;
    (*funcs->put_char)(data, *s);
    s++;
    written++;
  }

  return written;
}

#if REUSE_COMPILE_CVSID - 0 != 0
static char const rcsid [] =
"$Id: _vprintf.c 354 2004-07-17 06:28:14Z cher $";
#endif /* REUSE_COMPILE_CVSID */

/*
 * Local variables:
 *  c-font-lock-extra-types: ("\\sw+_t" "FILE" "va_list")
 *  compile-command: "make"
 * End:
 */

