/* -*- mode:c -*- */
/* $Id: _vscanf.c 354 2004-07-17 06:28:14Z cher $ */

/* Copyright (C) 1991-1999, 2000, 2001 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, write to the Free
   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
   02111-1307 USA.  */

#define __REUSE__ 1
#define __REUSE_FORCE_STRTOLL

/* internal include directives */
#include "lconfig.h"
#include "reuse/config.h"
#include "embed_version.h"

/* reuse include directives */
#include <reuse/integral.h>
#include <reuse/format_io_int.h>
#include <reuse/number_io.h>

#include <errno.h>
#include <limits.h>
#include <ctype.h>
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "alloca_impl.h"

/* Those are flags in the conversion format. */
#define LONG		0x001	/* l: long or double */
#define LONGDBL		0x002	/* L: long long or long double */
#define SHORT		0x004	/* h: short */
#define SUPPRESS	0x008	/* *: suppress assignment */
#define POINTER		0x010	/* weird %p pointer (`fake hex') */
#define NOSKIP		0x020	/* do not skip blanks */
#define WIDTH		0x040	/* width was given */
#define MALLOC		0x100	/* a: malloc strings */
#define CHAR		0x200	/* hh: char */

#define ungetc(c)	((void) (c != EOF && --read_in), (*vfunc->prev_char)(vdata, c))
#define ungetc_not_eof(c)	(--read_in, (*vfunc->prev_char)(vdata, c))
#define inchar()	(c == EOF ? EOF : ((c = (*vfunc->next_char)(vdata)), (void) (c != EOF && ++read_in), c))

#define conv_error() return (done)
#define input_error() return (done ? done : EOF)

#define ADDW(Ch)							    \
  do									    \
    {									    \
      if (wpsize == wpmax)						    \
	{								    \
	  char *old = wp;						    \
	  wpmax = (UCHAR_MAX + 1 > 2 * wpmax ? UCHAR_MAX + 1 : 2 * wpmax);  \
	  wp = (char *) do_alloca (wpmax * sizeof (wchar_t));		    \
	  if (old != NULL)						    \
	    memcpy (wp, old, wpsize);					    \
	}								    \
      wp[wpsize++] = (Ch);						    \
    }									    \
  while (0)

#define ARG(type) (va_arg(arg, type))

/* Read formatted input from S according to the format string
   FORMAT, using the argument list in ARG.
   Return the number of assignments made, or -1 for an input error.  */
int
_reuse_vscanf(struct reuse_scanf_funcs *vfunc, void *vdata,
              char const *format, va_list arg)
{
  register const char *f = format;
  register ruchar_t fc;         /* Current character of the format. */
  register int done = 0;	/* Assignments done.  */
  register size_t read_in = 0;	/* Chars read in.  */
  register int c = 0;           /* Last char read.  */
  register int width;		/* Maximum field width.  */
  register int flags;		/* Modifiers for current format element.  */

  /* Status for reading F-P nums.  */
  char got_dot, got_e, negative;
  /* If a [...] is a [^...].  */
  char not_in;
#define exp_char not_in
  /* Base for integral numbers.  */
  int base;
  /* Signedness for integral numbers.  */
  int number_signed;
#define is_hexa number_signed
  /* Integral holding variables.  */
  union
    {
      rllong_t q;
      rullong_t uq;
      long int l;
      unsigned long int ul;
    } num;
  /* Character-buffer pointer.  */
  char *str = NULL;
  char **strptr = NULL;
  int strsize = 0;
  /* We must not react on white spaces immediately because they can
     possibly be matched even if in the input stream no character is
     available anymore.  */
  int skip_space = 0;
  /* Nonzero if we are reading a pointer.  */
  int read_pointer;
  /* Workspace.  */
  char *tw;			/* Temporary pointer.  */
  char *wp = NULL;		/* Workspace.  */
  size_t wpmax = 0;		/* Maximal size of workspace.  */
  size_t wpsize;		/* Currently used bytes in workspace.  */

  /* Run through the format string.  */
  while (*f != '\0')
    {
      fc = *f++;
      if (fc != '%')
	{
	  /* Remember to skip spaces.  */
	  if (isspace (fc))
	    {
	      skip_space = 1;
	      continue;
	    }

	  /* Read a character.  */
	  c = inchar ();

	  /* Characters other than format specs must just match.  */
	  if (c == EOF)
	    input_error ();

	  /* We saw white space char as the last character in the format
	     string.  Now it's time to skip all leading white space.  */
	  if (skip_space)
	    {
	      while (isspace (c))
		if (inchar () == EOF && errno == EINTR)
		  conv_error ();
	      skip_space = 0;
	    }

	  if (c != fc)
	    {
	      ungetc(c);
	      conv_error ();
	    }

	  continue;
	}

      /* This is the start of the conversion string. */
      flags = 0;

      /* Not yet decided whether we read a pointer or not.  */
      read_pointer = 0;

      /* Prepare temporary buffer.  */
      wpsize = 0;

      /* Check for a positional parameter specification.  */
      if (isdigit((ruchar_t) *f))
	{
	  width = (ruchar_t) *f++ - '0';
	  while (isdigit((ruchar_t) *f))
	    width = width * 10 + ((ruchar_t) *f++ - '0');
          flags |= WIDTH;
          goto got_width;
	}

      /* Check for the assignment-suppressing */
      while (*f == '*')
	switch (*f++)
	  {
	  case '*':
	    flags |= SUPPRESS;
	    break;
	  }

      /* Check for the assignment-suppressing */
      while (*f == '*')
	switch (*f++)
	  {
	  case '*':
	    flags |= SUPPRESS;
	    break;
	  }

      /* We have seen width. */
      if (isdigit ((ruchar_t) *f))
	flags |= WIDTH;

      /* Find the maximum field width.  */
      width = 0;
      while (isdigit ((ruchar_t) *f))
	{
	  width *= 10;
	  width += (ruchar_t) *f++ - '0';
	}
    got_width:
      if (width == 0)
	width = -1;

      /* Check for type modifiers.  */
      switch (*f++)
	{
	case 'h':
	  /* ints are short ints or chars.  */
	  if (*f == 'h')
	    {
	      ++f;
	      flags |= CHAR;
	    }
	  else
	    flags |= SHORT;
	  break;
	case 'l':
	  if (*f == 'l')
	    {
	      /* A double `l' is equivalent to an `L'.  */
	      ++f;
	      flags |= LONGDBL | LONG;
	    }
	  else
	    /* ints are long ints.  */
	    flags |= LONG;
	  break;
	case 'q':
	case 'L':
	  /* doubles are long doubles, and ints are long long ints.  */
	  flags |= LONGDBL | LONG;
	  break;
	case 'a':
	  /* The `a' is used as a flag only if followed by `s', `S' or
	     `['.  */
	  if (*f != 's' && *f != '[')
	    {
	      --f;
	      break;
	    }
	  /* String conversions (%s, %[) take a `char **'
	     arg and fill it in with a malloc'd pointer.  */
	  flags |= MALLOC;
	  break;
	case 'z':
          flags |= LONG;
	  break;
	case 'j':
          flags |= LONG;
	  break;
	case 't':
          flags |= LONG;
	  break;
	default:
	  /* Not a recognized modifier.  Backup.  */
	  --f;
	  break;
	}

      /* End of the format string?  */
      if (*f == '\0')
	conv_error ();

      /* Find the conversion specifier.  */
      fc = *f++;
      if (skip_space || (fc != '[' && fc != 'c' && fc != 'n'))
	{
	  /* Eat whitespace.  */
	  int save_errno = errno;
	  errno = 0;
	  do
	    if (inchar () == EOF && errno == EINTR)
	      input_error ();
	  while (isspace (c));
	  errno = save_errno;
	  ungetc(c);
	  skip_space = 0;
	}

      switch (fc)
	{
	case '%':	/* Must match a literal '%'.  */
	  c = inchar ();
	  if (c == EOF)
	    input_error ();
	  if (c != fc)
	    {
	      ungetc_not_eof(c);
	      conv_error ();
	    }
	  break;

	case 'n':	/* Answer number of assignments done.  */
	  /* Corrigendum 1 to ISO C 1990 describes the allowed flags
	     with the 'n' conversion specifier.  */
	  if (!(flags & SUPPRESS))
	    {
	      /* Don't count the read-ahead.  */
	      if ((flags & LONGDBL))
		*ARG (rint64_t *) = read_in;
	      else if (flags & SHORT)
		*ARG (short int *) = read_in;
	      else if (!(flags & CHAR))
		*ARG (int *) = read_in;
	      else
		*ARG (char *) = read_in;
	    }
	  break;

	case 'c':	/* Match characters.  */
          if (!(flags & SUPPRESS))
            {
              str = ARG (char *);
              if (str == NULL)
                conv_error ();
            }

          c = inchar ();
          if (c == EOF)
            input_error ();

          if (width == -1)
            width = 1;

          if (!(flags & SUPPRESS))
            {
              do
                *str++ = c;
              while (--width > 0 && inchar () != EOF);
            }
          else
            while (--width > 0 && inchar () != EOF);

          if (!(flags & SUPPRESS))
            ++done;

          break;

#define STRING_ARG(Str, Type)						      \
	      do if (!(flags & SUPPRESS))				      \
		{							      \
		  if (flags & MALLOC)					      \
		    {							      \
		      /* The string is to be stored in a malloc'd buffer.  */ \
		      strptr = ARG (char **);				      \
		      if (strptr == NULL)				      \
			conv_error ();					      \
		      /* Allocate an initial buffer.  */		      \
		      strsize = 100;					      \
		      *strptr = (char *) malloc (strsize * sizeof (Type));    \
		      Str = (Type *) *strptr;				      \
		    }							      \
		  else							      \
		    Str = ARG (Type *);					      \
		  if (Str == NULL)					      \
		    conv_error ();					      \
		} while (0)

	case 's':		/* Read a string.  */
          STRING_ARG (str, char);

          c = inchar ();
          if (c == EOF)
            input_error ();

          do
            {
              if (isspace (c))
                {
                  ungetc_not_eof(c);
                  break;
                }

              /* This is easy.  */
              if (!(flags & SUPPRESS))
                {
                  *str++ = c;
                  if ((flags & MALLOC)
                      && (char *) str == *strptr + strsize)
                    {
                      /* Enlarge the buffer.  */
                      str = (char *) realloc (*strptr, 2 * strsize);
                      if (str == NULL)
                        {
                          /* Can't allocate that much.  Last-ditch
                             effort.  */
                          str = (char *) realloc (*strptr, strsize + 1);
                          if (str == NULL)
                            {
                              /* We lose.  Oh well.  Terminate the
                                 string and stop converting,
                                 so at least we don't skip any input.  */
                              ((char *) (*strptr))[strsize - 1] = '\0';
                              ++done;
                              conv_error ();
                            }
                          else
                            {
                              *strptr = (char *) str;
                              str += strsize;
                              ++strsize;
                            }
                        }
                      else
                        {
                          *strptr = (char *) str;
                          str += strsize;
                          strsize *= 2;
                        }
                    }
                }
            }
          while ((width <= 0 || --width > 0) && inchar () != EOF);

          if (!(flags & SUPPRESS))
            {
              *str++ = '\0';

              if ((flags & MALLOC) && str - *strptr != strsize)
                {
                  char *cp = (char *) realloc (*strptr, str - *strptr);
                  if (cp != NULL)
                    *strptr = cp;
                }

              ++done;
            }
          break;

	case 'x':	/* Hexadecimal integer.  */
	case 'X':	/* Ditto.  */
	  base = 16;
	  number_signed = 0;
	  goto number;

	case 'o':	/* Octal integer.  */
	  base = 8;
	  number_signed = 0;
	  goto number;

	case 'u':	/* Unsigned decimal integer.  */
	  base = 10;
	  number_signed = 0;
	  goto number;

	case 'd':	/* Signed decimal integer.  */
	  base = 10;
	  number_signed = 1;
	  goto number;

	case 'i':	/* Generic number.  */
	  base = 0;
	  number_signed = 1;

	number:
	  c = inchar ();
	  if (c == EOF)
	    input_error ();

	  /* Check for a sign.  */
	  if (c == '-' || c == '+')
	    {
	      ADDW (c);
	      if (width > 0)
		--width;
	      c = inchar ();
	    }

	  /* Look for a leading indication of base.  */
	  if (width != 0 && c == '0')
	    {
	      if (width > 0)
		--width;

	      ADDW (c);
	      c = inchar ();

	      if (width != 0 && tolower (c) == 'x')
		{
		  if (base == 0)
		    base = 16;
		  if (base == 16)
		    {
		      if (width > 0)
			--width;
		      c = inchar ();
		    }
		}
	      else if (base == 0)
		base = 8;
	    }

	  if (base == 0)
	    base = 10;

          /* Read the number into workspace.  */
          while (c != EOF && width != 0)
            {
              if (base == 16)
                {
                  if (!isxdigit (c))
                    break;
                }
              else if (!isdigit (c) || c - '0' >= base)
                {
                  break;
                }
              ADDW (c);
              if (width > 0)
                --width;

              c = inchar ();
            }

	  if (wpsize == 0
	      || (wpsize == 1 && (wp[0] == '+' || wp[0] == '-')))
	    {
	      /* There was no number.  If we are supposed to read a pointer
		 we must recognize "(nil)" as well.  */
	      if (wpsize == 0 && read_pointer && (width < 0 || width >= 0)
		  && c == '('
		  && tolower (inchar ()) == 'n'
		  && tolower (inchar ()) == 'i'
		  && tolower (inchar ()) == 'l'
		  && inchar () == ')')
		/* We must produce the value of a NULL pointer.  A single
		   '0' digit is enough.  */
		ADDW ('0');
	      else
		{
		  /* The last read character is not part of the number
		     anymore.  */
		  ungetc(c);

		  conv_error ();
		}
	    }
	  else
	    /* The just read character is not part of the number anymore.  */
	    ungetc(c);

	  /* Convert the number.  */
	  ADDW ('\0');
	  if ((flags & LONGDBL))
	    {
	      if (number_signed)
		num.q = reuse_strtoll(wp, &tw, base);
	      else
		num.uq = reuse_strtoull(wp, &tw, base);
	    }
	  else
	    {
	      if (number_signed)
		num.l = strtol(wp, &tw, base);
	      else
		num.ul = strtoul(wp, &tw, base);
	    }
	  if (wp == tw)
	    conv_error ();

	  if (!(flags & SUPPRESS))
	    {
	      if (! number_signed)
		{
		  if ((flags & LONGDBL))
		    *ARG (ruint64_t *) = num.uq;
		  else if (flags & SHORT)
		    *ARG (unsigned short *) = (unsigned short) num.ul;
		  else if (!(flags & CHAR))
		    *ARG (unsigned int *) = (unsigned int) num.ul;
		  else
		    *ARG (unsigned char *) = (unsigned char) num.ul;
		}
	      else
		{
		  if ((flags & LONGDBL))
		    *ARG (rint64_t *) = num.q;
		  else if (flags & SHORT)
		    *ARG (short int *) = (short int) num.l;
		  else if (!(flags & CHAR))
		    *ARG (int *) = (int) num.l;
		  else
		    *ARG (signed char *) = (signed char) num.ul;
		}
	      ++done;
	    }
	  break;

	case 'e':	/* Floating-point numbers.  */
	case 'E':
	case 'f':
	case 'F':
	case 'g':
	case 'G':
	case 'a':
	case 'A':
	  c = inchar ();
	  if (c == EOF)
	    input_error ();

	  /* Check for a sign.  */
	  if (c == '-' || c == '+')
	    {
	      negative = c == '-';
	      if (width == 0 || inchar () == EOF)
		/* EOF is only an input error before we read any chars.  */
		conv_error ();
	      if (! isdigit (c) && tolower (c) != 'i'
		  && tolower (c) != 'n' && c != '.')
		{
                  ungetc(c);
                  conv_error();
		}
	      if (width > 0)
		--width;
	    }
	  else
	    negative = 0;

	  /* Take care for the special arguments "nan" and "inf".  */
	  if (tolower (c) == 'n')
	    {
	      /* Maybe "nan".  */
	      ADDW (c);
	      if (width == 0 || inchar () == EOF || tolower (c) != 'a')
		conv_error ();
	      if (width > 0)
		--width;
	      ADDW (c);
	      if (width == 0 || inchar () == EOF || tolower (c) != 'n')
		conv_error ();
	      if (width > 0)
		--width;
	      ADDW (c);
	      /* It is "nan".  */
	      goto scan_float;
	    }
	  else if (tolower (c) == 'i')
	    {
	      /* Maybe "inf" or "infinity".  */
	      ADDW (c);
	      if (width == 0 || inchar () == EOF || tolower (c) != 'n')
		conv_error ();
	      if (width > 0)
		--width;
	      ADDW (c);
	      if (width == 0 || inchar () == EOF || tolower (c) != 'f')
		conv_error ();
	      if (width > 0)
		--width;
	      ADDW (c);
	      /* It is as least "inf".  */
	      if (width != 0 && inchar () != EOF)
		{
		  if (tolower (c) == 'i')
		    {
		      if (width > 0)
			--width;
		      /* Now we have to read the rest as well.  */
		      ADDW (c);
		      if (width == 0 || inchar () == EOF
			  || tolower (c) != 'n')
			conv_error ();
		      if (width > 0)
			--width;
		      ADDW (c);
		      if (width == 0 || inchar () == EOF
			  || tolower (c) != 'i')
			conv_error ();
		      if (width > 0)
			--width;
		      ADDW (c);
		      if (width == 0 || inchar () == EOF
			  || tolower (c) != 't')
			conv_error ();
		      if (width > 0)
			--width;
		      ADDW (c);
		      if (width == 0 || inchar () == EOF
			  || tolower (c) != 'y')
			conv_error ();
		      if (width > 0)
			--width;
		      ADDW (c);
		    }
		  else
		    /* Never mind.  */
		    ungetc(c);
		}
	      goto scan_float;
	    }

	  is_hexa = 0;
	  exp_char = 'e';
	  if (width != 0 && c == '0')
	    {
	      ADDW (c);
	      c = inchar ();
	      if (width > 0)
		--width;
	      if (width != 0 && tolower (c) == 'x')
		{
		  /* It is a number in hexadecimal format.  */
		  ADDW (c);

		  is_hexa = 1;
		  exp_char = 'p';

		  /* Grouping is not allowed.  */
		  c = inchar ();
		  if (width > 0)
		    --width;
		}
	    }

	  got_dot = got_e = 0;
	  do
	    {
	      if (isdigit (c))
		ADDW (c);
	      else if (!got_e && is_hexa && isxdigit (c))
		ADDW (c);
	      else if (got_e && wp[wpsize - 1] == exp_char
		       && (c == '-' || c == '+'))
		ADDW (c);
	      else if (wpsize > 0 && !got_e && tolower (c) == exp_char)
		{
		  ADDW (exp_char);
		  got_e = got_dot = 1;
		}
	      else if (!got_dot && c == '.')
		{
                  ADDW ('.');
                  got_dot = 1;
		}
              else
                {
                  /* The last read character is not part of the number
                     anymore.  */
                  ungetc(c);
                  break;
                }

	      if (width > 0)
		--width;
	    }
	  while (width != 0 && inchar () != EOF);

	  /* Have we read any character?  If we try to read a number
	     in hexadecimal notation and we have read only the `0x'
	     prefix or no exponent this is an error.  */
	  if (wpsize == 0 || (is_hexa && (wpsize == 2 || ! got_e)))
	    conv_error ();

	scan_float:
	  /* Convert the number.  */
	  ADDW ('\0');
	  if (flags & LONGDBL)
	    {
	      long double d = reuse_strtold (wp, &tw);
	      if (!(flags & SUPPRESS) && tw != wp)
		*ARG (long double *) = negative ? -d : d;
	    }
	  else if (flags & LONG)
	    {
	      double d = reuse_strtod (wp, &tw);
	      if (!(flags & SUPPRESS) && tw != wp)
		*ARG (double *) = negative ? -d : d;
	    }
	  else
	    {
	      float d = reuse_strtof (wp, &tw);
	      if (!(flags & SUPPRESS) && tw != wp)
		*ARG (float *) = negative ? -d : d;
	    }

	  if (tw == wp)
	    conv_error ();

	  if (!(flags & SUPPRESS))
	    ++done;
	  break;

	case '[':	/* Character class.  */
          STRING_ARG (str, char);

	  if (*f == '^')
	    {
	      ++f;
	      not_in = 1;
	    }
	  else
	    not_in = 0;

	  if (width < 0)
	    /* There is no width given so there is also no limit on the
	       number of characters we read.  Therefore we set width to
	       a very high value to make the algorithm easier.  */
	    width = INT_MAX;

	  /* Fill WP with byte flags indexed by character.
	     We will use this flag map for matching input characters.  */
	  if (wpmax < UCHAR_MAX + 1)
	    {
	      wpmax = UCHAR_MAX + 1;
	      wp = (char *) do_alloca (wpmax);
	    }
	  memset (wp, '\0', UCHAR_MAX + 1);

	  fc = *f;
	  if (fc == ']' || fc == '-')
	    {
	      /* If ] or - appears before any char in the set, it is not
		 the terminator or separator, but the first char in the
		 set.  */
	      wp[fc] = 1;
	      ++f;
	    }

	  while ((fc = *f++) != '\0' && fc != ']')
	    if (fc == '-' && *f != '\0' && *f != ']'
		&& (unsigned char) f[-2] <= (unsigned char) *f)
	      {
		/* Add all characters from the one before the '-'
		   up to (but not including) the next format char.  */
		for (fc = (unsigned char) f[-2]; fc < (unsigned char) *f; ++fc)
		  wp[fc] = 1;
	      }
	    else
	      /* Add the character to the flag map.  */
	      wp[fc] = 1;

	  if (fc == '\0')
	    conv_error();

	    {
	      size_t now = read_in;

	      if (inchar () == EOF)
		input_error ();

	      do
		{
		  if (wp[c] == not_in)
		    {
		      ungetc_not_eof(c);
		      break;
		    }

		  /* This is easy.  */
		  if (!(flags & SUPPRESS))
		    {
		      *str++ = c;
		      if ((flags & MALLOC)
			  && (char *) str == *strptr + strsize)
			{
			  /* Enlarge the buffer.  */
			  str = (char *) realloc (*strptr, 2 * strsize);
			  if (str == NULL)
			    {
			      /* Can't allocate that much.  Last-ditch
				 effort.  */
			      str = (char *) realloc (*strptr, strsize + 1);
			      if (str == NULL)
				{
				  /* We lose.  Oh well.  Terminate the
				     string and stop converting,
				     so at least we don't skip any input.  */
				  ((char *) (*strptr))[strsize - 1] = '\0';
				  ++done;
				  conv_error ();
				}
			      else
				{
				  *strptr = (char *) str;
				  str += strsize;
				  ++strsize;
				}
			    }
			  else
			    {
			      *strptr = (char *) str;
			      str += strsize;
			      strsize *= 2;
			    }
			}
		    }
		}
	      while (--width > 0 && inchar () != EOF);

	      if (now == read_in)
		/* We haven't succesfully read any character.  */
		conv_error ();

	      if (!(flags & SUPPRESS))
		{
		  *str++ = '\0';

		  if ((flags & MALLOC) && str - *strptr != strsize)
		    {
		      char *cp = (char *) realloc (*strptr, str - *strptr);
		      if (cp != NULL)
			*strptr = cp;
		    }

		  ++done;
		}
	    }
	  break;

	case 'p':	/* Generic pointer.  */
	  base = 16;
	  /* A PTR must be the same size as a `long int'.  */
	  flags &= ~(SHORT|LONGDBL);
	  number_signed = 0;
	  read_pointer = 1;
	  goto number;

	default:
	  /* If this is an unknown format character punt.  */
	  conv_error ();
	}
    }

  /* The last thing we saw int the format string was a white space.
     Consume the last white spaces.  */
  if (skip_space)
    {
      do
	c = inchar ();
      while (isspace (c));
      ungetc(c);
    }

  return done;
}

#if REUSE_COMPILE_CVSID - 0 != 0
static char const rcsid [] =
"$Id: _vscanf.c 354 2004-07-17 06:28:14Z cher $";
#endif /* REUSE_COMPILE_CVSID */

/*
 * Local variables:
 *  c-font-lock-extra-types: ("\\sw+_t" "FILE" "va_list")
 *  compile-command: "make"
 * End:
 */
