/* $Id: stream_io.c 57 2001-06-23 07:04:26Z cher $ */

/* Copyright (C) 2000-2001 Alexander Chernov <cher@ispras.ru> */
/* Created: <2000-07-01 19:47:09 cher> */

/*
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 */

/**
 * FILE:    utils/stream_io.c
 * PURPOSE: stream abstraction layer
 */

static char const cvsid[] =
"$Id: stream_io.c 57 2001-06-23 07:04:26Z cher $";

#include "lconfig.h"
#include <p_config.h>

#include CONF_DEPFILE("reuse_version.h")

#include "reuse/stream_io.h"
#include "reuse/logger.h"
#include "reuse/xalloc.h"
#include "reuse/osdeps.h"

#define NOT_IMPLEMENTED() SWERR(("%s: not implemented", __FUNCTION__))

/*
#include <stdio.h>
#include <stdarg.h>
#include <errno.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
*/

/* stream descriptor structure */
#if 0
struct tStream
{
  int             kind;         /* sk_Inet or sk_File */
  char           *path;         /* path to the file */
  int             port;         /* TCP port */
  int             _fd;          /* file descriptor */
  int             ifd;
  int             ofd;
  int             listen_fd;    /* fd as the result of listen(2) call */
  int             mode;         /* stream mode (read,write,append) */
  int             is_opened;    /* 1, if the stream is opened */
  struct sockaddr_in inaddr;    /* peer address */
  struct sockaddr_in saddr;     /* bound address */
  int             status;       /* error status */
  int             reuseaddr;    /* set SO_REUSEADDR? */
  struct {
    int             bufmode;    /* buffering mode */
    int             bufsize;    /* size of the buffer */
    FILE           *file;       /* stream structure */
    char           *logname;    /* log file name */
    FILE           *logfile;    /* file to write the log */
  }               desc[2];      /* descriptors for read and write substreams */
};
#endif

/**
 * NAME:    strm_GetKind
 * PURPOSE: get the type (sk_Inet or sk_File) of the stream
 * ARGS:    s - stream
 * RETURN:  stream type
 */
  int
strm_GetKind(tpStream s)
{
  /*
  ASSERT(s != NULL);

  return s->kind;
  */
  NOT_IMPLEMENTED();
}

/**
 * NAME:    strm_New
 * PURPOSE: create new stream
 * ARGS:    k - stream type (sk_Inet or sk_File)
 * RETURN:  pointer to the newly created stream structure
 */
  tpStream
strm_New(int k)
{
  /*
  switch (k)
    {
    case sk_Inet:
      {
        tStream *r;

        r = (tStream*) xcalloc(1, sizeof(tStream));
        r->kind = sk_Inet;
        r->ifd = -1;
        r->ofd = -1;
        r->port = -1;
        r->desc[SFD_INPUT].bufsize = -1;
        r->desc[SFD_OUTPUT].bufsize = -1;
        r->desc[SFD_INPUT].bufmode = -1;
        r->desc[SFD_OUTPUT].bufmode = -1;

        return r;
      }
    case sk_File:
      {
        tStream *r;

        r = (tStream*) xcalloc(1, sizeof(tStream));
        r->kind = sk_File;
        r->ifd = -1;
        r->ofd = -1;
        r->port = -1;
        r->desc[SFD_INPUT].bufsize = -1;
        r->desc[SFD_OUTPUT].bufsize = -1;
        r->desc[SFD_INPUT].bufmode = -1;
        r->desc[SFD_OUTPUT].bufmode = -1;

        return r;
      }
    default:
      SWERR(("k == %d", k));
    }
  */
  NOT_IMPLEMENTED();
}

/**
 * NAME:    strm_Fcntl
 * PURPOSE: perform a control operation over the stream
 * ARGS:    s    - stream
 *          code - operation code
 *          ...  - extra operation specific arguments
 * RETURN:  0 - ok, -1 - error
 * NOTE:    extra parameters for operations:
 *          SFC_SETPORT
 *            int   port    - TCP port number
 *          SFC_SETFD
 *            int   fd      - opened file descriptor
 *          SFC_SETPATH
 *            char *path    - file path
 *          SFC_SETBUFSIZE
 *            int   dir     - direction (SFD_INPUT, SFD_OUTPUT)
 *            int   bufsize - size of the buffer
 *          SFC_SETBUFMODE
 *            int   dir     - direction
 *            int   bufmode - mode of the buffer
 *          SFC_SETMODE
 *            int   mode    - SFM_READ, SFM_WRITE, SFM_APPEND
 *          SFC_SETSOCKOPT
 *            int   opt     - socket options
 *          SFC_SETLOGFILE
 *            int   dir     - direction
 *            char *name    - log file name
 */
  int
strm_Fcntl(tpStream s, int code, ...)
{
  /*
  va_list args;
  int     r = 0;

  ASSERT(s != NULL);

  va_start(args, code);

  switch (code)
    {
    case SFC_SETPORT:
      {
        int port;

        port = va_arg(args, int);

        ASSERT(s->kind == sk_Inet);
        ASSERT(port >= 0 && port <= 65535);
        ASSERT(s->ifd == -1 && s->ofd == -1);

        s->port = port;
        r = 0;
      }
    break;
    case SFC_SETFD: case SFC_SETOFD: case SFC_SETIFD:
      {
        int fd;

        fd = va_arg(args, int);

        ASSERT(fd >= 0 && fd <= 65535);
        ASSERT(s->port == -1);
        ASSERT(s->path == NULL);

        switch (code)
          {
          case SFC_SETFD:  s->ifd = fd; s->ofd = fd; break;
          case SFC_SETIFD: s->ifd = fd; break;
          case SFC_SETOFD: s->ofd = fd; break;
          }
        r = 0;
      }
    break;
    case SFC_SETPATH:
      {
        char *path;

        path = va_arg(args, char*);

        ASSERT(path != NULL);
        ASSERT(s->kind == sk_File);
        ASSERT(s->ifd == -1 && s->ofd == -1);

        s->path = xstrdup(path);
        r = 0;
      }
    break;
    case SFC_SETBUFSIZE:
      {
        int dir;
        int bufsize;

        dir     = va_arg(args, int);
        bufsize = va_arg(args, int);

        ASSERT(dir >= 0 && dir <= 1);
        ASSERT(bufsize >= 0 && bufsize <= 65535);

        s->desc[dir].bufsize = bufsize;
        r = 0;
      }
    break;
    case SFC_SETBUFMODE:
      {
        int dir;
        int bufmode;

        dir     = va_arg(args, int);
        bufmode = va_arg(args, int);

        ASSERT(dir >= 0 && dir <= 1);
        ASSERT(bufmode >= 0 && bufmode <= 2);
        
        s->desc[dir].bufmode = bufmode;
        r = 0;
      }
    break;
    case SFC_SETMODE:
      {
        int mode;

        mode = va_arg(args, int);

        ASSERT(mode >= 0 && mode <=7);

        s->mode = mode;
        r = 0;
      }
    break;

    case SFC_GETPEERHOST:
      {
      }
    break;
    case SFC_GETPEERPORT:
      {
      }
    break;

    case SFC_SETSOCKOPT:
      {
        int         inopt;

        ASSERT(s != NULL);
        ASSERT(!s->is_opened);

        va_start(args, code);
        inopt   = va_arg(args, int);
        va_end(args);

        switch (inopt)
          {
          case SFCS_REUSEADDR:
            s->reuseaddr = 1;
            break;
          default:
            SWERR(("Invalid socket option"));
          }
      }
      break;

    case SFC_SETLOGFILE:
      {
        int   dir;
        char *name;

        dir  = va_arg(args, int);
        name = va_arg(args, char *);

        ASSERT(dir >= 0 && dir <= 1);
        s->desc[dir].logname = xstrdup(name);
        r = 0;
      }
      break;
      
    case SFC_SETTRIM:
    case SFC_SETBIN:
    case SFC_SETTEXT:
    case SFC_LENGTH:
      SWERR(("not supported"));
    default:
      SWERR(("fcntl code == %d", code));
    }

  va_end(args);

  return r;
  */
  NOT_IMPLEMENTED();
}

/**
 * NAME:    strm_Open
 * PURPOSE: open the stream
 * ARGS:    s - stream
 * RETURN:  0 - ok, <0 - error
 */
  int
strm_Open(tpStream s)
{
#if 0
  ASSERT(s != NULL);
  ASSERT(!s->is_opened);

  /* open logs */
  if (s->desc[SFD_OUTPUT].logname)
    {
      s->desc[SFD_OUTPUT].logfile = fopen(s->desc[SFD_OUTPUT].logname, "w");
      if (s->desc[SFD_OUTPUT].logfile)
        setbuf(s->desc[SFD_OUTPUT].logfile, 0);
    }
  if (s->desc[SFD_INPUT].logname)
    {
      s->desc[SFD_INPUT].logfile = fopen(s->desc[SFD_INPUT].logname, "w");
      if (s->desc[SFD_INPUT].logfile)
        setbuf(s->desc[SFD_INPUT].logfile, 0);
    }

  errno = 0;

  if (s->ifd >= 0 || s->ofd >= 0)
    {
      /* we're given a file descriptor */
      if ((s->mode & SFM_APPEND))
        {
          if (s->ofd < 0)
            {
              errno = EBADF;
              return -1;
            }

          errno = 0;
          s->desc[SFD_OUTPUT].file = fdopen(s->ofd, "a");
          if (s->desc[SFD_OUTPUT].file == NULL)
            {
              s->status = errno;
              return -1;
            }
        }
      else if ((s->mode & SFM_WRITE))
        {
          if (s->ofd < 0)
            {
              errno = EBADF;
              return -1;
            }

          errno = 0;
          s->desc[SFD_OUTPUT].file = fdopen(s->ofd, "w");
          if (s->desc[SFD_OUTPUT].file == NULL)
            {
              s->status = errno;
              return -1;
            }
        }

      if ((s->mode & SFM_READ))
        {
          if (s->ifd < 0)
            {
              errno = EBADF;
              return -1;
            }

          errno = 0;
          if (s->ifd == s->ofd)
            s->desc[SFD_INPUT].file = fdopen(dup(s->ifd), "r");
          else
            s->desc[SFD_INPUT].file = fdopen(s->ifd, "r");

          if (s->desc[SFD_INPUT].file == NULL)
            {
              if (s->desc[SFD_OUTPUT].file != NULL)
                {
                  fclose(s->desc[SFD_OUTPUT].file);
                  s->desc[SFD_OUTPUT].file = NULL;
                }
              s->status = errno;
              return -1;
            }
        }
      s->is_opened = 1;
      return 0;
    }

  if (s->path != NULL)
    {
      /* we're given path */
      errno = 0;
      switch (s->mode)
        {
        case SFM_READ:
          s->desc[SFD_INPUT].file = fopen(s->path, "r");
          if (s->desc[SFD_INPUT].file == NULL)
            {
              s->status = errno;
              return -1;
            }
        case SFM_WRITE:
          s->desc[SFD_OUTPUT].file = fopen(s->path, "w");
          if (s->desc[SFD_OUTPUT].file == NULL)
            {
              s->status = errno;
              return -1;
            }
        case SFM_APPEND:
          s->desc[SFD_OUTPUT].file = fopen(s->path, "a");
          if (s->desc[SFD_OUTPUT].file == NULL)
            {
              s->status = errno;
              return -1;
            }
        default:
          SWERR(("s->mode == %d", s->mode));
        }
      s->is_opened = 1;
      return 0;
    }

  if (s->port >= 0)
    {
      int inlen;

      /* we're given port number to listen on */
      errno = 0;

      /* create a socket */
      s->listen_fd = socket(PF_INET, SOCK_STREAM, 0);
      if (s->listen_fd < 0)
        {
          s->status = errno;
          return -1;
        }

      /* should we enable REUSEADDR */
      if (s->reuseaddr)
        {
          int val = 1;

          errno = 0;
          if (setsockopt(s->listen_fd,
                         SOL_SOCKET, SO_REUSEADDR,
                         (char*) &val, sizeof(val)) < 0)
            {
              s->status = errno;
              close(s->listen_fd);
              return -1;
            }
        }

      /* bind the socket */
      s->saddr.sin_family = AF_INET;
      s->saddr.sin_addr.s_addr = INADDR_ANY;
      s->saddr.sin_port = htons(s->port);
      if (bind(s->listen_fd,
               (struct sockaddr*) &s->saddr,
               sizeof(s->saddr)) < 0)
        {
          s->status = errno;
          close(s->listen_fd);
          return -1;
        }

      /* register the socket to listen */
      if (listen(s->listen_fd, 1))
        {
          s->status = errno;
          close(s->listen_fd);
          return -1;
        }

      /* wait for a peer to connect */
      inlen = sizeof(s->inaddr);
      s->ifd = accept(s->listen_fd, (struct sockaddr*) &s->inaddr, &inlen);
      if (s->ifd < 0)
        {
          s->status = errno;
          close(s->listen_fd);
          return -1;
        }
      s->ofd = s->ifd;

      ASSERT(inlen == sizeof(s->inaddr));

      /* now we have a valid file descriptor */
      return strm_Open(s);
    }

  SWERR(("stream is not set up"));
#endif
  NOT_IMPLEMENTED();
}

/**
 * NAME:    strm_Close
 * PURPOSE: close the stream
 * ARGS:    s - stream
 * RETURN:  NULL
 */
  tpStream
strm_Close(tpStream s)
{
  /*
  ASSERT(s != NULL);
  ASSERT(s->is_opened);

  errno = 0;

  if (s->desc[SFD_INPUT].file != NULL)
    {
      fclose(s->desc[SFD_INPUT].file);
      s->desc[SFD_INPUT].file = NULL;
    }
  if (s->desc[SFD_OUTPUT].file != NULL)
    {
      fclose(s->desc[SFD_OUTPUT].file);
      s->desc[SFD_OUTPUT].file = NULL;
    }

  if (s->desc[SFD_INPUT].logfile)
    {
      fclose(s->desc[SFD_INPUT].logfile);
      s->desc[SFD_INPUT].logfile = NULL;
    }
  if (s->desc[SFD_OUTPUT].logfile != NULL)
    {
      fclose(s->desc[SFD_OUTPUT].logfile);
      s->desc[SFD_OUTPUT].logfile = NULL;
    }

  xfree(s->path);
  xfree(s);

  xfree(s->desc[SFD_INPUT].logname);
  xfree(s->desc[SFD_OUTPUT].logname);

  return NULL;
  */
  NOT_IMPLEMENTED();
}

/**
 * NAME:    strm_Flush
 * PURPOSE: flush the output buffer
 * ARGS:    s - stream
 * RETURN:  0 - ok, <0 - error
 */
  int
strm_Flush(tpStream s)
{
  /*
  ASSERT(s != NULL);
  ASSERT(s->is_opened);

  if (s->status != 0)
    return -1;

  if (s->desc[SFD_OUTPUT].file == NULL)
    {
      s->status = EINVAL;
      return -1;
    }

  if (s->desc[SFD_OUTPUT].logfile)
    fflush(s->desc[SFD_OUTPUT].logfile);

  errno = 0;
  if (fflush(s->desc[SFD_OUTPUT].file) < 0)
    {
      s->status = errno;
      return -1;
    }
  return 0;
  */
  NOT_IMPLEMENTED();
}

/**
 * NAME:    strm_Recover
 * PURPOSE: reset stream error status
 * ARGS:    s - stream
 */
  void
strm_Recover(tpStream s)
{
  /*
  ASSERT(s != NULL);
  ASSERT(s->is_opened);

  s->status = 0;
  */
  NOT_IMPLEMENTED();
}

/**
 * NAME:    strm_Message
 * PURPOSE: get error message string
 * ARGS:    s - stream
 * RETURN:  error message string
 */
  char *
strm_Message(tpStream s)
{
  /*
  ASSERT(s != NULL);

  return os_GetErrorString(s->status);
  */
  NOT_IMPLEMENTED();
}

/**
 * NAME:    strm_Status
 * PURPOSE: get stream status
 * ARGS:    s - stream
 * RETURN:  stream status
 */
  int
strm_Status(tpStream s)
{
  /*
  ASSERT(s != NULL);
  ASSERT(s->is_opened);

  return s->status;
  */
  NOT_IMPLEMENTED();
}

/**
 * NAME:    strm_IsEOF
 * PURPOSE: check EOF condition of the stream
 * ARGS:    s - stream
 * RETURN:  -1 - error, 0 - not EOF, 1 - EOF
 */
  int
strm_IsEOF(tpStream s)
{
  /*
  ASSERT(s != NULL);
  ASSERT(s->is_opened);

  if (s->status)
    return -1;

  if (s->desc[SFD_INPUT].file == NULL)
    {
      s->status = EINVAL;
      return -1;
    }

  return feof(s->desc[SFD_INPUT].file);
  */
  NOT_IMPLEMENTED();
}

/* =================== Read =================== */
/**
 * NAME:    strm_GetChar
 * PURPOSE: get a char from the stream
 * ARGS:    s - stream
 * RETURN:  -1 - error, >=0 - char read
 */
  int
strm_GetChar(tpStream s)
{
  /*
  int r;

  ASSERT(s != NULL);
  ASSERT(s->is_opened);

  if (s->status != 0)
    return -1;

  if (s->desc[SFD_INPUT].file == NULL)
    {
      s->status = EINVAL;
      return -1;
    }

  r = fgetc(s->desc[SFD_INPUT].file);
  if (r == -1)
    {
      s->status = errno;
      return -1;
    }
  return r;
  */
  NOT_IMPLEMENTED();
}

/**
 * NAME:    strm_GetShort
 * PURPOSE: get short (16 bit) from the stream
 * ARGS:    s - stream
 *          p - pointer to store short value
 * RETURN:  <0 - error, >= 0 - ok
 * NOTE:    the read value converted from network format to host format
 */
  int
strm_GetShort(tpStream s, short *p)
{
  /*
  short b = 0;
  int   r;

  ASSERT(s != NULL);
  ASSERT(s->is_opened);

  if (s->status)
    return -1;

  if (s->desc[SFD_INPUT].file == NULL)
    {
      s->status = EINVAL;
      return -1;
    }

  errno = 0;
  r = fread(&b, sizeof(short), 1, s->desc[SFD_INPUT].file);
  if (r < 0)
    {
      s->status = errno;
      return -1;
    }
  *p = ntohs(b);
  return r;
  */
  NOT_IMPLEMENTED();
}

/**
 * NAME:    strm_GetInt
 * PURPOSE: read int value (32 bit) from the stream
 * ARGS:    s - stream
 *          p - pointer to store the read value
 * RETURN:  <0 - error, >= 0 - ok
 * NOTE:    the read value is converted from network format to host format
 */
  int
strm_GetInt(tpStream s, int *p)
{
  /*
  int   b = 0;
  int   r;

  ASSERT(s != NULL);
  ASSERT(s->is_opened);

  if (s->status)
    return -1;

  if (s->desc[SFD_INPUT].file == NULL)
    {
      s->status = EINVAL;
      return -1;
    }

  errno = 0;
  r = fread(&b, sizeof(int), 1, s->desc[SFD_INPUT].file);
  if (r < 0)
    {
      s->status = errno;
      return -1;
    }
  *p = ntohl(b);
  return r;
  */
  NOT_IMPLEMENTED();
}

/**
 * NAME:    strm_GetLong
 * PURPOSE: read a long value (64 bit) from the stream
 * ARGS:    s - stream
 *          p - pointer to store long value
 * RETURN:  0 - ok, <0 - error
 * NOTE:    the read value is converted from network to host format
 *          not implemented
 */
  int
strm_GetLong(tpStream s, long *p)
{
  SWERR(("not implemented yet"));
}

/**
 * NAME:    strm_GetString
 * PURPOSE: read string from the stream
 * ARGS:    s      - stream
 *          buf    - buffer to read to
 *          maxlen - buffer length
 * RETURN:  <0 - error, 0 - ok
 * NOTE:    maximum number of characters to be read: maxlen - 1
 */
  int
strm_GetString(tpStream s, char *buf, int maxlen)
{
  /*
  ASSERT(s != NULL);
  ASSERT(s->is_opened);
  ASSERT(buf != NULL);
  ASSERT(maxlen >= 0);

  if (s->status)
    return -1;

  if (s->desc[SFD_INPUT].file == NULL)
    {
      s->status = EINVAL;
      return -1;
    }
  errno = 0;
  if (fgets(buf, maxlen, s->desc[SFD_INPUT].file) == NULL)
    {
      s->status = errno;
      return -1;
    }
  return 0;
  */
  NOT_IMPLEMENTED();
}

/**
 * NAME:    strm_VScanf
 * PURPOSE: vscanf for stream
 * ARGS:    s      - stream
 *          format - format string
 *          args   - extra format specific args
 * RETURN:  <0 - error, >=0 - number of fields read
 * NOTE:    not implemented
 */
  int
strm_VScanf(tpStream s, char *format, va_list args)
{
  SWERR(("not implemented"));
}

/**
 * NAME:    strm_Scanf
 * PURPOSE: vscanf for stream
 * ARGS:    s      - stream
 *          format - format string
 *          ...    - extra format specific args
 * RETURN:  <0 - error, >=0 - number of fields read
 * NOTE:    not implemented
 */
  int
strm_Scanf(tpStream s, char *format, ...)
{
  SWERR(("not implemented"));
}

/**
 * NAME:    strm_Read
 * PURPOSE: read from the stream
 * ARGS:    s    - stream
 *          p    - pointer to the buffer to read to
 *          size - number of bytes to read
 * RETURN:  <0 - error, >=0 - number of bytes read
 */
  int
strm_Read(tpStream s, void *p, int size)
{
  /*
  int r;

  ASSERT(s != NULL);
  ASSERT(p != NULL);
  ASSERT(size >= 0);
  ASSERT(s->is_opened);

  if (s->status)
    return -1;
  if (s->desc[SFD_INPUT].file == NULL)
    {
      s->status = EINVAL;
      return -1;
    }
  errno = 0;
  r = fread(p, 1, size, s->desc[SFD_INPUT].file);
  if (r < 0)
    {
      s->status = errno;
      return -1;
    }
  return r;
  */
  NOT_IMPLEMENTED();
}

/**
 * NAME:    strm_Ungetc
 * PURPOSE: put back the last read character
 * ARGS:    s - stream
 *          c - character to put back
 * RETURN:  <0 - error, 0 - ok
 */
  int
strm_Ungetc(tpStream s, int c)
{
  /*
  ASSERT(s != NULL);
  ASSERT(s->is_opened);
  if (s->status)
    return -1;
  if (s->desc[SFD_INPUT].file == NULL)
    {
      s->status = EINVAL;
      return -1;
    }
  errno = 0;
  if (ungetc(c, s->desc[SFD_INPUT].file) < 0)
    {
      s->status = errno;
      return -1;
    }
  return 0;
  */
  NOT_IMPLEMENTED();
}

/**
 * NAME:    strm_Get3Bytes
 * PURPOSE: get 3 bytes from the stream
 * ARGS:    s - stream
 *          l - pointer to store 3 bytes in the long value
 * RETURN:  <0 - error, 0 - ok
 * NOTE:    not implemented
 */
  int
strm_Get3Bytes(tpStream s, long *l)
{
  SWERR(("not implemented"));
}

/**
 * NAME:    strm_GetFloat
 * PURPOSE: get float number (32 bits)
 * ARGS:    s - stream
 *          f - pointer to store the float value
 * RETURN:  <0 - error, >=0 - ok
 * NOTE:    not implemented
 */
  int
strm_GetFloat(tpStream s, float *f)
{
  SWERR(("not implemented"));
}

/**
 * NAME:    strm_GetDouble
 * PURPOSE: get double number (64 bits)
 * ARGS:    s - stream
 *          d - pointer to store the double value
 * RETURN:  <0 - error, >=0 - ok
 * NOTE:    not implemented
 */
  int
strm_GetDouble(tpStream s, double *d)
{
  SWERR(("not implemented"));
}

/**
 * NAME:    strm_GetLongDouble
 * PURPOSE: get long double number (128 bits?)
 * ARGS:    s  - stream
 *          ld - pointer to store the long double value
 * RETURN:  <0 - error, >=0 - ok
 * NOTE:    not implemented
 */
  int
strm_GetLongDouble(tpStream s, long double *ld)
{
  SWERR(("not implemented"));
}

/* =================== Write ================== */
/**
 * NAME:    strm_Write
 * PURPOSE: write data to the stream
 * ARGS:    s    - stream
 *          p    - pointer to the data to write
 *          size - size of the data
 * RETURN:  <0 - error, >= 0 - number of bytes actually written
 */
  int
strm_Write(tpStream s, void *p, int size)
{
  /*
  int r;

  ASSERT(s != NULL);
  ASSERT(p != NULL);
  ASSERT(size >= 0);
  ASSERT(s->is_opened);

  if (s->status)
    return -1;
  if (s->desc[SFD_OUTPUT].file == NULL)
    {
      s->status = EINVAL;
      return -1;
    }

  if (s->desc[SFD_OUTPUT].logfile)
    fwrite(p, 1, size, s->desc[SFD_OUTPUT].logfile);

  errno = 0;
  r = fwrite(p, 1, size, s->desc[SFD_OUTPUT].file);
  if (r < 0)
    {
      s->status = errno;
      return -1;
    }
  return r;
  */
  NOT_IMPLEMENTED();
}

/**
 * NAME:    strm_PutChar
 * PURPOSE: write a character to the stream
 * ARGS:    s - stream
 *          c - character
 * RETURN:  0 - ok, -1 - error
 */
  int
strm_PutChar(tpStream s, int c)
{
  /*
  ASSERT(s != NULL);
  ASSERT(s->is_opened);

  if (s->status)
    return -1;
  if (s->desc[SFD_OUTPUT].file == NULL)
    {
      s->status = EINVAL;
      return -1;
    }

  if (s->desc[SFD_OUTPUT].logfile)
    fputc(c, s->desc[SFD_OUTPUT].logfile);

  errno = 0;
  if (fputc(c, s->desc[SFD_OUTPUT].file) < 0)
    {
      s->status = errno;
      return -1;
    }
  return 0;
  */
  NOT_IMPLEMENTED();
}

/**
 * NAME:    strm_PutShort
 * PURPOSE: write a short value (16 bits) to the stream
 * ARGS:    s   - stream
 *          val - short value
 * RETURN:  >= 0 - ok, <0 - error
 * NOTE:    the value is converted from host to network format
 */
  int
strm_PutShort(tpStream s, short val)
{
  /*
  short t;
  int   r;

  ASSERT(s != NULL);
  ASSERT(s->is_opened);

  if (s->status)
    return -1;
  if (s->desc[SFD_OUTPUT].file == NULL)
    {
      s->status = EINVAL;
      return -1;
    }
  
  t = htons(val);

  if (s->desc[SFD_OUTPUT].logfile)
    fwrite(&t, 1, 2, s->desc[SFD_OUTPUT].logfile);

  errno = 0;
  if ((r = fwrite(&t, 1, 2, s->desc[SFD_OUTPUT].file)) < 2)
    {
      if (r >= 0)
        return 0;
      s->status = errno;
      return -1;
    }
  return 1;
  */
  NOT_IMPLEMENTED();
}

/**
 * NAME:    strm_Put3Bytes
 * PURPOSE: write 3 bytes to the stream
 * ARGS:    s   - stream
 *          val - value
 * RETURN:  <0 - error, >= 0 - ok
 * NOTE:    value converted from host to network format
 *          the function is used to send color specifications to client
 */
  int
strm_Put3Bytes(tpStream s, long val)
{
  /*
  int  t;
  int  r;

  ASSERT(s != NULL);
  ASSERT(s->is_opened);

  if (s->status)
    return -1;
  if (s->desc[SFD_OUTPUT].file == NULL)
    {
      s->status = EINVAL;
      return -1;
    }
  
  t = htonl(val & 0x00FFFFFF);

  if (s->desc[SFD_OUTPUT].logfile)
    fwrite((char*) &t + 1, 1, 3, s->desc[SFD_OUTPUT].logfile);

  errno = 0;
  if ((r = fwrite((char*) &t + 1, 1, 3, s->desc[SFD_OUTPUT].file)) < 3)
    {
      if (r >= 0)
        return 0;
      s->status = errno;
      return -1;
    }
  return 1;
  */
  NOT_IMPLEMENTED();
}

/**
 * NAME:    strm_PutInt
 * PURPOSE: write int value (32 bits) to the stream
 * ARGS:    s   - stream
 *          val - value to write
 * RETURN:  <0 - error. >=0 - ok
 * NOTE:    value is converted from host to network format
 */
  int
strm_PutInt(tpStream s, int val)
{
  /*
  int t;
  int r;

  ASSERT(s != NULL);
  ASSERT(s->is_opened);

  if (s->status)
    return -1;
  if (s->desc[SFD_OUTPUT].file == NULL)
    {
      s->status = EINVAL;
      return -1;
    }

  t = htonl(val);
  
  if (s->desc[SFD_OUTPUT].logfile)
    fwrite(&t, 1, 4, s->desc[SFD_OUTPUT].logfile);

  errno = 0;
  if ((r = fwrite(&t, 1, 4, s->desc[SFD_OUTPUT].file)) < 4)
    {
      if (r >= 0)
        return 0;
      s->status = errno;
      return -1;
    }
  return 1;
  */
  NOT_IMPLEMENTED();
}

/**
 * NAME:    strm_PutLong
 * PURPOSE: write long value (64 bits) to the stream
 * ARGS:    s - stream
 *          l - value to write to the stream
 * RETURN:  <0 - error, >= 0 - ok
 * NOTE:    value is converted from host to network format
 *          function is not implemented yet
 */
  int
strm_PutLong(tpStream s, long l)
{
  SWERR(("not implemented yet"));
}

/**
 * NAME:    strm_PutString
 * PURPOSE: write a string to the stream
 * ARGS:    s   - stream
 *          str - string to read
 * RETURN:  <0 - error, >= 0 - ok
 * NOTE:    not implemented
 */
  int
strm_PutString(tpStream s, char *str)
{
  SWERR(("not implemented yet"));
}

/**
 * NAME:    strm_VPrintf
 * PURPOSE: vprintf for the stream
 * ARGS:    s - stream
 *          f - format string
 *          l - extra format specific args
 * RETURN:  <0 - error, >=0 - number of chars actually written
 */ 
 int
strm_VPrintf(tpStream s, char *f, va_list l)
{
  /*
  int r;

  ASSERT(s != NULL);
  ASSERT(s->is_opened);
  ASSERT(f != NULL);

  if (s->status)
    return -1;
  if (s->desc[SFD_OUTPUT].file == NULL)
    {
      s->status = EINVAL;
      return -1;
    }

  if (s->desc[SFD_OUTPUT].logfile)
    vfprintf(s->desc[SFD_OUTPUT].logfile, f, l);

  errno = 0;
  if ((r = vfprintf(s->desc[SFD_OUTPUT].file, f, l)) < 0)
    {
      s->status = errno;
      return -1;
    }
  return r;
  */
  NOT_IMPLEMENTED();
}

/**
 * NAME:    strm_Printf
 * PURPOSE: printf for the stream
 * ARGS:    s   - stream
 *          f   - format string
 *          ... - extra format specific args
 * RETURN:  <0 - error, >=0 - number of chars actually written
 */
  int
strm_Printf(tpStream s, char *f, ...)
{
  /*
  int     r;
  va_list l;

  ASSERT(s != NULL);
  ASSERT(s->is_opened);
  ASSERT(f != NULL);

  if (s->status)
    return -1;
  if (s->desc[SFD_OUTPUT].file == NULL)
    {
      s->status = EINVAL;
      return -1;
    }
  va_start(l, f);

  if (s->desc[SFD_OUTPUT].logfile)
    vfprintf(s->desc[SFD_OUTPUT].logfile, f, l);

  errno = 0;
  r = vfprintf(s->desc[SFD_OUTPUT].file, f, l);
  va_end(l);
  if (r < 0)
    {
      s->status = errno;
      return -1;
    }
  return r;
  */
  NOT_IMPLEMENTED();
}

/**
 * NAME:    strm_PutFloat
 * PURPOSE: write float value (32 bits) to the stream
 * ARGS:    s - stream
 *          f - value to write
 * RETURN:  <0 - error, >=0 - ok
 * NOTE:    function is not implemented
 */
  int
strm_PutFloat(tpStream s, float f)
{
  SWERR(("not implemented"));
}

/**
 * NAME:    strm_PutDouble
 * PURPOSE: write double value (64 bits) to the stream
 * ARGS:    s - stream
 *          d - value to write
 * RETURN:  <0 - error, >=0 - ok
 * NOTE:    function is not implemented
 */
  int
strm_PutDouble(tpStream s, double d)
{
  SWERR(("not implemented"));
}

/**
 * NAME:    strm_PutLongDouble
 * PURPOSE: write long double value (128 bits?) to the stream
 * ARGS:    s  - stream
 *          ld - value to write
 * RETURN:  <0 - error, >=0 - ok
 * NOTE:    function is not implemented
 */
  int
strm_PutLongDouble(tpStream s, long double ld)
{
  SWERR(("not implemented"));
}

/* === Do not add stuff after this line === */

/*
 * Local variables:
 *  compile-command: "make -C .."
 *  c-font-lock-extra-types: ("FILE" "\\sw+_t" "tStream" "tpStream" "va_list")
 * End:
 */
