/* -*- mode:c -*- */
/* $Id: fsread.c 354 2004-07-17 06:28:14Z cher $ */

/* Copyright (C) 2002-2004 Alexander Chernov <cher@ispras.ru> */

/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 */

#define __REUSE__ 1

/* internal include directives */
#include "lconfig.h"
#include "reuse/config.h"
#include "embed_version.h"

/* reuse include directives */
#include <reuse/FlexString.h>
#include <reuse/ReuseDefs.h>

#include <assert.h>
#include <ctype.h>

#define CLUSTER           32
#define ALIGN(size)       ((size+31) & 0xffffffe0)

#ifdef REUSE_DEBUG
#  define CHECK(ptr)        assert((ptr) != NULL)
#  define CHECKPFS(pfs)     assert((pfs)->Used == 0 || (pfs)->Used < (pfs)->Allocd)
#else
#  define CHECK(p)
#  define CHECKPFS(p)
#endif /* REUSE_DEBUG */

void
_fsRead(tFString *pfs,FILE *f)
{
  int iCh;
  
  CHECKPFS(pfs);
  CHECK(f);
  
  fsClear(*pfs);
  for(iCh = getc(f); iCh != EOF && isspace(iCh); iCh = getc(f))
    /* EMPTY */;
  
  assert(iCh == '\'');
  
  iCh = getc(f);
  while(1)
    {
      if(iCh == '\'')
        {
          iCh = getc(f);
          if(iCh != '\'')
            break;
        }
      
      fsAdd(*pfs,(char) iCh);
      iCh = getc(f);
    }
}

#if REUSE_COMPILE_CVSID - 0 != 0
static const unsigned char cvsid[] __attribute__((unused)) =
"$Id: fsread.c 354 2004-07-17 06:28:14Z cher $";
#endif /* REUSE_COMPILE_CVSID */

/*
 * Local variables:
 *  compile-command: "make"
 *  c-font-lock-extra-types: ("FILE" "\\sw+_t" "tFString")
 * End:
 */
