/* $Id: BalTree.h 315 2003-08-01 19:55:42Z cher $ */

#ifndef __REUSE_BALTREE_H__
#define __REUSE_BALTREE_H__

/* Copyright (C) 1996-2003 Alexander Chernov <cher@ispras.ru> */
/* Created: Tue Oct 29 15:39:01 1996 by cher (Alexander Chernov) */

/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 */

#include <reuse/ReuseDefs.h>

#define BT_ORD_PREORDER  1
#define BT_ORD_INORDER   2
#define BT_ORD_POSTORDER 4

/* Some macros for well known predefined types */
#define bt_int_cmp(a,b)   ((a) - (b))
#define bt_int_copy(s,d)  d = s
#define bt_int_move(s,d)  d = s
#define bt_int_drop(s)
#define bt_int_eq(a,b)    ((a) == (b))
#define bt_int_lt(a,b)    ((a) < (b))
#define bt_int_gt(a,b)    ((a) > (b))

#define bt_tIdent_cmp(a,b)     ((a) - (b))
#define bt_tIdent_copy(s,d)    d = s
#define bt_tIdent_move(s,d)    d = s
#define bt_tIdent_drop(s)
#define bt_tIdent_eq(a,b)      ((a) == (b))
#define bt_tIdent_lt(a,b)      ((a) <  (b))
#define bt_tIdent_gt(a,b)      ((a) >  (b))

#ifndef BT_NO_BALANCE_PROTO
/* This conditional compilation is used to prevent definition
 * of such prototype in implementation file, where such
 * prototype is invalid
 */
/* Note, that bt_balance operates on generic tree structure
 * (left and right pointers and tree height)
 * without any knowlegde about key and data field
 */
#  ifdef __cplusplus
extern "C" R_DLL void *bt_balance(void *);
#  else
extern R_DLL void *bt_balance(void *);
#  endif
#else
struct tree_t;
#  ifdef __cplusplus
extern "C" R_DLL struct tree_t * bt_balance(struct tree_t *p);
#  else
extern R_DLL struct tree_t * bt_balance(struct tree_t *p);
#  endif
#endif

#define DEF_BT(tname, key_t, data_t) \
typedef struct tname##_s { \
  struct tname##_s *left, *right; \
  int height; \
  key_t        key; \
  data_t       data; \
} tname##_s; \
typedef struct tname##_s * tname

#define DEF_BT_DROP_PROTO(tname, key_t, data_t) \
  tname tname##_drop(tname)

#define DEF_BT_INSERT_PROTO(tname, key_t, data_t) \
  tname tname##_insert(tname, key_t, data_t)

#define DEF_BT_FIND_PROTO(tname, key_t, data_t) \
  data_t *tname##_find(tname, key_t)

#define DEF_BT_DELETE_PROTO(tname, key_t, data_t) \
  tname tname##_delete(tname, key_t)

#define DEF_BT_INIT_PROTO(tname, key_t, data_t) \
  tname tname##_init(void)

#define DEF_BT_COPY_PROTO(tname, key_t, data_t) \
  tname tname##_copy(tname)

#define DEF_BT_TRAVERSE_PROTO(tname, key_t, data_t) \
  void tname##_traverse(tname, int, void (*)(key_t *, data_t *, void*), void*)

#define DEF_BT_PROTOS(tname, key_t, data_t,sspec) \
  sspec DEF_BT_DROP_PROTO(tname, key_t, data_t); \
  sspec DEF_BT_INSERT_PROTO(tname, key_t, data_t); \
  sspec DEF_BT_FIND_PROTO(tname, key_t, data_t); \
  sspec DEF_BT_DELETE_PROTO(tname, key_t, data_t); \
  sspec DEF_BT_INIT_PROTO(tname,key_t,data_t); \
  sspec DEF_BT_COPY_PROTO(tname, key_t, data_t); \
  sspec DEF_BT_TRAVERSE_PROTO(tname, key_t, data_t)

#define DEF_BT_DROP_BODY(tname, key_t, data_t) \
tname tname##_drop(tname tree) { \
  if (tree == NULL) return NULL; \
  bt_##key_t##_drop(tree->key); \
  bt_##data_t##_drop(tree->data); \
  tname##_drop(tree->left); \
  tname##_drop(tree->right); \
  tname##_free(tree, sizeof(tname##_s)); \
  return NULL; \
}

#define DEF_BT_INIT_BODY(tname, key_t, data_t) \
tname tname##_init(void) { return NULL; }

#define DEF_BT_COPY_BODY(tname, key_t, data_t) \
tname tname##_copy(tname tree) { \
  tname nit; \
  if (tree == NULL) return NULL; \
  tname##_alloc(nit, sizeof(tname##_s)); \
  bt_##key_t##_copy(tree->key, nit->key); \
  bt_##data_t##_copy(tree->data, nit->data); \
  nit->left = tname##_copy(tree->left); \
  nit->right = tname##_copy(tree->right); \
  return nit; \
}

#if defined BT_USE_CMP
#define DEF_BT_FIND_BODY(tname, key_t, data_t) \
data_t *tname##_find(tname tree, key_t key) { \
  int cmpr;
  while (tree != NULL) { \
    cmpr = bt_##key_t##_cmp(tree->key, key); \
    if (cmpr == 0) return &tree->data; \
    else if (cmpr < 0) tree = tree->right; \
    else tree = tree->left; \
  } \
  return NULL; \
}
#else
#define DEF_BT_FIND_BODY(tname, key_t, data_t) \
data_t *tname##_find(tname tree, key_t key) { \
  while (tree != NULL) { \
    if (bt_##key_t##_eq(tree->key, key)) return &tree->data; \
    else if (bt_##key_t##_lt(tree->key, key)) tree = tree->right; \
    else tree = tree->left; \
  } \
  return NULL; \
}
#endif

#if defined BT_USE_CMP
#define DEF_BT_INSERT_BODY(tname, key_t, data_t) \
tname tname##_insert(tname tree, key_t key, data_t data) {\
  int cmpr; \
  if (tree == NULL) { \
    tname##_alloc(tree, sizeof(tname##_s)); \
    tree->left = NULL; \
    tree->right = NULL; \
    tree->height = 1; \
    bt_##key_t##_move(key, tree->key); \
    bt_##data_t##_move(data, tree->data); \
    return tree; \
  } \
  cmpr = bt_##key_t##_cmp(tree->key, key); \
  if (cmpr == 0) { \
    bt_##data_t##_drop(tree->data); \
    bt_##data_t##_move(data, tree->data); \
  } else if (cmpr < 0) { \
    tree->right = tname##_insert(tree->right, key, data); \
    tree = (tname) bt_balance(tree); \
  } else { \
    tree->left = tname##_insert(tree->left, key, data); \
    tree = (tname) bt_balance(tree); \
  } \
  return tree; \
}
#else
#define DEF_BT_INSERT_BODY(tname, key_t, data_t) \
tname tname##_insert(tname tree, key_t key, data_t data) {\
  if (tree == NULL) { \
    tname##_alloc(tree, sizeof(tname##_s)); \
    tree->left = NULL; \
    tree->right = NULL; \
    tree->height = 1; \
    bt_##key_t##_move(key, tree->key); \
    bt_##data_t##_move(data, tree->data); \
    return tree; \
  } \
  if (bt_##key_t##_eq(tree->key, key)) { \
    bt_##data_t##_drop(tree->data); \
    bt_##data_t##_move(data, tree->data); \
  } else if (bt_##key_t##_lt(tree->key,key)) { \
    tree->right = tname##_insert(tree->right, key, data); \
    tree = (tname) bt_balance(tree); \
  } else { \
    tree->left = tname##_insert(tree->left, key, data); \
    tree = (tname) bt_balance(tree); \
  } \
  return tree; \
}
#endif

#if defined BT_USE_CMP
#define DEF_BT_DELETE_BODY(tname, key_t, data_t) \
tname tname##_delete(tname tree, key_t key) {                   \
  tname p1;                                                     \
  int   cmpr;                                                   \
  if (tree == NULL) return tree;                                \
  cmpr = bt_##key_t##_cmp(tree->key, key);                      \
  if (cmpr == 0) {                                              \
    if (tree->left == NULL || tree->right == NULL) {            \
      if (tree->left == NULL) p1 = tree->right;                 \
      else p1 = tree->left;                                     \
      bt_##key_t##_drop(tree->key);                             \
      bt_##data_t##_drop(tree->data);                           \
      tname##_free(tree, sizeof(tname##_s));                    \
      return p1;                                                \
    }                                                           \
    for (p1 = tree->left; p1->right != NULL; p1 = p1->right);   \
    bt_##key_t##_drop(tree->key);                               \
    bt_##data_t##_drop(tree->data);                             \
    bt_##key_t##_copy(p1->key, tree->key);                      \
    bt_##data_t##_copy(p1->data, tree->data);                   \
    tree->left = tname##_delete(tree->left, tree->key);         \
  } else if (cmpr > 0) {                                        \
    tree->left = tname##_delete(tree->left, key);               \
  } else {                                                      \
    tree->right = tname##_delete(tree->right, key);             \
  }                                                             \
  return (tname) bt_balance(tree);                              \
}
#else
#define DEF_BT_DELETE_BODY(tname, key_t, data_t) \
tname tname##_delete(tname tree, key_t key) {                   \
  tname p1;                                                     \
  if (tree == NULL) return tree;                                \
  if (bt_##key_t##_eq(tree->key, key)) {                        \
    if (tree->left == NULL || tree->right == NULL) {            \
      if (tree->left == NULL) p1 = tree->right;                 \
      else p1 = tree->left;                                     \
      bt_##key_t##_drop(tree->key);                             \
      bt_##data_t##_drop(tree->data);                           \
      tname##_free(tree, sizeof(tname##_s));                    \
      return p1;                                                \
    }                                                           \
    for (p1 = tree->left; p1->right != NULL; p1 = p1->right);   \
    bt_##key_t##_drop(tree->key);                               \
    bt_##data_t##_drop(tree->data);                             \
    bt_##key_t##_copy(p1->key, tree->key);                      \
    bt_##data_t##_copy(p1->data, tree->data);                   \
    tree->left = tname##_delete(tree->left, tree->key);         \
  } else if (bt_##key_t##_lt(key, tree->key)) {                 \
    tree->left = tname##_delete(tree->left, key);               \
  } else {                                                      \
    tree->right = tname##_delete(tree->right, key);             \
  }                                                             \
  return (tname) bt_balance(tree);                              \
}
#endif

#define DEF_BT_TRAVERSE_BODY(tname, key_t, data_t) \
void tname##_traverse(tname tree, int ord, void(*func)(key_t *, data_t *, void *), void *usr){ \
  if (tree == NULL) return; \
  if (ord & BT_ORD_PREORDER)  (*func)(&tree->key, &tree->data, usr); \
  tname##_traverse(tree->left, ord, func, usr); \
  if (ord & BT_ORD_INORDER)   (*func)(&tree->key, &tree->data, usr); \
  tname##_traverse(tree->right, ord, func, usr); \
  if (ord & BT_ORD_POSTORDER) (*func)(&tree->key, &tree->data, usr); \
}

#define DEF_BT_BODIES(tname, key_t, data_t, sspec) \
  sspec DEF_BT_DROP_BODY(tname, key_t, data_t); \
  sspec DEF_BT_INSERT_BODY(tname, key_t, data_t); \
  sspec DEF_BT_FIND_BODY(tname, key_t, data_t); \
  sspec DEF_BT_DELETE_BODY(tname, key_t, data_t); \
  sspec DEF_BT_INIT_BODY(tname,key_t,data_t); \
  sspec DEF_BT_COPY_BODY(tname, key_t, data_t); \
  sspec DEF_BT_TRAVERSE_BODY(tname, key_t, data_t)

#endif /* __REUSE_BALTREE_H__ */

/*
 * Local variables:
 *  compile-command: "make -C .."
 *  c-font-lock-extra-types: ("\\sw+_t" "FILE" "R_DLL")
 * End:
 */
