/* $Id: FlexArray.h 352 2004-07-17 06:11:56Z cher $ */

#ifndef __REUSE_FLEXARRAY_H__
#define __REUSE_FLEXARRAY_H__

/* Copyright (C) 1995-2004 Alexander Chernov <cher@ispras.ru> */
/* Created: <95/12/03 13:25:04 cher> */

/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 */

#include <reuse/ReuseDefs.h>

typedef void (*prc_assign_t) (void *to, void *from);
/* Assignment function -- data structure at address `to'
   considered to be initialized */

typedef void (*prc_copy_t) (void *to, void *from);
/* copy function -- data structure at `to' contains `garbage' */

typedef void (*prc_destroy_t) (void *ptr);
/* destroys data */

typedef void (*prc_compare_t) (void *ptr_1, void *ptr_2);
/* compares two entries */

typedef struct fa_virtual_table
{
  prc_assign_t   prc_assign;
  prc_copy_t     prc_copy;
  prc_destroy_t  prc_destroy;
  prc_compare_t  prc_compare;
} fa_virtual_table;

struct FA_INTERNAL;

#define fa_create_type(type,name,assign_proc,copy_proc,destroy_proc,cmp_proc) \
static fa_virtual_table _##name##_vt = { \
(prc_assign_t) assign_proc, (prc_copy_t) copy_proc, \
(prc_destroy_t) destroy_proc, (prc_compare_t) cmp_proc }; \
typedef (type) _##name##_ts; \
typedef struct name { \
fa_virtual_table *Virtual; \
size_t           Size; \
(type)           *Data; \
uint             Used; \
uint             Allocd; \
type             Dummy; \
} name;

#define faInsert(arr,index,elem) \
{ (arr).Dummy = elem; \
  _faInsert((struct FA_INTERNAL*) &(arr),index); }

#define faRemove(arr,index) _faRemove((struct FA_INTERNAL*) &(arr),index)

#define faArray(arr)  (arr).Data

#define faLength(arr) (arr).Used

#define fa_static_instance(type_name,inst_name) \
static type_name inst_name = { & _##type_name##_vt, \
sizeof(_##type_name##_ts), NULL, 0, 0 };

#define fa_instance(type_name,inst_name) \
type_name inst_name = { & _##type_name##_vt, \
sizeof(_##type_name##_ts), NULL, 0, 0 };

#define fa_extern_instance(type_name,inst_name) \
extern type_name inst_name;

#define fa_local_instance(type_name,inst_name) \
type_name inst_name;

#define fa_local_init(type_name,inst) \
{ (inst).Virtual = _##type_name##_vt; \
  (inst).Size = sizeof(_##type_name##_ts); \
  (arr).Data = NULL; (arr).Used = 0; (arr).Allocd = 0; }

#define faClear(arr)   _faClear(&(arr))
#define faDestroy(arr) _faDestroy(&(arr))

#if defined __cplusplus
extern "C" {
#endif /* __cplusplus */

  R_DLL void _faRemove(struct FA_INTERNAL *,int index);
  R_DLL void _faInsert(struct FA_INTERNAL*, int);
  R_DLL void _faClear(struct FA_INTERNAL *);
  R_DLL void _faDestroy(struct FA_INTERNAL *);

#if defined __cplusplus
}
#endif /* __cplusplus */

#endif /* __REUSE_FLEXARRAY_H__ */

/*
 * Local variables:
 *  compile-command: "make -C ../.."
 *  c-font-lock-extra-types: ("\\sw+_t" "FILE" "R_DLL")
 * End:
 */
