/* $Id: GenOut.h 352 2004-07-17 06:11:56Z cher $ */

#ifndef __REUSE_GENOUT_H__
#define __REUSE_GENOUT_H__

/* Copyright (C) 1995-2004 Alexander Chernov <cher@ispras.ru> */
/* Created: <95/11/09 16:23:36 cher> */

/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 */

#include <reuse/integral.h>
#include <reuse/ReuseDefs.h>

#include <stdio.h>
#include <stdarg.h>

#if defined __cplusplus
extern "C" {
#endif /* __cplusplus */

struct tGenOut;

typedef void (*tCharOutFunc)(struct tGenOut*,int);
typedef void (*tStringOutFunc)(struct tGenOut*,char*);
typedef void (*tFormatOutFunc)(struct tGenOut*,char *,...);
typedef void (*tVFormatOutFunc)(struct tGenOut*,char *,va_list);

typedef struct tGenOutVT
{
  tCharOutFunc    CharOut;
  tStringOutFunc  StringOut;
  tFormatOutFunc  FormatOut;
  tVFormatOutFunc VFormatOut;
} tGenOutVT;

typedef struct tGenOut
{
  tGenOutVT *Virtual;
} tGenOut;

/* This structure is a base (abstract) "class" for Generic Output.
 * Other classes such as tFileOut or tFixedOut inherits this structure.
 * The structure contains pointers to 4 functions which emulate virtual
 * methods in C++.
 */

typedef struct tFileOut
{
  tGenOut GenOutPart;
  FILE    *file;
} tFileOut;

/* Structure `tFileOut' is a derived class from `tGenOut'. This class
 * outputs data to a file denoted by C structure FILE.
 */

typedef struct tFixedOut
{
  tGenOut GenOutPart;
  char *String;
  ruint_t Used;
  ruint_t Size;
} tFixedOut;

/* Structure `tFixedOut' is a derived class from `tGenOut'. It outputs data
 * to a string of fixed length. If the string is overflown, output is
 * truncated.
 */

R_DLL void goInitModule(void);
/* Initializes internal data structures. This routine MUST be called before
 * use of functions described below
 */

R_DLL void goCloseModule(void);
/* Release memory used by internal structures. */

R_DLL void goStatistics(FILE *f);
/* Prints certain statistics. */

R_DLL tFileOut goFileOut(FILE *f);
/* Fills up the virtual methods table of structure of type `tFileOut'.
 * This function is equivalent to a constructor in C++ language
 */

R_DLL tFixedOut goFixedOut(char *strBuf,ruint_t size);
/* Fills up data fields (include pointers to functions -- virtual methods)
 * of structure of type `tFixedOut'. Pointer to character string and its
 * length is passed to this constructor. If this string is overflown, output
 * will be truncated.
 */

R_DLL void go_printf(tGenOut*,char *format,...);
/* this function emulates virtual method call. It calls function of
 * formatted output corresponding to the object passed to the function as
 * a parameter
 */

#define go_putc(go,c)      (*((go)->Virtual->CharOut))(go,c)
#define go_puts(go,s)      (*((go)->Virtual->StringOut))(go,s)
#define go_vprintf(go,f,a) (*((go)->Virtual->VFormatOut))(go,f,a)
/* These macros emulate virtual function call for output of character,
 * string and formatted output with parameter list as `va_list'.
 */

/* function `go_printf' and macros `go_puts', `go_puts' and `go_vprintf'
 * may be used in user routines which take a pointer to a Generic Output
 * structure as an argument
 */

#if defined __cplusplus
}
#endif /* __cplusplus */

#endif /* __REUSE_GENOUT_H__ */

/**
 * Local variables:
 *  compile-command: "make"
 *  c-font-lock-extra-types: ("FILE" "\\sw+_t" "Bool" "va_list" "tGenOutVT" "tGenOut" "tFileOut" "tFixedOut" "R_DLL")
 * End:
 */
