/* $Id: Idents.h 300 2002-06-15 22:00:00Z cher $ */

#ifndef __REUSE_IDENTS_H__
#define __REUSE_IDENTS_H__

/* Copyright (C) 1995-2002 Alexander Chernov <cher@ispras.ru> */
/* Ich, Doktor Josef Grosch, Informatiker, Sept. 1987 */

/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 */

#include <reuse/Defs.h>
#include <reuse/ReuseDefs.h>
#include <reuse/StringMem.h>
#include <reuse/FlexString.h>

#include <stdio.h>

#if defined __cplusplus
extern "C" {
#endif /* __cplusplus */

typedef Cardinal        tIdent;

/* default identifier */
enum { NoIdent = 1 };

extern  R_DLL tIdent  MakeIdent(char * string, Cardinal length);
/* The string (of length) is mapped to a unique */
/* identifier (an integer) which is returned.   */

extern  R_DLL void    GetString(tIdent ident, char * string);
/* Returns the string whose identifier is 'ident'.*/

extern  R_DLL tStringRef GetStringRef(tIdent ident);
/* Returns a reference to the string identified */
/* by 'ident'.                                  */

extern  R_DLL tIdent  MaxIdent(void);
/* Returns the currently maximal identifier.    */

extern  R_DLL void    WriteIdent(FILE * file, tIdent ident);
/* The string encoded by the identifier 'ident' */
/* is printed on the file.                      */

extern  R_DLL void    WriteIdents(FILE*);
/* The contents of the identifier table is      */
/* printed on the given file.           */

#define idInitModule() InitIdents()
extern  R_DLL void    InitIdents(void);
/* The identifier table is initialized.         */

extern R_DLL void     idCloseModule(void);
/* Frees internal data. */

extern R_DLL void     idClear(void);
/* Clears identifier table, but do not release memory */

extern R_DLL void     idStatistics(FILE *);
/* Prints some statistics. */

extern  R_DLL void    WriteHashTable(FILE *);

/* NEW FUNCTIONS */
extern  R_DLL char    *idGetString(tIdent ident);
/* Returns string corresponding to ident `ident'. This string is
   stored in static area. There may be problems for simultaneous use
   of this function (e. g. when two function arguments call it */

extern R_DLL char     *idDupString(tIdent ident);
/* Returns copy of a string corresponding to ident `ident'. Memory for
   string is allocated in heap using `strdup' function. Take care of
   freeing it with `free' call */

extern R_DLL tIdent    idMake(char *str);
/* Makes an identifier from null-teminated string `str' */

#define idMakeFS(fs) MakeIdent(fsString(fs),fsLength(fs))
/* Makes an identifier from flexible string */

extern R_DLL tIdent idReadIdent(FILE *);
extern R_DLL tIdent idGetIdent(FILE *);

/* Cocktail type-specific macros */
#define readtIdent(i)        i = idReadIdent(yyf);
#define writetIdent(i)       fsWriteStr(idGetString(i),yyf);
#define gettIdent(i)         i = idGetIdent(yyf);
#define puttIdent(i)         fsPutStr(idGetString(i),yyf);

#define begintIdent(i)       i = NoIdent;
#define closetIdent(i)       i = NoIdent;
#define copytIdent(d,s)      (d) = (s);
#define equaltIdent(i1,i2)   ((i1) == (i2))

#if defined __cplusplus
}
#endif /* __cplusplus */

# endif /* __REUSE_IDENTS_H__ */

/**
 * Local variables:
 *  compile-command: "make -C .."
 *  c-font-lock-extra-types: ("tIdent" "FILE" "\\sw+_t" "Bool" "Cardinal" "tStringRef" "R_DLL")
 * End:
 */
