/* $Id: RErrors.h 302 2002-06-15 22:02:24Z cher $ */

#ifndef __REUSE_RERRORS_H__
#define __REUSE_RERRORS_H__

/* Copyright (C) 1995-2002 Alexander Chernov <cher@ispras.ru> */
/* Ich, Doktor Josef Grosch, Informatiker, Juli 1992 */
/* Alexander Chernov, November 1995 */

/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 */

/* This file origins from file `Errors.h' from Reuse library
 * of Karlsruhe Compiler Toolkit (Cocktail).
 */

#include <stdio.h>
#include <stdarg.h>

#include <reuse/Defs.h>
#include <reuse/ReuseDefs.h>
#include <reuse/Positions.h>

/* error codes */
/* These error codes are used by scanners and parsers produced by
 * scanner and parser generating tools `lalr' `ell' and `rex'.
 */
  
#define erUnknown               -1

#define xxNoText                0
#define xxSyntaxError           1
#define xxExpectedTokens        2
#define xxRestartPoint          3
#define xxTokenInserted         4
#define xxTooManyErrors         5
  
/* total number of error codes */
#define xxErrorCodes            6

/* Error classes */
#define xxFatal                 1
#define xxRestriction           2
#define xxError                 3
#define xxWarning               4
#define xxRepair                5
#define xxNote                  6
#define xxInformation           7
#define xxAbort                 8

/* total number of error classes */
#define xxErrorClasses          9

/* Alternate names for error classes */
#define esFatal                 1
#define esError                 3
#define esWarning               4
#define esInfo                  7
  
/* Information classes */
#define xxNone                  0
#define xxInteger               1
#define xxShort                 2
#define xxLong                  3
#define xxReal                  4
#define xxBoolean               5
#define xxCharacter             6
#define xxString                7
#define xxSet                   8
#define xxIdent                 9
  
/* valid message formats */
  
typedef enum
{
  mf_standard, /* standart format: <file>:<line>:<column>:<kind>:<message> */
  mf_no_file                    /* format: <line>:<column>:<kind>:<message> */
#if defined ENABLE_GREF || defined __REUSE__
  , mf_SDT       /* SDT organizer log format */
#endif
} mf_type;
  
typedef struct tErrorTable
{
  int Number;
  int Class;
  char *String;
} tErrorTable;
/*
 * This structure described one user-defined error message with user code.
 * User may provide a table of its messages to call routines of this
 * module with error number rather than error string provided.
 * This table must be passed to function `errInit' during start-up.
 * Field `Number' defines number of an error message. This number should be
 * used instead of full error message text. Field `Class' defines class
 * of this error message. A class maybe one of error classes defined
 * above (e. g. xxError). Field `String' contains error message string.
 * This string may contain format specifiers for function `printf'. In
 * this case user should take care to provide proper auxiluary parameters
 * to error reporting routine.
 */

/* Original cocktail functions (see description of them in
 *   cocktail documentation)
 */

#if defined __cplusplus
extern "C" {
#endif /* __cplusplus */

#define StoreMessages(s)    errStoreMessages(s)
  R_DLL void ErrorMessage(int ErrorCode, int ErrorClass, tPosition Position);
  R_DLL void ErrorMessageI(int ErrorCode, int ErrorClass, tPosition Position,
                           int InfoClass, void *Info);
  R_DLL void Message(char * ErrorText, int ErrorClass,tPosition Position);
  R_DLL void MessageI(char * ErrorText, int ErrorClass, tPosition Position,
                     int InfoClass,void *Info);
#define WriteMessages(f)    errFlushMessages(f)

/* New functions and variables*/

  /* In this table number of error ocurences for each error class is
   * stored. Correspoding error class (for example xxError) may be used
   * for accessing this number. This array may be accessed for read only.
   */
  R_DLL int *errGetMsgCounters(void);

  /* function pointer to which is stored in this variable is called when
   * fatal error occured before exiting. The variable is accessible for
   * read and write.
   */
  typedef void (*errFatalHookFunc_t)(void);
  R_DLL errFatalHookFunc_t errGetFatalHook(void);
  R_DLL errFatalHookFunc_t errSetFatalHook(errFatalHookFunc_t);

#if defined ENABLE_GREF
#define errInitModule(t,f) _errInitModule(1,t,f)
#elif defined __REUSE__
#define errInitModule(t,f) _errInitModule(-1,t,f)
#else
#define errInitModule(t,f) _errInitModule(0,t,f)
#endif

  R_DLL void _errInitModule(int sys, tErrorTable *petTable, int mfFormat);
/* initializes internal tables. Must be called during start-up of the
 * program. The first argument of this function is user error table which
 * is described before. The second argument is a format which will be used
 * for printing a source position.
 */

  R_DLL void errCloseModule(void);
/*
 * Closes the module. Deallocates internal buffers and other data
 * structures. You MUST NOT call functions of the module after call of
 * `errCloseModule'. The only functions you may call are `errInitModule'
 * and `errStatistics'
 */

  R_DLL void errSetMessageFormat(int mfFormat);
/*
 * Sets format of printing sourse position according to format 
 * `mfFormat'.
*/

  R_DLL void errReport(int code,tPosition pos,...);
/* reports error by its number. This number must correspond to number
 * of error message defined in user error table. Additional parameters
 * must correspond to format specifiers in the error message text.
 */

  R_DLL void errVReport(int code,tPosition pos,va_list args);
/* The same as above except that additional parameters are passed in
 * `args' argument of type `va_list' (from <stdarg.h>).
 */

  R_DLL void errMessage(int Class,tPosition pos,char *format,...);
/* reports error by its text rather than number. `class' argument is
 * error class of this message. If error class is `xxFatal' program
 * execution is terminated immediately after printing of this error
 * message (and all error messages in error store, if error messages
 * are stored in error store rather than direct print them.
 * Argument `pos' defines position in source text where the error
 * occured. `format' is a format string for function `printf'. User
 * may pass additional parameters corresponding to format specification.
 */

  R_DLL void errVMessage(int Class,tPosition pos,char *format,va_list args);
/* The same as above except way of passing of additional parameters */

  R_DLL void errFlushMessages(FILE *f);
/* Prints all messages in error store to file f. All messages are
 * sorted by their position before printing .
 */

  R_DLL void errStoreMessages(int fStore);
/* Switch error accumulating mode. If argument `fStore' is 1 (true),
 * all subsequent error messages will be store in a table in memory
 * rather that printed to stderr. If the argument is 0 (false),
 * all messages that currently stored in error store are printed to
 * `stderr' and all subsequent messages are also be printed to `stderr'
 */

  R_DLL void errDisable(int ErrorCode);
/* Disables printing of error message with error code `ErrorCode'.
 * Error code is one of error codes described above (e. g. xxSyntaxError).
 * This routine may be useful to suppress certain error messages of
 * automatically generated parser (for example message xxTokenInserted).
 */


  R_DLL void errClear(void);
/* Clears the accumulated error statistics
 */

  R_DLL int errStatistics(FILE *f,int iMode);
/* Prints error statistics to FILE *f and return recommended
 * return code. If iMode == 1, do not print statistics to file, return
 * recommended error code only.
 * NOTE, that signature of this function differs from signatures of
 * similar functions from other modules.
 */

#if defined __cplusplus
}
#endif /* __cplusplus */

#endif /* __REUSE_RERRORS_H__ */

/**
 * Local variables:
 *  compile-command: "make -C .."
 *  c-font-lock-extra-types: ("FILE" "\\sw+_t" "tPosition" "tErrorTable" "R_DLL" "va_list")
 * End:
 */
