/* $Id: Sets.h 407 2005-10-27 08:59:05Z cher $ */

#ifndef __REUSE_SETS_H__
#define __REUSE_SETS_H__

/* Copyright (C) 1995-2005 Alexander Chernov <cher@ispras.ru> */
/* Ich, Doktor Josef Grosch, Informatiker, Sept. 1987 */
/* Alexander Chernov, Nov. 1995 */

/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 */

#include <reuse/integral.h>
#include <reuse/Defs.h>
#include <reuse/ReuseDefs.h>
#include <reuse/GenOut.h>

#include <stdio.h>
#include <stdlib.h>

typedef void (*tsetElemPrint)(rushort_t,tGenOut*);

#define BitsPerBitset           32
#define LdBitsPerBitset         5
#define MaskBitsPerBitset       0x0000001f

typedef ruint32_t  BITSET;

typedef struct tSet
{
  rushort_t        MaxElmt;
  rushort_t        LastBitset;
  BITSET *      BitsetPtr;
  short         Card;
  rushort_t        FirstElmt;
  rushort_t        LastElmt;
  tsetElemPrint PrintFunc;
} tSet;

#define IsElement(Elmt,Set)        ((int) ((Set)->BitsetPtr [(Elmt) >> LdBitsPerBitset] << ((Elmt) & MaskBitsPerBitset)) < 0)
#define Size(Set)                  ((Set)->MaxElmt)
#define Select(Set)                Minimum (Set)
#define IsNotEqual(Set1, Set2)     (! IsEqual (Set1, Set2))
#define IsStrictSubset(Set1, Set2) (IsSubset (Set1, Set2) && IsNotEqual (Set1, Set2))

#if defined __cplusplus
extern "C" {
#endif /* __cplusplus */

  R_DLL void      MakeSet(tSet * Set, rushort_t);

  R_DLL void      WriteSet        (FILE *f,tSet *Set);
  R_DLL void      ReleaseSet      (tSet * Set);
  R_DLL void      Union           (tSet * Set1, tSet * Set2);
  R_DLL void      Difference      (tSet * Set1, tSet * Set2);
  R_DLL void      Intersection    (tSet * Set1, tSet * Set2);
  R_DLL void      SymDiff         (tSet * Set1, tSet * Set2);
  R_DLL void      Complement      (tSet * Set);
  R_DLL void      Include         (tSet * Set, rushort_t Elmt);
  R_DLL void      Exclude         (tSet * Set, rushort_t Elmt);
  R_DLL rushort_t Card            (tSet * Set);
/* extern rushort_t        Size            (tSet * Set); */
  R_DLL rushort_t Minimum         (tSet * Set);
  R_DLL rushort_t Maximum         (tSet * Set);
/* extern rushort_t        Select          (tSet * Set); */
  R_DLL rushort_t Extract         (tSet * Set);
  R_DLL Bool      IsSubset        (tSet * Set1, tSet * Set2);
/* extern Bool  IsStrictSubset  (tSet * Set1, tSet * Set2); */
  R_DLL Bool      IsEqual         (tSet * Set1, tSet * Set2);
/* extern Bool  IsNotEqual      (tSet * Set1, tSet * Set2); */
/* extern Bool  IsElement       (rushort_t Elmt, tSet * Set); */
  R_DLL Bool      IsEmpty         (tSet * Set);
  R_DLL Bool      Forall          (tSet * Set, Bool (* Proc) (rushort_t));
  R_DLL Bool      Exists          (tSet * Set, Bool (* Proc) (rushort_t));
  R_DLL Bool      Exists1         (tSet * Set, Bool (* Proc) (rushort_t));
  R_DLL void      Assign          (tSet * Set1, tSet * Set2);
  R_DLL void      AssignElmt      (tSet * Set, rushort_t Elmt);
  R_DLL void      AssignEmpty     (tSet * Set);
  R_DLL void      ForallDo        (tSet * Set, void (* Proc) (rushort_t));
  R_DLL void      ReadSet         (FILE * File, tSet * Set);

/* New functions */
  R_DLL void      setInit         (tSet *, rushort_t, tsetElemPrint);
  R_DLL void      setWrite        (tGenOut *,tSet *,char *left,char *delim,
                             char *right);

/* Module initialization/closure/statistics */
  R_DLL void      setInitModule(void);
  R_DLL void      setCloseModule(void);
  R_DLL void      setStatistics(FILE *out);

#if defined __cplusplus
}
#endif /* __cplusplus */

#endif /* __REUSE_SETS_H__ */

/**
 * Local variables:
 *  compile-command: "make -C ../.."
 *  c-font-lock-extra-types: ("FILE" "\\sw+_t" "Bool" "tGenOut" "tsetElemPrint" "tSet" "BITSET" "R_DLL")
 * End:
 */
