/* $Id: StringMem.h 352 2004-07-17 06:11:56Z cher $ */

#ifndef __REUSE_STRINGMEM_H__
#define __REUSE_STRINGMEM_H__

/* Copyright (C) 1995-2004 Alexander Chernov <cher@ispras.ru> */
/* Ich, Doktor Josef Grosch, Informatiker, Sept. 1987 */
/* Alexander Chernov, December 1995 */

/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 */

#include <reuse/ReuseDefs.h>

#include <stdio.h>

#if defined __cplusplus
extern "C" {
#endif /* __cplusplus */

typedef unsigned int   tStringRefLen;
typedef tStringRefLen *tStringRef;

extern R_DLL tStringRef PutString(const char * s, tStringRefLen length);
/* Stores string 's' in the string memory and   */
/* returns a reference to the stored string.    */

extern R_DLL void StGetString(tStringRef r, char * s);
/* Returns the string 's' from the string       */
/* memory which is referenced by 'r'.           */

/* extern tStringRefLen LengthSt ARGS((register tStringRef r)); */
#ifdef REUSE_DEBUG
#define LengthSt(r) _LengthSt(r)
extern R_DLL tStringRefLen _LengthSt(tStringRef r);
#else
#define LengthSt(stringref) (* stringref)
#endif
/* Returns the length of the string 's'         */
/* which is referenced by 'r'.                  */

/* extern char *stmString(tStringRef r) */
#ifdef REUSE_DEBUG
#define stmString(r) _stmString(r)
extern R_DLL char *_stmString(tStringRef r);
#else
#define stmString(r) ((char*) (r + 1))
#endif
/* Returns pointer to internal storage of string r */

extern R_DLL Bool IsEqualSt(tStringRef r, char *s);
/* Compares the string referenced by 'r' and    */
/* the string 's'.                              */
/* Returns true if both are equal.              */

extern R_DLL void WriteString(FILE * f, tStringRef r);
/* The string referenced by 'r' is printed on   */
/* the file 'f'.                                */

extern R_DLL void WriteStringMemory(FILE *);
/* The contents of the string memory is printed */
/* on the given file.                           */

#define stmInitModule()     InitStringMemory()
extern R_DLL void InitStringMemory(void);
/* The string memory is initialized.            */

extern R_DLL void stmStatistics(FILE *);
/* Prints statistics. */

extern R_DLL void stmClear(void);
/* Clears storage, but does not releases memory allocated for it */

extern R_DLL void stmCloseModule(void);
/* release internal data structures */

extern R_DLL void *stmSaveState(void);
/* saves state of the module */

extern R_DLL void *stmRestoreState(void *);
/* restores state of the module */

#if defined __cplusplus
}
#endif /* __cplusplus */

#endif /* __REUSE_STRINGMEM_H__ */

/**
 * Local variables:
 *  compile-command: "make -C ../.."
 *  c-font-lock-extra-types: ("FILE" "\\sw+_t" "Bool" "tStringRef" "tStringRefLen" "R_DLL")
 * End:
 */
