/* $Id: System.h 305 2002-06-15 22:18:12Z cher $ */

# ifndef __REUSE_SYSTEM_H__
# define __REUSE_SYSTEM_H__

/* Copyright (C) 1995-2002 Alexander Chernov <cher@ispras.ru> */
/* Ich, Doktor Josef Grosch, Informatiker, Jan. 1992 */

/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 */

#ifdef __cplusplus
extern "C" {
#endif

/* interface for machine dependencies */

#include <reuse/Defs.h>
#include <reuse/ReuseDefs.h>

#define tFile int

/* binary IO */

R_DLL tFile    OpenInput(char const * FileName);
                        /* Opens the file whose name is given by the    */
                        /* string parameter 'FileName' for input.       */
                        /* Returns an integer file descriptor.          */

R_DLL tFile    OpenOutput(char const * FileName);
                        /* Opens the file whose name is given by the    */
                        /* string parameter 'FileName' for output.      */
                        /* Returns an integer file descriptor.          */

R_DLL int      Read(tFile File, char * Buffer, int Size);
                        /* Reads 'Size' bytes from file 'tFile' and     */
                        /* stores them in a buffer starting at address  */
                        /* 'Buffer'.                                    */
                        /* Returns the number of bytes actually read.   */

R_DLL int      Write(tFile File, char * Buffer, int Size);
                        /* Writes 'Size' bytes from a buffer starting   */
                        /* at address 'Buffer' to file 'tFile'.         */
                        /* Returns the number of bytes actually written.*/

R_DLL void     Close(tFile File);
                        /* Closes file 'tFile'.                         */

R_DLL Bool IsCharacterSpecial(tFile File);
                        /* Returns TRUE when file 'tFile' is connected  */
                        /* to a character device like a terminal.       */


/* calls other than IO */

R_DLL char *   SysAlloc(long ByteCount);
                        /* Returns a pointer to dynamically allocated   */
                        /* memory space of size 'ByteCount' bytes.      */
                        /* Returns NIL if space is exhausted.           */

R_DLL long     Time(void);
                        /* Returns consumed cpu-time in milliseconds.   */

R_DLL int      GetArgCount(void);
                        /* Returns number of arguments.                 */

R_DLL void     GetArgument(int ArgNum, char * Argument);
                        /* Stores a string-valued argument whose index  */
                        /* is 'ArgNum' in the memory area 'Argument'.   */

R_DLL void     PutArgs(int Argc, char * * Argv);
                        /* Dummy procedure that passes the values       */
                        /* 'argc' and 'argv' from Modula-2 to C.        */

R_DLL int      ErrNum(void);
                        /* Returns the current system error code.       */

R_DLL int      System(char const *String);
                        /* Executes an operating system command given   */
                        /* as the string 'String'. Returns an exit or   */
                        /* return code.                                 */

R_DLL void     Exit(int Status);
                        /* Terminates program execution and passes the  */
                        /* value 'Status' to the operating system.      */

R_DLL void     BEGIN_System(void);
                        /* Dummy procedure with empty body.             */

#ifdef __cplusplus
};
#endif

#endif /* __REUSE_SYSTEM_H__ */

/**
 * Local variables:
 *  compile-command: "make -C .."
 *  c-font-lock-extra-types: ("FILE" "\\sw+_t" "Bool" "tFile" "R_DLL")
 * End:
 */
