/* -*- mode:c -*- */
/* $Id: bitset.h 352 2004-07-17 06:11:56Z cher $ */

#ifndef __REUSE_BITSET_H__
#define __REUSE_BITSET_H__

/* Copyright (C) 2002-2004 Alexander Chernov <cher@ispras.ru> */

/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 */

#include <reuse/ReuseDefs.h>
#include <reuse/integral.h>

#define BITSET_BPW 32
#define BITSET_BPS 5
#define BITSET_BPM 0x0000001f

typedef struct bitset_t
{
  unsigned int size;
  unsigned int arr_size;
  int *bits;
  int card;
  int first;
  int last;
} bitset_t;

#ifdef __GNUC__
/* provide inlined versions of some functions */
extern R_DLL inline int
set_is_member(unsigned int elem, bitset_t const *pset)
{
  return (pset->bits[elem >> BITSET_BPS] << (elem & BITSET_BPM)) < 0;
}
extern R_DLL inline unsigned int
set_get_size(bitset_t const *pset)
{
  return pset->size;
}
#else
R_DLL int set_is_member(unsigned int, bitset_t const *);
R_DLL unsigned int set_get_size(bitset_t const *);
#endif /* __GNUC__ */

R_DLL int set_is_eq(bitset_t const*, bitset_t const*);
R_DLL int set_is_ne(bitset_t const*, bitset_t const*);
R_DLL int set_is_le(bitset_t const*, bitset_t const*);
R_DLL int set_is_lt(bitset_t const*, bitset_t const*);
R_DLL int set_is_ge(bitset_t const*, bitset_t const*);
R_DLL int set_is_gt(bitset_t const*, bitset_t const*);

R_DLL void set_init(bitset_t*, unsigned int);
R_DLL void set_free(bitset_t*);

R_DLL void set_plus(bitset_t*, bitset_t const*);
R_DLL void set_minus(bitset_t*, bitset_t const*);
R_DLL void set_and(bitset_t*, bitset_t const*);
R_DLL void set_xor(bitset_t*, bitset_t const*);
R_DLL void set_not(bitset_t*);
R_DLL void set_add(unsigned int, bitset_t*);
R_DLL void set_remove(unsigned int, bitset_t*);
R_DLL unsigned int set_get_card(bitset_t*);
R_DLL int set_get_first(bitset_t *);
R_DLL int set_get_last(bitset_t *);
R_DLL int set_is_empty(bitset_t const *);
R_DLL void set_assign(bitset_t*, bitset_t const*);
R_DLL void set_copy(bitset_t*, bitset_t const*);
R_DLL void set_clear(bitset_t*);

#endif /* __REUSE_BITSET_H__ */

/*
 * Local variables:
 *  compile-command: "make -C ../.."
 *  c-font-lock-extra-types: ("FILE" "\\sw+_t" "inline" "R_DLL")
 * End:
 */
