/* $Id: exec.h 476 2007-10-30 18:54:40Z cher $ */

#ifndef __REUSE_EXEC_H__
#define __REUSE_EXEC_H__

/* Copyright (C) 1998-2007 Alexander Chernov <cher@ejudge.ru> */
/* Created: <1998-01-21 14:26:50 cher> */

/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 */

#include <reuse/ReuseDefs.h>

#include <stdio.h>

#ifdef __cplusplus
extern "C"
{
#endif /* __cplusplus */

/* Task states */
enum
{ 
  TSK_ERROR,                    /* error occured */
  TSK_STOPPED,                  /* task is not started */
  TSK_RUNNING,                  /* task is runnning */
  TSK_EXITED,                   /* task has exited */
  TSK_SIGNALED                  /* task was terminated by a signal */
};

/* file descriptor operations */
enum
{
  TSR_CLOSE,                    /* close file descriptor */
  TSR_FILE,                     /* redirect to/from file */
  TSR_DUP,                      /* duplicate file descriptor */
  TSR_PIPE                      /* pipe the file descriptor */
};

/* file open flags */
enum
{
  TSK_REWRITE = 0x10000,
  TSK_WRITE   = 0x20000,
  TSK_READ    = 0x30000
};

/* file open modes */
enum
{
  TSK_FULL_RW = 0666
};

/* task descriptor structure, not exported */
struct tTask;
typedef struct tTask tTask, *tpTask;

R_DLL int      task_SetFlag(char *, int);

R_DLL tpTask   task_New(void);
R_DLL int      task_AddArg(tpTask, char const *arg);
R_DLL int      task_nAddArgs(tpTask, int n, ...);
R_DLL int      task_zAddArgs(tpTask, ... /* NULL */);
R_DLL int      task_pnAddArgs(tpTask, int n, char **p);
R_DLL int      task_pzAddArgs(tpTask, char **p);
R_DLL int      task_SetPath(tpTask, char const *arg);
R_DLL int      task_SetRedir(tpTask, int fd, int mode, ...);
R_DLL void     task_Delete(tpTask);
R_DLL char    *task_SetPathAsArg0(tpTask);
R_DLL int      task_SetEntryFunction(tpTask, int (*)(int, char **));
R_DLL int      task_SetWorkingDir(tpTask, char const *);
R_DLL int      task_SetMaxTime(tpTask, int);
R_DLL int      task_SetMaxTimeMillis(tpTask, int);
R_DLL int      task_SetMaxRealTime(tpTask, int);
R_DLL int      task_SetMaxProcCount(tTask *tsk, int cnt);

R_DLL int      task_PutEnv(tpTask, char const *);
R_DLL int      task_ClearEnv(tpTask);

R_DLL int      task_SetKillSignal(tpTask, char const *);
R_DLL int      task_SetStackSize(tpTask, int);
R_DLL int      task_SetDataSize(tpTask, int);
R_DLL int      task_SetVMSize(tpTask, int);
R_DLL int      task_DisableCoreDump(tpTask);
R_DLL int      task_EnableMemoryLimitError(tpTask);
R_DLL int      task_EnableSecureExec(tpTask);
R_DLL int      task_EnableAllSignals(tpTask);
R_DLL int      task_EnableSecurityViolationError(tpTask);

R_DLL int      task_PrintArgs(tpTask);
R_DLL void     task_fPrintArgs(tpTask, FILE *);
R_DLL int      task_SetQuietFlag(tpTask);
R_DLL char*    task_GetErrorMessage(tpTask);

R_DLL int      task_Start(tpTask);
R_DLL tpTask   task_Wait(tpTask);
R_DLL int      task_Kill(tpTask);
R_DLL int      task_Status(tpTask);
R_DLL int      task_TermSignal(tpTask);
R_DLL int      task_ExitCode(tpTask);
R_DLL int      task_ErrorCode(tpTask, int *, int *);
R_DLL long     task_GetRunningTime(tpTask);
R_DLL long     task_GetRealTime(tpTask);
R_DLL void     task_Log(tpTask, int fac, int sev);
R_DLL int      task_IsAbnormal(tpTask);
R_DLL int      task_IsTimeout(tpTask);
R_DLL int      task_IsMemoryLimit(tpTask);
R_DLL int      task_IsSecurityViolation(tpTask);
R_DLL int      task_GetPipe(tpTask, int);

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* __REUSE_EXEC_H__ */

/*
 * Local variables:
 *  compile-command: "make -C .."
 *  c-font-lock-extra-types: ("FILE" "\\sw+_t" "tpTask" "tTask" "va_list" "R_DLL")
 * End:
 */
