/* $Id: stream_io.h 315 2003-08-01 19:55:42Z cher $ */

#ifndef __REUSE_STREAM_IO_H__
#define __REUSE_STREAM_IO_H__

/* Copyright (C) 1997-2003 Alexander Chernov <cher@ispras.ru> */
/* Created: <97/11/01 16:41:01 cher> */

/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 */

#include <reuse/ReuseDefs.h>

#include <stdio.h>
#include <stdarg.h>

/* This is platform-specific structure */
struct tStream;
typedef struct tStream tStream, *tpStream;

/* stream types */
#define sk_Inet  0
#define sk_File  1

/* stream open modes */
#define SFM_READ       1        /* read */
#define SFM_WRITE      2        /* write */
#define SFM_APPEND     4        /* append */

/* stream directions */
#define SFD_INPUT      0
#define SFD_OUTPUT     1

/* Stream control codes */
#define SFC_SETPORT    0        /* set port to listen on */
#define SFC_SETFD      1        /* set fd to use */
#define SFC_SETPATH    2        /* set path to open */

#define SFC_SETTRIM    3
#define SFC_SETBUFSIZE 4
#define SFC_SETMODE    5

#define SFC_SETBIN     6
#define SFC_SETTEXT    7

#define SFC_LENGTH     8        /* get file length (if applicable) */
#define SFC_SETBUFMODE 9

#define SFC_GETPEERHOST 10      /* get the peer host */
#define SFC_GETPEERPORT 11      /* get the peer port */

#define SFC_SETSOCKOPT  12      /* set socket options */

#define SFC_SETLOGFILE  13      /* set the name of the log file */

#define SFC_SETIFD      14      /* set the reading file descriptor */
#define SFC_SETOFD      15      /* set the writing file descriptor */

/* options for SETSOCKOPT */
#define SFCS_REUSEADDR  1

R_DLL tpStream strm_New(int);
R_DLL int      strm_GetKind(tpStream);

R_DLL int      strm_Fcntl(tpStream, int, ...);

R_DLL int      strm_Open(tpStream);
R_DLL tpStream strm_Close(tpStream);

R_DLL int      strm_GetChar(tpStream);
R_DLL int      strm_Get3Bytes(tpStream, long *);
R_DLL int      strm_GetShort(tpStream, short *);
R_DLL int      strm_GetInt(tpStream, int *);
R_DLL int      strm_GetLong(tpStream, long *);
R_DLL int      strm_GetFloat(tpStream, float *);
R_DLL int      strm_GetDouble(tpStream, double *);
R_DLL int      strm_GetLongDouble(tpStream, long double *);
R_DLL int      strm_GetString(tpStream, char *str, int maxlen);
R_DLL int      strm_Scanf(tpStream, char *, ...);
R_DLL int      strm_VScanf(tpStream, char *, va_list);

R_DLL int      strm_IsEOF(tpStream);
R_DLL int      strm_Ungetc(tpStream, int);

R_DLL int      strm_Read(tpStream, void *, int);

R_DLL int      strm_PutChar(tpStream, int);
R_DLL int      strm_Put3Bytes(tpStream, long);
R_DLL int      strm_PutShort(tpStream, short);
R_DLL int      strm_PutInt(tpStream, int);
R_DLL int      strm_PutLong(tpStream, long);
R_DLL int      strm_PutFloat(tpStream, float);
R_DLL int      strm_PutDouble(tpStream, double);
R_DLL int      strm_PutLongDouble(tpStream, long double);
R_DLL int      strm_PutString(tpStream, char *);
R_DLL int      strm_Printf(tpStream, char *, ...);
R_DLL int      strm_VPrintf(tpStream, char *, va_list);

R_DLL int      strm_Write(tpStream, void *, int);

R_DLL int      strm_Flush(tpStream);

R_DLL int      strm_Status(tpStream);
R_DLL char    *strm_Message(tpStream);
R_DLL void     strm_Recover(tpStream);

#endif /* __REUSE_STREAM_IO_H__ */

/*
 * Local variables:
 *  compile-command: "make -C .."
 *  c-font-lock-extra-types: ("\\sw+_t" "FILE" "va_list" "tStream" "tpStream" "R_DLL")
 * End:
 */
