/* -*- mode:c -*- */
/* $Id: os_vfprintf.c 354 2004-07-17 06:28:14Z cher $ */

/* Copyright (C) 2002-2004 Alexander Chernov <cher@ispras.ru> */

/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 */

#define __REUSE__ 1

/* internal include directives */
#include "lconfig.h"
#include "reuse/config.h"
#include "embed_version.h"

/* reuse include directives */
#include <reuse/integral.h>
#include <reuse/format_io_int.h>

#include <stdio.h>
#include <stdarg.h>

#if CONF_USE_SYSTEM_PRINTF - 0 != 1
struct fprintf_data
{
  FILE *stream;
};
static int
fprintf_put_char(void *vp, int c)
{
  struct fprintf_data *p = (struct fprintf_data*) vp;
  return putc(c, p->stream);
}
static int
fprintf_put_string(void *vp, char const *str)
{
  struct fprintf_data *p = (struct fprintf_data*) vp;
  return fputs(str, p->stream);
}
static int
fprintf_put_mem(void *vp, char const *str, size_t size)
{
  struct fprintf_data *p = (struct fprintf_data*) vp;
  int r = fwrite(str, 1, size, p->stream);
  if (r != size) {
    r = EOF;
  }
  return r;
}
struct reuse_printf_funcs fprintf_funcs =
{
  fprintf_put_char,
  fprintf_put_string,
  fprintf_put_mem
};
#endif

int
os_vfprintf(FILE *stream, char const *format, va_list args)
{
#if CONF_USE_SYSTEM_PRINTF - 0 == 1
  return vfprintf(stream, format, args);
#else
  struct fprintf_data d;
  d.stream = stream;
  return _reuse_vprintf(&fprintf_funcs, &d, format, args);
#endif /* CONF_USE_SYSTEM_PRINTF */
}

int
os_fprintf(FILE *stream, char const *format, ...)
{
  va_list args;

  va_start(args, format);
#if CONF_USE_SYSTEM_PRINTF - 0 == 1
  return vfprintf(stream, format, args);
#else
  return os_vfprintf(stream, format, args);
#endif /* CONF_USE_SYSTEM_PRINTF */
}

#if REUSE_COMPILE_CVSID - 0 != 0
static char const rcsid [] =
"$Id: os_vfprintf.c 354 2004-07-17 06:28:14Z cher $";
#endif /* REUSE_COMPILE_CVSID */

/*
 * Local variables:
 *  c-font-lock-extra-types: ("\\sw+_t" "FILE" "va_list")
 *  compile-command: "make"
 * End:
 */
