/* -*- mode:c -*- */
/* $Id: os_vprintf.c 354 2004-07-17 06:28:14Z cher $ */

/* Copyright (C) 2002-2004 Alexander Chernov <cher@ispras.ru> */

/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 */

#define __REUSE__ 1

/* internal include directives */
#include "lconfig.h"
#include "reuse/config.h"
#include "embed_version.h"

/* reuse include directives */
#include <reuse/integral.h>
#include <reuse/format_io_int.h>

#include <stdio.h>
#include <stdarg.h>

#if CONF_USE_SYSTEM_PRINTF - 0 != 1
static int
printf_put_char(void *vp, int c)
{
  return putchar(c);
}
static int
printf_put_string(void *vp, char const *str)
{
  return fputs(str, stdout);
}
static int
printf_put_mem(void *vp, char const *str, size_t size)
{
  int r = fwrite(str, 1, size, stdout);
  if (r != size) r = EOF;
  return r;
}
struct reuse_printf_funcs printf_funcs =
{
  printf_put_char,
  printf_put_string,
  printf_put_mem
};
#endif

int
os_vprintf(char const *format, va_list args)
{
#if CONF_USE_SYSTEM_PRINTF - 0 == 1
  return vprintf(format, args);
#else
  return _reuse_vprintf(&printf_funcs, 0, format, args);
#endif /* CONF_USE_SYSTEM_PRINTF */
}

int
os_printf(char const *format, ...)
{
  va_list args;

  va_start(args, format);
#if CONF_USE_SYSTEM_PRINTF - 0 == 1
  return vprintf(format, args);
#else
  return os_vprintf(format, args);
#endif /* CONF_USE_SYSTEM_PRINTF */
}

#if REUSE_COMPILE_CVSID - 0 != 0
static char const rcsid [] =
"$Id: os_vprintf.c 354 2004-07-17 06:28:14Z cher $";
#endif /* REUSE_COMPILE_CVSID */

/*
 * Local variables:
 *  c-font-lock-extra-types: ("\\sw+_t" "FILE" "va_list")
 *  compile-command: "make"
 * End:
 */
