/* -*- mode:c -*- */
/* $Id: os_vscanf.c 354 2004-07-17 06:28:14Z cher $ */

/* Copyright (C) 2002-2004 Alexander Chernov <cher@ispras.ru> */

/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 */

#define __REUSE__ 1

/* internal include directives */
#include "lconfig.h"
#include "reuse/config.h"
#include "embed_version.h"

/* reuse include directives */
#include <reuse/integral.h>
#include <reuse/format_io_int.h>

#include <stdio.h>
#include <stdarg.h>

#if CONF_USE_SYSTEM_SCANF - 0 != 1
static int
scanf_next_char(void *vp)
{
  return getchar();
}
static int
scanf_prev_char(void *vp, int c)
{
  return ungetc(c, stdin);
}
static struct reuse_scanf_funcs scanf_vtab =
{
  scanf_next_char,
  scanf_prev_char
};
#else
/* it's better not to have a prototype, than to have a wrong one */
int vscanf();
#endif /* local scanf will not be used */

int
os_vscanf(char const *format, va_list args)
{
#if CONF_USE_SYSTEM_SCANF - 0 == 1
  return vscanf(format, args);
#else
  return _reuse_vscanf(&scanf_vtab, 0, format, args);
#endif /* vscanf is useable */
}

int
os_scanf(char const *format, ...)
{
  va_list args;
  int ret_value;

  va_start(args, format);
#if CONF_USE_SYSTEM_SCANF - 0 == 1
  ret_value = vscanf(format, args);
#else
  ret_value = os_vscanf(format, args);
#endif /* scanf is useable */
  va_end(args);
  return ret_value;
}

#if REUSE_COMPILE_CVSID - 0 != 0
static char const rcsid [] =
"$Id: os_vscanf.c 354 2004-07-17 06:28:14Z cher $";
#endif /* REUSE_COMPILE_CVSID */

/*
 * Local variables:
 *  c-font-lock-extra-types: ("\\sw+_t" "FILE" "va_list")
 *  compile-command: "make"
 * End:
 */
