/* -*- mode:c -*- */
/* $Id: os_vsnprintf.c 413 2005-10-27 09:01:56Z cher $ */

/* Copyright (C) 2002-2005 Alexander Chernov <cher@ispras.ru> */

/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 */

#define __REUSE__ 1

/* internal include directives */
#include "lconfig.h"
#include "reuse/config.h"
#include "embed_version.h"

/* reuse include directives */
#include <reuse/integral.h>
#include <reuse/format_io_int.h>

#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>

#if CONF_USE_SYSTEM_PRINTF - 0 != 1
# define GENCODE 1
#else
# if CONF_HAS_SNPRINTF - 0 != 1 && CONF_HAS__SNPRINTF - 0 != 1
#  define GENCODE 1
# else
#  define GENCODE 0
# endif
#endif /* system printf cannot be used */

#if GENCODE - 0 == 1
struct snprintf_data
{
  unsigned char *buf;
  int size;
  int limit;
  int cur;
};
static int
snprintf_put_char(void *vp, int c)
{
  struct snprintf_data *p = (struct snprintf_data*) vp;
  if (p->cur < p->limit) {
    p->buf[p->cur++] = c;
  }
  if (p->size > 0) {
    p->buf[p->cur] = 0;
  }
  return c;
}
static int
snprintf_put_string(void *vp, char const *str)
{
  struct snprintf_data *p = (struct snprintf_data*) vp;
  size_t size = strlen(str);
  int nbytes;

  if (p->cur + size > p->limit) {
    nbytes = p->limit - p->cur;
    if (nbytes > 0) {
      memcpy(&p->buf[p->cur], str, nbytes);
      p->buf[p->limit] = 0;
    }
    p->cur += size;
  } else {
    memcpy(&p->buf[p->cur], str, size + 1);
    p->cur += size;
  }
  return size;
}
static int
snprintf_put_mem(void *vp, char const *str, size_t size)
{
  struct snprintf_data *p = (struct snprintf_data*) vp;
  int nbytes;

  if (p->cur + size > p->limit) {
    nbytes = p->limit - p->cur;
    if (nbytes > 0) {
      memcpy(&p->buf[p->cur], str, nbytes);
      p->buf[p->limit] = 0;
    }
    p->cur += size;
  } else {
    memcpy(&p->buf[p->cur], str, size + 1);
    p->cur += size;
  }
  return size;
}
static struct reuse_printf_funcs snprintf_funcs =
{
  snprintf_put_char,
  snprintf_put_string,
  snprintf_put_mem
};
#endif /* generate code */

int
os_vsnprintf(char *buf, size_t size, char const *format, va_list args)
{
#if CONF_USE_SYSTEM_PRINTF - 0 == 1 && CONF_HAS_SNPRINTF - 0 == 1
  return vsnprintf(buf, size, format, args);
#elif CONF_USE_SYSTEM_PRINTF - 0 == 1 && CONF_HAS__SNPRINTF - 0 == 1
  return _vsnprintf(buf, size, format, args);
#else
  struct snprintf_data d;

  d.buf = (unsigned char*) buf;
  d.size = size;
  d.limit = size - 1;
  d.cur = 0;
  return _reuse_vprintf(&snprintf_funcs, &d, format, args);
#endif /* CONF_USE_SYSTEM_PRINTF */
}

int
os_snprintf(char *buf, size_t size, char const *format, ...)
{
  va_list args;

  va_start(args, format);
#if CONF_USE_SYSTEM_PRINTF - 0 == 1 && CONF_HAS_SNPRINTF - 0 == 1
  return vsnprintf(buf, size, format, args);
#elif CONF_USE_SYSTEM_PRINTF - 0 == 1 && CONF_HAS__SNPRINTF - 0 == 1
  return _vsnprintf(buf, size, format, args);
#else
  return os_vsnprintf(buf, size, format, args);
#endif /* CONF_USE_SYSTEM_PRINTF */
}

#if REUSE_COMPILE_CVSID - 0 != 0
static char const rcsid [] =
"$Id: os_vsnprintf.c 413 2005-10-27 09:01:56Z cher $";
#endif /* REUSE_COMPILE_CVSID */

/*
 * Local variables:
 *  c-font-lock-extra-types: ("\\sw+_t" "FILE" "va_list")
 *  compile-command: "make"
 * End:
 */
