/* -*- mode:c -*- */
/* $Id: os_vsprintf.c 413 2005-10-27 09:01:56Z cher $ */

/* Copyright (C) 2002-2005 Alexander Chernov <cher@ispras.ru> */

/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 */

#define __REUSE__ 1

/* internal include directives */
#include "lconfig.h"
#include "reuse/config.h"
#include "embed_version.h"

/* reuse include directives */
#include <reuse/integral.h>
#include <reuse/format_io_int.h>

#include <stdio.h>
#include <stdarg.h>
#include <string.h>

#if CONF_USE_SYSTEM_PRINTF - 0 != 1
struct sprintf_data
{
  char *str;
  char *cur;
};
static int
sprintf_put_char(void *vp, int c)
{
  struct sprintf_data *p = (struct sprintf_data*) vp;
  *p->cur++ = c;
  *p->cur = 0;
  return (unsigned char) c;
}
static int
sprintf_put_string(void *vp, char const *str)
{
  struct sprintf_data *p = (struct sprintf_data*) vp;
  size_t len = strlen(str);
  memcpy(p->cur, str, len + 1);
  p->cur += len;
  return len;
}
static int
sprintf_put_mem(void *vp, char const *str, size_t size)
{
  struct sprintf_data *p = (struct sprintf_data*) vp;
  memcpy(p->cur, str, size);
  p->cur += size;
  *p->cur = 0;
  return size;
}
static struct reuse_printf_funcs sprintf_funcs =
{
  sprintf_put_char,
  sprintf_put_string,
  sprintf_put_mem
};
#endif

int
os_vsprintf(char *buf, char const *format, va_list args)
{
#if CONF_USE_SYSTEM_PRINTF - 0 == 1
  return vsprintf(buf, format, args);
#else
  struct sprintf_data d;

  d.str = buf;
  d.cur = buf;
  return _reuse_vprintf(&sprintf_funcs, &d, format, args);
#endif /* CONF_USE_SYSTEM_PRINTF */
}

int
os_sprintf(char *buf, char const *format, ...)
{
  va_list args;

  va_start(args, format);
#if CONF_USE_SYSTEM_PRINTF - 0 == 1
  return vsprintf(buf, format, args);
#else
  return os_vsprintf(buf, format, args);
#endif /* CONF_USE_SYSTEM_PRINTF */
}

#if REUSE_COMPILE_CVSID - 0 != 0
static char const rcsid [] =
"$Id: os_vsprintf.c 413 2005-10-27 09:01:56Z cher $";
#endif /* REUSE_COMPILE_CVSID */

/*
 * Local variables:
 *  c-font-lock-extra-types: ("\\sw+_t" "FILE" "va_list")
 *  compile-command: "make"
 * End:
 */
