/* -*- mode:c -*- */
/* $Id: os_env.c 404 2005-10-27 08:57:17Z cher $ */

/* Copyright (C) 2002-2005 Alexander Chernov <cher@ispras.ru> */

/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 */

#define __REUSE__ 1

/* internal include directives */
#include "lconfig.h"
#include "reuse/config.h"
#include "embed_version.h"

/* reuse include directives */
#include <reuse/integral.h>
#include <reuse/osdeps.h>
#include <reuse/xalloc.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>

#if defined CONF_ENV_PREFIX
static char env_project_home[64] = CONF_ENV_PREFIX "_HOME";
static char env_project_config[64] = CONF_ENV_PREFIX "_CONFIG";
#else
static char env_project_home[64];
static char env_project_config[64];
#endif /* CONF_ENV_PREFIX */

void
set_PRJ_HOME_name(char const *new_prj_home)
{
  memset(env_project_home, 0, sizeof(env_project_home));
  strncpy(env_project_home, new_prj_home, sizeof(env_project_home) - 1);
}

void
set_PRJ_CONFIG_name(char const *new_prj_config)
{
  memset(env_project_config, 0, sizeof(env_project_config));
  strncpy(env_project_config, new_prj_config, sizeof(env_project_config) - 1);
}

char *
get_PRJ_HOME_name(void)
{
  return env_project_home;
}

  char *
get_PRJ_HOME(void)
{
  return getenv(get_PRJ_HOME_name());
}

  char *
get_PRJ_CONFIG_name(void)
{
  return env_project_config;
}

  char *
get_PRJ_CONFIG(void)
{
  return getenv(get_PRJ_CONFIG_name());
}

/**
 * NAME:    os_GuessProjectEnv
 * PURPOSE: guess the values of <project>_HOME and <project>_CONFIG variables
 *          if those are not defined
 * ARGS:    path - path to the program (argv[0])
 * RETURN:  0 - ok, <0 - error
 */
  int
os_GuessProjectEnv(char *path, char const *project)
{
  int   rcode    = 0;
  char *path1    = 0;
  int   curdirfd = -1;
  char *path2    = 0;
  char *base1    = 0;
  char *path3    = 0;
  char *rep_home = 0;
  char *base2    = 0;
  char *env_str  = 0;
  char *p_config = 0;
  char *p_home   = 0;

  if (project) {
    p_home = xstrmerge2(project, "_HOME");
    p_config = xstrmerge2(project, "_CONFIG");
    set_PRJ_HOME_name(p_home);
    set_PRJ_CONFIG_name(p_config);
    xfree(p_home);
    xfree(p_config);
  }

  p_config = get_PRJ_CONFIG_name();
  p_home   = get_PRJ_HOME_name();

  /* set CONFIG variable */  
  if (!getenv(p_config)) {
#if defined CONF_CANONIC_STR
    /* guess <project>_CONFIG use compiled-in value */
    char *str = xmalloc(strlen(CONF_CANONIC_STR) + 32);
    sprintf(str, "%s=%s", p_config, CONF_CANONIC_STR);
    putenv(str);
    //xfree(str);
#else
    return -1;
#endif /* CONF_CANONIC_STR */
  }

  /* set HOME variable */
  if (!getenv(p_home)) {
#if CONF_HAS_FCHDIR - 0 == 0
    /* CYGWIN32 lacks fchdir (as of versiont B20.1) */
#if defined __CYGWIN32__
    /* Win32 does not put full path to the executed file when
     * the file is searched in PATH variable, so we have to search
     * the full path by ourself.
     *
     * The current directory is always checked first.
     */

    /* check, that we have no dir separators in path */
    if (!strchr(path, '/' && os_CheckAccess(path, REUSE_X_OK) < 0)) {
      if (!(path1 = os_FindInPath((char*) path))) return -1;
      path2 = os_DirName(path1);
    } else {
      XCALLOC(path1, PATH_MAX);
      cygwin_conv_to_full_posix_path(path, path1);
      path2 = os_DirName(path1);
    }
#else
    path2 = os_DirName(path);
#endif /* __CYGWIN32__ */
#else
#ifdef __linux__
    /* use the Linux way to get the full path to the executable */
    if (!strchr(path, '/')) {
      unsigned char exe_info_path[1024];
      unsigned char exe_path[1024];
      int link_len;

      snprintf((char*) exe_info_path, sizeof(exe_info_path),
               "/proc/%d/exe", getpid());
      memset(exe_path, 0, sizeof(exe_path));
      if ((link_len=readlink((char*)exe_info_path,(char*)exe_path,sizeof(exe_path)-1))>=0) {
        path = alloca(strlen((char*)exe_path) + 1);
        strcpy(path, (char*)exe_path);
      }
    }
#endif
    if (!strchr(path, '/')) {
      /* we've got plain "filename" in argv[0], this might be if
       *  1) the program is started from the current directory
       *  2) the program is in PATH directory
       * we may check conditions in both orders, but both orders
       * may work badly in certain situations
       */
    }
    /* guess <project>_HOME, as wrapper script does */
    path1 = os_DirName(path);
    if ((curdirfd = open(".", O_RDONLY)) < 0) {
      /* something wrong. so give up. Fixme, if you want */
      return -1;
    }

    if (chdir(path1) < 0 || !(path2 = os_GetWorkingDir())) {
      /* cannot change dir to or retrieve the name of the start directory */
      path2 = path1;
      path1 = 0;
    }

    /* now return to the current working directory */
    if (fchdir(curdirfd) < 0) {
      goto failure;
    }
    close(curdirfd);
    curdirfd = -1;
#endif /* CONF_HAS_FCHDIR == 0 */

    /* now path2 is the ABSOLUTE path to the start directory */
    if (!(base1 = os_GetBasename(path2)))
      goto failure;
    if (!strcmp(base1, "bin")) {
      if (!(rep_home = os_DirName(path2)))
        goto failure;
    }
#if defined CONF_CANONIC_STR
    else if (!strcmp(base1, CONF_CANONIC_STR)
	     || !strcmp(base1, CONF_RUNTIME_STR)) {
      if (!(path3 = os_DirName(path2)))
        goto failure;
      if (!(base2 = os_GetBasename(path3)))
        goto failure;
      if (strcmp(base2, "exec") && strcmp(base2, "libexec"))
        goto failure;
      if (!(rep_home = os_DirName(path3)))
        goto failure;
    }
#endif /* CONF_CANONIC_STR */
    else {
      goto failure;
    }

    /* now rep_home should contain value of <project>_HOME */
    env_str = xmalloc(strlen(rep_home) + 32);
    sprintf(env_str, "%s=%s", p_home, rep_home);
    putenv(env_str);
    env_str = 0;
  }

  rcode = 0;
  goto free_resources;

 failure:
  rcode = -1;

 free_resources:
  /* free resources */
  if (curdirfd >= 0) close(curdirfd);
  xfree(path1);
  xfree(path2);
  xfree(base1);
  xfree(path3);
  xfree(base2);
  xfree(rep_home);
  xfree(env_str);
  return rcode;
}

#if REUSE_COMPILE_CVSID - 0 != 0
static char const rcsid [] =
"$Id: os_env.c 404 2005-10-27 08:57:17Z cher $";
#endif /* REUSE_COMPILE_CVSID */

/*
 * Local variables:
 *  compile-command: "make -C .."
 * End:
 */
