/* -*- mode:c -*- */
/* $Id: os_geterrorstring.c 347 2004-07-10 18:47:03Z cher $ */

/* Copyright (C) 2002-2004 Alexander Chernov <cher@ispras.ru> */

/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 */

#define __REUSE__ 1

/* internal include directives */
#include "lconfig.h"
#include "reuse/config.h"
#include "embed_version.h"

/* reuse include directives */
#include <reuse/osdeps.h>

#include <stdio.h>
#include <string.h>
#include <errno.h>

static char errbuf[256];

/**
 * NAME:    os_GetErrorString
 * PURPOSE: get error string by error code
 * ARGS:    e - error code
 * RETURN:  the corresponding error string
 */
  char *
os_GetErrorString(int e)
{
#if CONF_HAS_STRERROR
  char *s = strerror(e);
  if (s == NULL)
    {
      sprintf(errbuf, "unknown error (%d)", e);
      return errbuf;
    }
  return s;
#elif CONF_HAS_SYS_ERRLIST
  if (e < 0 || e >= sys_nerr)
    {
      sprintf(errbuf, "unknown error (%d)", e);
      return errbuf;
    }
  else 
    return sys_errlist[e];
#elif CONF_HAS__SYS_ERRLIST
  if (e < 0 || e >= _sys_nerr)
    {
      sprintf(errbuf, "unknown error (%d)", e);
      return errbuf;
    }
  else 
    return _sys_errlist[e];
#else
  sprintf(errbuf, "unknown error (%d)", e);
  return errbuf;
#endif
}

#if REUSE_COMPILE_CVSID - 0 != 0
static char const rcsid [] =
"$Id: os_geterrorstring.c 347 2004-07-10 18:47:03Z cher $";
#endif /* REUSE_COMPILE_CVSID */

/*
 * Local variables:
 *  compile-command: "make -C .."
 * End:
 */
