/* -*- mode:c -*- */
/* $Id: os_makedir.c 347 2004-07-10 18:47:03Z cher $ */

/* Copyright (C) 2002-2004 Alexander Chernov <cher@ispras.ru> */

/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 */

#define __REUSE__ 1

/* internal include directives */
#include "lconfig.h"
#include "reuse/config.h"
#include "embed_version.h"

/* reuse include directives */
#include <reuse/osdeps.h>

#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>

/**
 * NAME:    os_MakeDir
 * PURPOSE: create directory
 * ARGS:    path - path to the directory
 *          mode - directory permissions mode
 * RETURN:  <0 - error, >0 - ok, 1 - directory already exists
 */
  int
os_MakeDir(char const *path, int mode)
{
  struct stat buf;
  int         r;

  if (!path) return -1;

  errno = 0;
  r = stat(path, &buf);
  if (r < 0 && errno != ENOENT) return r;
  if (r >= 0 && !S_ISDIR(buf.st_mode)) return -1;
  if (r >= 0) return 1;

  return mkdir(path, mode);
}

#if REUSE_COMPILE_CVSID - 0 != 0
static char const rcsid [] =
"$Id: os_makedir.c 347 2004-07-10 18:47:03Z cher $";
#endif /* REUSE_COMPILE_CVSID */

/*
 * Local variables:
 *  compile-command: "make -C .."
 * End:
 */
