/* -*- mode:c -*- */
/* $Id: os_env.c 422 2006-01-26 20:35:52Z cher $ */

/* Copyright (C) 2002-2005 Alexander Chernov <cher@ispras.ru> */

/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 */

#define __REUSE__ 1

/* internal include directives */
#include "lconfig.h"
#include "reuse/config.h"
#include "embed_version.h"

/* reuse include directives */
#include <reuse/osdeps.h>
#include <reuse/xalloc.h>
#include <reuse/integral.h>

#include <windows.h>
#include <stdio.h>
#include <string.h>
#include <limits.h>

#if defined CONF_ENV_PREFIX
static char env_project_home[64] = CONF_ENV_PREFIX "_HOME";
static char env_project_config[64] = CONF_ENV_PREFIX "_CONFIG";
#else
static char env_project_home[64];
static char env_project_config[64];
#endif /* CONF_ENV_PREFIX */

void
set_PRJ_HOME_name(char const *new_prj_home)
{
  memset(env_project_home, 0, sizeof(env_project_home));
  strncpy(env_project_home, new_prj_home, sizeof(env_project_home) - 1);
}

void
set_PRJ_CONFIG_name(char const *new_prj_config)
{
  memset(env_project_config, 0, sizeof(env_project_config));
  strncpy(env_project_config, new_prj_config, sizeof(env_project_config) - 1);
}

char *
get_PRJ_HOME_name(void)
{
  return env_project_home;
}

  char *
get_PRJ_HOME(void)
{
  return getenv(get_PRJ_HOME_name());
}

  char *
get_PRJ_CONFIG_name(void)
{
  return env_project_config;
}

  char *
get_PRJ_CONFIG(void)
{
  return getenv(get_PRJ_CONFIG_name());
}

int
os_putenv(char const *s)
{
#ifdef _MSC_VER
  extern int _putenv(const char *);
  return _putenv(s);
#else
  return putenv(s);
#endif /* _MSC_VER */
}

char *
os_GetFilename(char const *path)
{
  int         l;
  char const *s;

  if (!path) return xstrdup("");
  l = strlen(path);
  for (s = path + l - 1; s >= path; s--) {
    if (*s == '/' || *s == '\\' || *s == ':') {
      break;
    }
  }
  return xstrdup(s + 1);
}

#if defined __BORLANDC__
#define os_strcasecmp(a,b) stricmp(a,b)
#else
#define os_strcasecmp(a,b) _stricmp(a,b)
#endif /* __BORLANDC__ */

/**
 * NAME:    os_GuessProjectEnv
 * PURPOSE: guess the values of <project>_HOME and <project>_CONFIG variables
 *          if those are not defined
 * ARGS:    path - path to the program (argv[0])
 * RETURN:  0 - ok, <0 - error
 */
int
os_GuessProjectEnv(char *path, char const *project)
{
  char *p_config = 0;
  char *p_home   = 0;

  if (project) {
    p_home = xstrmerge2(project, "_HOME");
    p_config = xstrmerge2(project, "_CONFIG");
    set_PRJ_HOME_name(p_home);
    set_PRJ_CONFIG_name(p_config);
    xfree(p_home);
    xfree(p_config);
  }

  p_config = get_PRJ_CONFIG_name();
  p_home   = get_PRJ_HOME_name();

  if (!getenv(p_config)) {
#if defined CONF_CANONIC_STR
    char *str = 0;

    XCALLOC(str, strlen(CONF_CANONIC_STR) + 32);
    sprintf(str, "%s=%s", p_config, CONF_CANONIC_STR);
    if (os_putenv(str) < 0) {
      /* FIXME: report error */
      xfree(str);
      return -1;
    }
#else
    return -1;
#endif /* CONF_CANONIC_STR */
  }
  //fprintf(stderr, "%s=%s\n", p_config, getenv(p_config));

  if (!getenv(p_home)) {
    char *base  = 0;
    char *path1 = 0;
    char *path2 = 0;
    char *value = 0;
    char *str   = 0;

    XCALLOC(path1, PATH_MAX);

    base = os_GetFilename(path);
    if (!strcmp(base, path)) {
      char *t = 0;
      //fprintf(stderr, "path=%s\n", path);
      if (!SearchPath(NULL, path, CONF_EXE_SUFFIX, PATH_MAX, path1, &t)) {
	/* FIXME: report error */
	//fprintf(stderr, "SearchPath failed: %d\n", GetLastError());
	xfree(base);
	xfree(path1);
	return -1;
      }
    } else {
      char *t = 0;
      if (!GetFullPathName(path, PATH_MAX, path1, &t)) {
	/* FIXME: report error */
	xfree(base);
	xfree(path1);
	return -1;
      }

      {
	char buf[32];
	if (!os_rGetSuffix(path1, buf, 32)) {
	  strcat(path1, ".exe");
	}
      }
    }
    xfree(base);

    //fprintf(stderr, "path1=%s\n", path1);
    path2 = os_DirName(path1);
    xfree(path1);
    base = os_GetFilename(path2);

    {
      WIN32_FIND_DATA f;
      HANDLE h = FindFirstFile(path2, &f);

      if (h == INVALID_HANDLE_VALUE) {
	//fprintf(stderr, "FindFirstFile failed %d\n", GetLastError());
      } else {
	//fprintf(stderr, "file name=%s\n", f.cFileName);
	//fprintf(stderr, "alternate name=%s\n", f.cAlternateFileName);
	xfree(base); base = xstrdup(f.cFileName);
	FindClose(h);
      }
    }

    //fprintf(stderr, "path2=%s, base=%s\n", path2, base);

    if (!os_strcasecmp(base, "bin")) {
      value = os_DirName(path2);
      xfree(path2); xfree(base);
    } else if (!os_strcasecmp(base, CONF_CANONIC_STR)
	       || !os_strcasecmp(base, CONF_RUNTIME_STR)) {
      xfree(base);
      path1 = os_DirName(path2);
      xfree(path2);
      base = os_GetFilename(path1);
      //fprintf(stderr, "path3=%s, base=%s\n", path1, base);
      if (os_strcasecmp(base, "libexec")) {
	xfree(path1); xfree(base);
	return -1;
      }
      value = os_DirName(path1);
      xfree(path1); xfree(base);
    } else if (!strcmp(base, ".obj-" CONF_CANONIC_STR)) {
      xfree(base);
      path1 = os_DirName(path2);
      xfree(path2);
      //fprintf(stderr, "path3=%s\n", path1);
      path2 = os_DirName(path1);
      xfree(path1);
      base = os_GetFilename(path2);
      //fprintf(stderr, "path4=%s, base=%s\n", path2, base);
      if (strcmp(base, "sources")) {
	xfree(path2); xfree(base);
	return -1;
      }
      value = os_DirName(path2);
      xfree(path2); xfree(base);
    } else {
      xfree(path2); xfree(base);
      return -1;
    }

    XCALLOC(str, strlen(value) + 32);
    sprintf(str, "%s=%s", p_home, value);
    xfree(value);
    if (os_putenv(str) < 0) {
      /* FIXME: report error */
      xfree(str);
      return -1;
    }
  }
  //fprintf(stderr, "%s=%s\n", p_home, getenv(p_home));

  return 0;
}

#if REUSE_COMPILE_CVSID - 0 != 0
static char const rcsid [] =
"$Id: os_env.c 422 2006-01-26 20:35:52Z cher $";
#endif /* REUSE_COMPILE_CVSID */

/*
 * Local variables:
 *  compile-command: "make -C ../.."
 * End:
 */
