/* -*- mode:c -*- */
/* $Id: os_findinpath.c 369 2004-09-11 06:13:25Z cher $ */

/* Copyright (C) 2002-2004 Alexander Chernov <cher@ispras.ru> */

/*
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 */

#define __REUSE__ 1

/* internal include directives */
#include "lconfig.h"
#include "reuse/config.h"
#include "embed_version.h"

/* reuse include directives */
#include <reuse/osdeps.h>
#include <reuse/xalloc.h>

#include <windows.h>

/**
 * NAME:    os_FindInPath
 * PURPOSE: find the given executable in the PATH
 * ARGS:    name - name of the executable
 * RETURN:  full path to the file, or NULL if file not found
 */
  char *
os_FindInPath(char const *name)
{
  char buf1[1024];
  char *buf = buf1, *pext = 0;
  unsigned len = sizeof(buf1), new_len;
  
  new_len = SearchPath(NULL, name, NULL, len, buf, &pext);
  if (!new_len) return NULL;
  if (new_len < len) return xstrdup(buf);
  buf = 0;
  while (1) {
    len = (new_len + 16) & ~0xF;
    xfree(buf);
    buf = (char*) xmalloc(len);
    new_len = SearchPath(NULL, name, NULL, len, buf, &pext);
    if (!new_len) return NULL;
    if (new_len < len) return buf;
  }
}

#if REUSE_COMPILE_CVSID - 0 != 0
static char const rcsid [] =
"$Id: os_findinpath.c 369 2004-09-11 06:13:25Z cher $";
#endif /* REUSE_COMPILE_CVSID */

/*
 * Local variables:
 *  compile-command: "make -C ../.."
 * End:
 */
