/* -*- mode: C -*- */
/* $Id: writedld.c 354 2004-07-17 06:28:14Z cher $ */

/* Copyright (C) 2002-2004 Alexander Chernov <cher@ispras.ru> */

/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 */

#define __REUSE__ 1

/* internal include directives */
#include "lconfig.h"
#include "reuse/config.h"
#include "embed_version.h"

/* reuse include directives */
#include <reuse/integral.h>
#include <reuse/number_io.h>
#include <reuse/format_io.h>
#include <reuse/str_utils.h>
#include <reuse/fp_props.h>

#include <string.h>
#include <ctype.h>
#include <errno.h>
#include <stdlib.h>

#include "alloca_impl.h"

#if R_LONG_DOUBLE_IS_DOUBLE - 0 != 1 && CONF_PRINTF_LG_WORKS - 0 != 1
static long double
get10pow(int i_exp)
{
  long double pp = 1.0L;
  if (i_exp >= 0) {
    while (i_exp && !reuse_is_infinity_ld(&pp)) {
      i_exp--;
      pp *= 10.0L;
    }
    if (i_exp) errno = ERANGE;
  } else {
    while (i_exp && pp != 0.0L) {
      i_exp++;
      pp *= 0.1L;
    }
    if (i_exp) errno = ERANGE;
  }
  return pp;
}
#endif

extern const char _reuse_dig2charlower[];
extern const char _reuse_dig2charupper[];

#define PUT_TO_STRING(s,sz,i,v) do { if (sz > 0 && i < sz - 1) s[i]=(v); i++; } while (0)
#define LETTER(x) (digit_conv[x - 'a' + 10])

int
reuse_writedld(char *buf, size_t size, long double const *pv,
               int prec, int flags, int format)
{
#if R_LONG_DOUBLE_IS_DOUBLE - 0 == 1
  char format_buf[32];
  int i = 0;

  format_buf[i++] = '%';
  if ((flags & REUSE_FORMAT_PLUS)) format_buf[i++] = '+';
  else if ((flags & REUSE_FORMAT_SPC)) format_buf[i++] = ' ';
  if ((flags & REUSE_FORMAT_ALT)) format_buf[i++] = '#';
  if (prec >= 0) {
    format_buf[i++] = '.';
    format_buf[i++] = '*';
  }
  if ((flags & REUSE_FORMAT_UP)) {
    format = toupper(format);
  } else {
    format = tolower(format);
  }
  format_buf[i++] = format;
  format_buf[i] = 0;
  if (prec >= 0) {
    return os_snprintf(buf, size, format_buf, prec, *(double const*) pv);
  } else {
    return os_snprintf(buf, size, format_buf, *(double const*) pv);
  }
#else
#if CONF_PRINTF_LG_WORKS - 0 == 1
  char format_buf[32];
  int i = 0;

  format_buf[i++] = '%';
  if ((flags & REUSE_FORMAT_PLUS)) format_buf[i++] = '+';
  else if ((flags & REUSE_FORMAT_SPC)) format_buf[i++] = ' ';
  if ((flags & REUSE_FORMAT_ALT)) format_buf[i++] = '#';
  if (prec >= 0) {
    format_buf[i++] = '.';
    format_buf[i++] = '*';
  }
  format_buf[i++] = 'L';
  if ((flags & REUSE_FORMAT_UP)) {
    format = toupper(format);
  } else {
    format = tolower(format);
  }
  format_buf[i++] = format;
  format_buf[i] = 0;
  if (prec >= 0) {
    return os_snprintf(buf, size, format_buf, prec, *pv);
  } else {
    return os_snprintf(buf, size, format_buf, *pv);
  }
#else
  size_t out_size = 0;
  long double vv = *pv;
  long double v2;
  long double mult = 1.0L;
  long double rounder;
  int i_exp = 0;
  char exp_buf[32];
  char *mant_buf = 0;
  int i;
  int d;
  unsigned char const *digit_conv = _reuse_dig2charlower;

  if (format == 'E' || format == 'F' || format == 'G')
    flags |= REUSE_FORMAT_UP;
  if ((flags & REUSE_FORMAT_UP)) digit_conv = _reuse_dig2charupper;

  if (reuse_is_nan_ld(pv)) {
    PUT_TO_STRING(buf, size, out_size, LETTER('n'));
    PUT_TO_STRING(buf, size, out_size, LETTER('a'));
    PUT_TO_STRING(buf, size, out_size, LETTER('n'));
    return reuse_strnput0(buf, size, out_size);
  }
  if (reuse_get_sign_bit_ld(pv)) {
    PUT_TO_STRING(buf, size, out_size, '-');
    vv = -vv;
  } else if ((flags & REUSE_FORMAT_PLUS)) {
    PUT_TO_STRING(buf, size, out_size, '+');
  } else if ((flags & REUSE_FORMAT_SPC)) {
    PUT_TO_STRING(buf, size, out_size, ' ');
  }
  if (reuse_is_infinity_ld(&vv)) {
    PUT_TO_STRING(buf, size, out_size, LETTER('i'));
    PUT_TO_STRING(buf, size, out_size, LETTER('n'));
    PUT_TO_STRING(buf, size, out_size, LETTER('f'));
    return reuse_strnput0(buf, size, out_size);
  }

  if (format == 'e' || format == 'E') {
    if (vv == 0.0L) {
      if (prec < 0) prec = 6;
      PUT_TO_STRING(buf, size, out_size, '0');
      if (prec > 0) {
        PUT_TO_STRING(buf, size, out_size, '.');
        for (; prec; prec--) {
          PUT_TO_STRING(buf, size, out_size, '0');
        }
      } else if ((flags & REUSE_FORMAT_ALT)) {
        PUT_TO_STRING(buf, size, out_size, '.');
      }
      PUT_TO_STRING(buf, size, out_size, LETTER('e'));
      PUT_TO_STRING(buf, size, out_size, '+');
      PUT_TO_STRING(buf, size, out_size, '0');
      PUT_TO_STRING(buf, size, out_size, '0');
      return reuse_strnput0(buf, size, out_size);
    }

    if (prec < 0) prec = 6;
    if (vv < 1.0L) {
      while (vv * mult < 1.0L) {
        mult *= 10.0L;
        i_exp--;
      }
      v2 = vv * mult;
    } else {
      while (vv / mult >= 10.0L) {
        mult *= 10.0L;
        i_exp++;
      }
      v2 = vv / mult;
    }
    rounder = 5.0L * get10pow(- prec - 1);
    if (v2 + rounder >= 10.0L) {
      mult *= 10.0L;
      i_exp++;
      v2 = vv / mult;
      v2 += rounder;
    } else {
      v2 += rounder;
    }

    d = (int) (v2);
    PUT_TO_STRING(buf, size, out_size, digit_conv[d]);
    if (prec > 0) {
      PUT_TO_STRING(buf, size, out_size, '.');
      v2 = (v2 - d) * 10.0;
      for (i = 0; i < prec; i++) {
        d = (int) (v2);
        PUT_TO_STRING(buf, size, out_size, digit_conv[d]);
        v2 = (v2 - d) * 10.0;
      }
    } else if ((flags && REUSE_FORMAT_ALT)) {
      PUT_TO_STRING(buf, size, out_size, '.');
    }
    PUT_TO_STRING(buf, size, out_size, LETTER('e'));
    sprintf(exp_buf, "%+03d", i_exp);
    return reuse_strncatx(buf, size, out_size, exp_buf);
  } else if (format == 'f' || format == 'F') {
    if (vv == 0.0L) {
      if (prec < 0) prec = 6;
      PUT_TO_STRING(buf, size, out_size, '0');
      if (prec > 0) {
        PUT_TO_STRING(buf, size, out_size, '.');
        for (; prec; prec--) {
          PUT_TO_STRING(buf, size, out_size, '0');
        }
      } else if ((flags & REUSE_FORMAT_ALT)) {
        PUT_TO_STRING(buf, size, out_size, '.');
      }
      return reuse_strnput0(buf, size, out_size);
    }

    if (prec < 0) prec = 6;
    vv += 5.0L * get10pow(- prec - 1);
    if (vv >= 1) {
      mult = 1.0L;
      i_exp = 0;
      while (vv / mult >= 10.0L) {
        mult *= 10.0L;
        i_exp++;
      }
      while (i_exp >= 0) {
        d = (int) (vv / mult);
        PUT_TO_STRING(buf, size, out_size, digit_conv[d]);
        vv = vv - d * mult;
        mult /= 10.0L;
        i_exp--;
      }
    } else {
      PUT_TO_STRING(buf, size, out_size, '0');
    }
    if (prec > 0) {
      PUT_TO_STRING(buf, size, out_size, '.');
      vv *= 10.0L;
      for (i = 0; i < prec; i++) {
        d = (int) vv;
        PUT_TO_STRING(buf, size, out_size, digit_conv[d]);
        vv = (vv - d) * 10.0;
      }
    } else if ((flags & REUSE_FORMAT_ALT)) {
      PUT_TO_STRING(buf, size, out_size, '.');
    }
    return reuse_strnput0(buf, size, out_size);
  } else {
    // format is 'G' or 'g'
    if (vv == 0.0L) {
      if (prec < 0) prec = 6;
      PUT_TO_STRING(buf, size, out_size, '0');
      if ((flags & REUSE_FORMAT_ALT)) {
        PUT_TO_STRING(buf, size, out_size, '.');
        for (i = 1; i < prec; i++) {
          PUT_TO_STRING(buf, size, out_size, '0');
        }
      }
      return reuse_strnput0(buf, size, out_size);
    }

    if (prec < 0) prec = 6;
    if (!prec) prec = 1;
    if (vv < 1.0L) {
      while (vv * mult < 1.0L) {
        mult *= 10.0L;
        i_exp--;
      }
    } else {
      while (vv / mult >= 10.0L) {
        mult *= 10.0L;
        i_exp++;
      }
    }

    v2 = vv + 5.0L * get10pow(i_exp - prec);
    mult = 1.0L;
    i_exp = 0;
    if (v2 < 1.0L) {
      while (v2 * mult < 1.0L) {
        mult *= 10.0L;
        i_exp--;
      }
    } else {
      while (v2 / mult >= 10.0L) {
        mult *= 10.0L;
        i_exp++;
      }
    }

    if (i_exp < -4 || i_exp >= prec) {
      prec--;
      vv = v2;
      if (vv < 1.0L) {
        while (vv * mult < 1.0L) {
          mult *= 10.0L;
          i_exp--;
        }
        v2 = vv * mult;
      } else {
        while (vv / mult >= 10.0L) {
          mult *= 10.0L;
          i_exp++;
        }
        v2 = vv / mult;
      }
      d = (int) (v2);
      PUT_TO_STRING(buf, size, out_size, digit_conv[d]);
      if (prec > 0) {
        v2 = (v2 - d) * 10.0;
        mant_buf = (char*) do_alloca(prec + 16);
        i = 0;
        for (;prec; prec--) {
          d = (int) (v2);
          mant_buf[i++] = digit_conv[d];
          v2 = (v2 - d) * 10.0;
        }
        mant_buf[i] = 0;
        if (!(flags & REUSE_FORMAT_ALT)) {
          while (i > 0 && mant_buf[i - 1] == '0') i--;
          mant_buf[i] = 0;
        }
        if (i > 0) {
          PUT_TO_STRING(buf, size, out_size, '.');
          for (i = 0; mant_buf[i]; i++) {
            PUT_TO_STRING(buf, size, out_size, mant_buf[i]);
          }
        }
      } else if ((flags & REUSE_FORMAT_ALT)) {
        PUT_TO_STRING(buf, size, out_size, '.');
      }
      PUT_TO_STRING(buf, size, out_size, LETTER('e'));
      sprintf(exp_buf, "%+03d", i_exp);
      return reuse_strncatx(buf, size, out_size, exp_buf);
    }

    vv = v2;
    if (vv >= 1.0) {
      while (i_exp >= 0) {
        d = (int) (vv / mult);
        PUT_TO_STRING(buf, size, out_size, digit_conv[d]);
        vv = vv - d * mult;
        mult /= 10.0L;
        i_exp--;
        prec--;
      }
      if (prec > 0) {
        mant_buf = (char*) do_alloca(prec + 16);
        i = 0;
        vv *= 10.0L;
        for (; prec; prec--) {
          d = (int) vv;
          mant_buf[i++] = '0' + d;
          vv = (vv - d) * 10.0;
        }
        mant_buf[i] = 0;
        if (!(flags & REUSE_FORMAT_ALT)) {
          while (i > 0 && mant_buf[i - 1] == '0') i--;
          mant_buf[i] = 0;
        }
        if (i > 0) {
          PUT_TO_STRING(buf, size, out_size, '.');
          return reuse_strncatx(buf, size, out_size, mant_buf);
        }
      } else if ((flags & REUSE_FORMAT_ALT)) {
        PUT_TO_STRING(buf, size, out_size, '.');
      }
    } else {
      // mantissa is < 1
      PUT_TO_STRING(buf, size, out_size, '0');
      PUT_TO_STRING(buf, size, out_size, '.');
      vv *= 10.0L;
      while (vv < 1) {
        PUT_TO_STRING(buf, size, out_size, '0');
        vv *= 10.0L;
      }
      mant_buf = (char*) do_alloca(prec + 16);
      i = 0;
      for (; prec; prec--) {
        d = (int) vv;
        mant_buf[i++] = '0' + d;
        vv = (vv - d) * 10.0;
      }
      mant_buf[i] = 0;
      if (!(flags & REUSE_FORMAT_ALT)) {
        while (i > 0 && mant_buf[i - 1] == '0') i--;
        mant_buf[i] = 0;
      }
      return reuse_strncatx(buf, size, out_size, mant_buf);
    }
  }
  return reuse_strnput0(buf, size, out_size);
#endif
#endif /* R_LONG_DOUBLE_IS_DOUBLE */
}

#if REUSE_COMPILE_CVSID - 0 != 0
static char const rcsid [] =
"$Id: writedld.c 354 2004-07-17 06:28:14Z cher $";
#endif /* REUSE_COMPILE_CVSID */

/*
 * Local variables:
 *  compile-command: "make"
 *  c-font-lock-extra-types: ("\\sw+_t" "FILE")
 * End:
 */
